/*
 * Query.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.hog;

/**
 * A small class for storing query specific data such as span and depth. Contains same
 * field values as analyse methods of <code>MoCoMan</code> which there are:
 * a session ID, an annotation ID, a span and a depth <br>
 * To test whether an incoming query has been processed already, <code>MoCoMan</code>
 * calls {@link MoCoMan#queryProcessed} for all <code>Query</code> objects in its QueryQueue.
 *
 * @author <a href="mailto:robba@dfki.de">Robert Barbey</a>
 * @version 2003-10-13
 */
public class Query
{
	/**
	 * identification for a session this query was processed in */
	private String querySessionID;

	/**
	 * identification for an annotation which was processed in this query */
	private String queryAnnotationID;

	/**
	 * annotation's start of span */
	private int queryStartSpan;

	/**
	 * annotation's end of spand */
	private int queryEndSpan;

	/**
	 * depth of this query */
	private int queryDepth;

	/**
	 * Initialises this <code>Query</code> object with given parameters equal to those of
	 * <code>analyse()</code> methods <code>MoCoMan</code> provides.
	 * @param sessionID identification of current session
	 * @param annotationID identification of processed annotation
	 * @param startSpan first character position in given annotation
	 * @param endSpan last character position in given annotation
	 * @param depth depth of current analysis
	 */
	public Query(String sessionID, String annotationID, int startSpan, int endSpan, int depth)
	{
		querySessionID = sessionID;
		queryAnnotationID = annotationID;
		queryStartSpan = startSpan;
		queryEndSpan = endSpan;
		queryDepth = depth;
	}

	/**
	 * Tests equality of given query data and this <code>Query</code> object.
	 * @param sessionID
	 * @param annotationID
	 * @param startSpan
	 * @param endSpan
	 * @param depth
	 * @return <tt>-1<tt> in case there's no congruency between this <code>Query</code>
	 *   object and given query data; otherwise, the depth of this <code>Query</code>.
	 */
	int processed(String sessionID, String annotationID, int startSpan, int endSpan, int depth)
	{
		if (querySessionID == sessionID
			  && queryAnnotationID == annotationID
			  && queryStartSpan == startSpan
			  && queryEndSpan == endSpan)
		{
			return queryDepth;
		}

		return -1;
	}

}
