/*
 * Session.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.hog;

import java.util.*;

import de.dfki.lt.hog.exceptions.ShutdownException;
import de.dfki.lt.hog.util.Utility;

/**
 * In a session a
 * @author <a href="mailto:robba@dfki.de">Robert Barbey</a>
 * @version 2003-11-20
 */
public class Session {
	/**
	 * The ID for this object as <code>int</code> value.
	 */
	private String sessionId;

	private Hashtable annotationCollections;

	private String configFileName;

	/* OLD!
	 *//**
	 * Configuration of this <code>Session</code> */
	/*
	 private Properties properties;
	 */

	/**
	 * Configuration of this <code>Session</code> */
	private Map properties;

	/**
	 * A <code>Vector</code> of <code>Module</code>s
	 * containing all needed modules, sorted by depth. This is needed
	 * for <code>analyse</code> methods of <code>MoCoMan</code>.
	 */
	private Vector modules;

	/*OLD!
	*//**
	 * Creates a new <code>Session</code> with an own ID and a
	 * <code>Hashtable</code> to store all the <code>Document</code> objects
	 * this session is going to handle.
	 *
	 * @param props properties for this <code>Session</code>
	 * @param mods modules sorted by depth
	 *//*
	public Session(Properties props, Vector mods) {
		init(props, mods, createSessionId());
	}

	public Session(Properties props, Vector mods, String sessionId) {
		init(props, mods, sessionId);
	}
*/
	
	/**
	 * Creates a new <code>Session</code> with an own ID and a
	 * <code>Hashtable</code> to store all the <code>Document</code> objects
	 * this session is going to handle.
	 *
	 * @param sessionProperties configuration for this <code>Session</code>
	 *  in a hashtable
	 * @param mods modules sorted by depth
	 */
	public Session(Map<String, String> sessionProperties, Vector mods) {
		init(sessionProperties, mods, createSessionId());
	}

	public Session(Map<String, String> sessionProperties, Vector mods,
			String sessionId) {
		init(sessionProperties, mods, sessionId);
	}

	
	private void init(Map<String, String> sessionProperties, Vector mods,
			String sessionId) {
		this.sessionId = sessionId;
		annotationCollections = new Hashtable();
		properties = sessionProperties;
		modules = mods;
		configFileName = "";

	}

	/*OLD°
	private void init(Properties props, Vector mods, String sessionId) {
		this.sessionId = sessionId;
		annotationCollections = new Hashtable();
		properties = props;
		modules = mods;
		configFileName = "";
	}*/

	/**
	 * Creates an ID for this <code>Session</code> object in order to ease up
	 * communictaion with the application. The ID is also associated with a node in
	 * the XML database.
	 *
	 * @return an ID for this <code>Session</code> object
	 */
	private String createSessionId() {
		String id = "session" + MoCoMan.getInstance().getUniqueID();
		//	    + String.valueOf(MoCoMan.getSessions().size() + 1);
		return id;
	}

	/**
	 *
	 * @return the ID for this <code>Session</code> object
	 */
	public String getSessionId() {
		return sessionId;
	}

	public String createAnnotationCollection() {
		AnnotationCollection annColl = new AnnotationCollection(getSessionId());
		String annCollId = annColl.getAnnotationCollectionId();

		annotationCollections.put(annCollId, annColl);

		return annCollId;
	}

	/*public String createAnnotation(String annotation, String language)
	 {
	 Annotation anAnnotation = new Annotation(annotation, language);
	 String annotationId = anAnnotation.getAnnotationId();

	 annotationCollections.put(annotationId, anAnnotation);

	 return annotationId;
	 }*/

	public Hashtable getAnnotationCollections() {
		return this.annotationCollections;
	}

	public Annotation getAnnotation(String annotationId) {
		return (Annotation) annotationCollections.get(annotationId);
	}

	/**
	 * Closes the session and cleares the <code>documents</code> Hashtable
	 * and the <code>modules</code> Vector.
	 */
	public void closeSession() {
		annotationCollections.clear();

		while (!getModuleVector().isEmpty()) {
			Module m = (Module) getModuleVector().firstElement();
			try {
				m.shutdown();
			} catch (ShutdownException se) {
				MoCoMan.log.error(Utility.exception2String(se));
			}

			getModuleVector().remove(m);
		}

	}

	public Vector getModuleVector() {
		return modules;
	}

	public void setConfigFileName(String configFileName) {
		this.configFileName = configFileName;
	}

	public String getConfigFileName() {
		return this.configFileName;
	}
}
