/*
 * XmlRpcClientInterface.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.hog;

/**
 * This interface provides three basic operation for XML-RPC communication as a client.
 * <code>XmlRpcModule</code> implements this interface as it needs to communicate
 * with a corresponding remote adaptor.
 *
 * @author <a href="mailto:robba@dfki.de">Robert Barbey</a>
 * @version 2003-09-23
 */
public interface XmlRpcClientInterface {

    /**
     * Starts a remote adaptor to communicate with. This method must be implemented by
     * each class derived from <code>XmlRpcModule</code> and could start an
     * adaptor via a shell or Python script, for example.
     *
     * @param aModuleName name of the module to start
     */
    public void start(String aModuleName);

    /**
     * Sends parameters to a remote module by an <code>XmlRpcClient</code>.
     * Calls therefore the <code>execute</code> method of <code>XmlRpcClient</code>.
     *
     * @param aMethodName name of a method which process <code>params</code> remotely
     * @param params paramters for remote method
     */
    public String process(String aMethodName, java.util.Vector params);

    /**
     * Shutsdown a remote module.
     *
     * @param aModuleName name of the module to shut down
     */
    public void shutdown(String aModuleName);
}
