/*
 * XmlRpcServerInterface.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.hog;

import org.apache.xmlrpc.*;

/**
 * Provides basic operations for XML-RPC communication as a server.
 * These operations are basically methods from <code>WebServer</code>
 * class of the XML-RPC package by apache. To have a class responding
 * to remote calls a so called handler must be set. This is done by
 * the <code>addHandler</code> method. Other methods return objects
 * like the <code>WebServer</code> itself.
 *
 * @author <a href="mailto:robba@dfki.de">Robert Barbey</a>
 * @version 2003-12-05
 */
public interface XmlRpcServerInterface {

    /**
     * Adds a handler object to a <code>WebServer</code>.
     * This object then processes incoming calls associated with
     * the given handler name.
     *
     * @param aHandlerName a String to which an handling object is mapped
     * @param aHandler remote calls handling object
     */
    public void addHandler(String aHandlerName, Object aHandler);

    /**
     * Returns the <code>WebServer</code> of an implementing class.
     *
     * @return a <code>WebServer</code> object
     */
    public WebServer getServer();

    /**
     * Returns the port on which the server is listening for remote
     * calls.
     *
     * @return port as <code>int</code> value
     */
    public int getServerPort();

    /**
     * Returns the object associated with given handler name.
     *
     * @param aHandlerName a name of a handling class
     * @return a handler object
     */
    public Object getHandler(String aHandlerName);

}
