/*
 * AnnotationEncapsulator.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.hog.container;

import de.dfki.lt.hog.Metadata;
import de.dfki.lt.hog.MoCoMan;

/**
 * Encapsulates XML annotation (not necessarily HoG-generated) and relevant
 * metadata such as session ID or annotation collection ID.
 * Main purpose is transport of dynamic data for/within the SDL module.
 *
 * @author <a href=mailto:ulrich.schaefer@dfki.de>Ulrich Schaefer</a>
 * @version $Revision: 1.2 $
 * $Id: AnnotationEncapsulator.java,v 1.2 2005/04/26 17:37:25 oeze Exp $
 */
public class AnnotationEncapsulator  {

    private String sessionID;
    private String annotationCollectionID;
    private String annotationID;
    private String language;
    private MoCoMan mocoman;
    private Object annotation;

    /** Create new AnnotationEncapsulator object.
        Use subsequent setter methods to fill attributes.
     */
    public AnnotationEncapsulator() {
        //relax
    }

    /** Create new AnnotationEncapsulator object from Metadata.
     */
    public AnnotationEncapsulator(Metadata metadata) {
        setSessionID(metadata.getSessionID());
        setAnnotationCollectionID(metadata.getAcID());
        setAnnotationID(metadata.getComponent());
    }

    /** Create new AnnotationEncapsulator object from Metadata and
        an annotation object.
    */
    public AnnotationEncapsulator(Metadata metadata, Object annotation) {
        this(metadata);
        setAnnotation(annotation);
    }

    /** Create new AnnotationEncapsulator object from
        an annotation object.
    */
    public AnnotationEncapsulator(Object annotation) {
        setAnnotation(annotation);
    }

    /**
     * Creates a new AnnotationEncapsulator, copies all
     * attributes exception the annotation object from
     * the original AnnotationEncapsulator. The new
     * AnnotationEncapsulator contains the annotation
     * objects specified as parameter.
     * @param newAnnotation the new annotation object
     * @return a new AnnotationEncapsulator
     */
    public AnnotationEncapsulator clone(Object newAnnotation) {
        AnnotationEncapsulator ae = new AnnotationEncapsulator(newAnnotation);
        ae.setSessionID(this.getSessionID());
        ae.setAnnotationCollectionID(this.getAnnotationCollectionID());
        ae.setAnnotationID(this.getAnnotationID());
        ae.setMoCoMan(this.getMoCoMan());
        ae.setLanguage(this.getLanguage());
        return ae;
    }

    /**
     * Return sessionID attribute.
     * @return String sessionID
     */
    public String getSessionID() {
        return this.sessionID;
    }

    /**
     * Set sessionID attribute.
     * @param sid the new sessionID
     */
    public void setSessionID(String sid) {
        this.sessionID = sid;
    }

    /**
     * Return annotationCollectionID attribute.
     * @return String annotationCollectionID
     */
    public String getAnnotationCollectionID() {
        return this.annotationCollectionID;
    }

    /**
     * Set annotationCollectionID attribute.
     */
    public void setAnnotationCollectionID(String acid) {
        this.annotationCollectionID = acid;
    }

    /**
     * Return annotationID attribute.
     * @return String annotationID
     */
    public String getAnnotationID() {
        return this.annotationID;
    }

    /**
     * Set annotationID attribute.
     */
    public void setAnnotationID(String acid) {
        this.annotationID = acid;
    }

    /**
     * Return annotation obejct.
     * @return Object sessionID
     */
    public Object getAnnotation() {
        return this.annotation;
    }

    /**
     * Set annotation object.
     */
    public void setAnnotation(Object o) {
        this.annotation = o;
    }

    /**
     * Return MoCoMan obejct.
     * @return MoCoMan MoCoMan
     */
    public MoCoMan getMoCoMan() {
        return this.mocoman;
    }

    /**
     * Set MoCoMan object.
     */
    public void setMoCoMan(MoCoMan mcm) {
        this.mocoman = mcm;
    }

    /**
     * Return language attribute.
     * @return String language
     */
    public String getLanguage() {
        return this.language;
    }

    /**
     * Set language attribute.
     * @param lang the new language
     */
    public void setLanguage(String lang) {
        this.language = lang;
    }


}
