/*
 * JToken.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */


package de.dfki.lt.hog.container;

import java.io.*;
import java.util.*;
import javax.xml.parsers.*;
import org.apache.log4j.Logger;
import org.w3c.dom.*;
import org.xml.sax.*;
import de.dfki.lt.hog.util.Utility;

/**
 * Represents a single token as segmented by Jtok. Each JToken consists of
 * the token itself as a <code>String</code>, its length as <code>int</code>,
 * and its offset as <code>int</code>.
 *
 * This class also provides the static method
 * {@link #parseJTokAnnotation(String)}, which receives a JTok annotation and
 * returns an <code>ArrayList</code> of <code>JToken</code> instances.
 *
 * @author <a href=mailto:oeze@dfki.de>Oezguer Demir</a>
 * @version $Revision: 1.2 $
 * $Id: JToken.java,v 1.2 2005/04/26 17:37:27 oeze Exp $
 */
public class JToken {

  /** A logger for <code>Module</code>s */
  static protected Logger log = Logger.getLogger(JToken.class);

  private String type;
  private String token;
  private int length;
  private int offset;

  public JToken() {
    this.token = null;
    this.length = -1;
    this.offset = -1;
  } // constructor()

  public JToken(String token, int length, int offset) {
    this.token = token;
    this.length = length;
    this.offset = offset;
  } // constructor(string, int, int)

  public void setType(String type) {
    this.type = type;
  }

  public void setToken(String token) {
      //this.token = Utility.cleanXMLString(token);
      this.token=token;
  }

  public void setLength(int lenght) {
    this.length = lenght;
  }

  public void setOffset(int offset) {
    this.offset = offset;
  }

  public String getType() {
    return this.type;
  }

  public String getToken() {
    return this.token;
  }

  public int getLength() {
    return this.length;
  }

  public int getOffset() {
    return this.offset;
  }

  /**
   * Parses a JTok XML and returns an <code>ArrayList</code> of
   * <code>JToken</code> instances (each instance containing
   * the token string, the character offset, and the token length <br>
   * 2004-04-21 US
   * mod. 2004-05-14 oe
   */
  public static ArrayList parseJTokAnnotation(String sJTok) {
    try {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      //log.debug("parseJTokAnnotation: sJTok: " + sJTok);
      Document document =
          builder.parse(new InputSource(new StringReader(sJTok)));
      int numTokens = document.getElementsByTagName("Token").getLength();
      ArrayList jtokens = new ArrayList(numTokens);
      NodeList sentences = document.getElementsByTagName("tu");
      int ssl = sentences.getLength();
      log.debug("parseJTokAnnotation: " + numTokens + " tokens.");
      int i = 0;
      for (int s = 0; s < ssl; s++) {
        Node si = sentences.item(s);
        if (si.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE) {
          NodeList sentence = si.getChildNodes();
          int sl = sentence.getLength();
          //token[i++] = "\n";
          for (int t = 0; t < sl; t++) {
            Node ti = sentence.item(t);
            if (ti.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE) {
              if (ti.getNodeName().equals("Token")) {
                JToken jtoken = new JToken();
                jtoken.setType( ( (Element) ti).getAttribute(
                    "type"));
                jtoken.setToken( ( (Element) ti).getAttribute(
                    "string"));
                jtoken.setOffset(Integer.parseInt( ( (Element) ti).getAttribute(
                    "offset")));
                jtoken.setLength(Integer.parseInt( ( (Element) ti).getAttribute(
                    "length")));
                jtokens.add(jtoken);
                i++;
              }
            }
          }
        }
      }
      return jtokens;
    }
    catch (SAXException sxe) {
      log.error(Utility.exception2String(sxe));
      return new ArrayList();
    }
    catch (ParserConfigurationException pce) {
      log.error(Utility.exception2String(pce));
      return new ArrayList();
    }
    catch (IOException ioe) {
      log.error(Utility.exception2String(ioe));
      return new ArrayList();
    }
  }

  public String toString() {
    StringBuffer out = new StringBuffer();
    out.append("  String: " + this.token + "\n");
    out.append("  Length: " + this.length + "\n");
    out.append("  Offset: " + this.offset);
    return out.toString();
  }

} // class
