/*
 * TnToken.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.hog.container;

import java.util.*;
import de.dfki.lt.hog.util.Utility;

/**
 * Represents a single token as pos-tagged by TnT. This class is derived
 * by {@link JToken} and extends it by an array which contains
 * all possible pos-tags and their probabilities.
 *
 * @author <a href=mailto:oeze@dfki.de>Oezguer Demir</a>
 * @version $Revision: 1.3 $
 * $Id: TnToken.java,v 1.3 2007/03/13 14:15:07 uschaefer Exp $
 */
public class TnToken
    extends JToken {

  private ArrayList tags;
  private ArrayList probs;

  public TnToken() {
    super();
    this.tags = new ArrayList();
    this.probs = new ArrayList();
  }

  public TnToken(JToken jtok) {
    super(jtok.getToken(), jtok.getLength(), jtok.getOffset());
    this.setType(jtok.getType());
    this.tags = new ArrayList();
    this.probs = new ArrayList();
  }

  public void addTag(String pos, String prob) {
    this.tags.add(pos);
    this.probs.add(prob);
  }

  public String getPos(int i) {
    return (String)this.tags.get(i);
  }

  public String getProb(int i) {
    return (String)this.probs.get(i);
  }

  public String toString() {
    StringBuffer out = new StringBuffer();
    out.append("  String: " + this.getToken() + "\n");
    out.append("  Length: " + this.getLength() + "\n");
    out.append("  Offset: " + this.getOffset() + "\n");
    out.append("  Pos (p): ");
    for (int i = 0; (i < this.tags.size() && i < this.probs.size()); i++) {
      out.append(this.getPos(i) + " (" + this.getProb(i) + ") ");
    }
    return out.toString();
  }

  public String toXML() {
    StringBuffer out = new StringBuffer(
        "<w str=\"" + Utility.cleanXMLString(this.getToken()) +
        "\" cstart=\"" + this.getOffset() +
        "\" cend=\"" + (this.getOffset() + this.getLength() - 1) + "\">\n");
    for (int i = 0; (i < this.tags.size() && i < this.probs.size()); i++) {
      out.append("  <p pos=\"" + this.getPos(i) +
                 "\" p=\"" + this.getProb(i) + "\"/>\n");
    }
    out.append("</w>\n");
    return out.toString();
  }

  public String toPiXML(int id) {
  	boolean isInterpunct = Utility.isInterpunct(this.getToken());
  	StringBuffer out = new StringBuffer(
  	        "<w id=\"TNT" + id + "\" cstart=\"" + this.getOffset() +
  	        "\" cend=\"" + (this.getOffset() + this.getLength() - 1) + "\"" +
			( isInterpunct ? " constant=\"yes\"" : "" ) + ">\n" +
  		    "  <surface>" + Utility.cleanXMLString(this.getToken()) + "</surface>\n");
  		if (isInterpunct)
  			out.append("  <pos tag=\"" + Utility.cleanXMLString(this.getToken()) +
  					   "\" prio=\"1.0\"/>\n");
  		else
  			for (int i = 0; (i < this.tags.size() && i < this.probs.size()); i++) {
  				out.append("  <pos tag=\"" + this.getPos(i) +
  						   "\" prio=\"" + this.getProb(i) + "\"/>\n");
  				// special handling for numbers: insert HPSG lextype
				// removed on 2007-03-13 (uschaefer): CD/CARD should be mapped to
                                // $generic_number as part of HPSG grammar PoS mapping configuration, 
                                // e.g. line "CD $generic_number" in english.set/general.set
//   				if (this.getPos(i).equals("CD") || this.getPos(i).equals("CARD")) {
//   					out.append("  <typeinfo id=\"TYI" + id + "\" baseform=\"no\">\n");
//   					out.append("    <stem>$generic_number</stem>\n");
//   					out.append("  </typeinfo>\n");
//   					break;
//   				}
  	        }
  	    out.append("</w>\n");
  	    return out.toString();

  }

} // class
