/*
 * TreeTaggerToken.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.hog.container;

import java.util.*;
import de.dfki.lt.hog.util.Utility;

/**
 * Represents a single token as pos-tagged by TreeTagger. This class is derived
 * by {@link JToken} and extends it by a String tag and a String lemma.
 *
 * @author <a href=mailto:oeze@dfki.de>Oezguer Demir</a>
 * @version $Revision: 1.1 $
 * $Id: TreeTaggerToken.java,v 1.1 2005/09/27 17:10:16 oeze Exp $
 */
public class TreeTaggerToken extends JToken {

  private String pos;
  private String lemma;

  public TreeTaggerToken() {
    super();
    this.pos = "";
    this.lemma = "";
  }

  public TreeTaggerToken(JToken jtok) {
    super(jtok.getToken(), jtok.getLength(), jtok.getOffset());
    this.setType(jtok.getType());
    this.pos = "";
    this.lemma = "";
  }

  public void setPos(String pos) {
    this.pos = pos;
  }

  public String getPos() {
    return this.pos;
  }

  public void setLemma(String lemma) {
    this.lemma = lemma;
  }

  public String getLemma() {
    return this.lemma;
  }

  public String toString() {
    StringBuffer out = new StringBuffer();
    out.append("  String: " + this.getToken() + "\n");
    out.append("  Pos: " + this.getPos() + "\n");
    out.append("  Lemma: " + this.getLemma() + "\n");
    out.append("  Length: " + this.getLength() + "\n");
    out.append("  Offset: " + this.getOffset() + "\n");
    return out.toString();
  }

  public String toXML() {
    StringBuffer out = new StringBuffer();
    out.append("<t pos=\"" + Utility.cleanXMLString(this.getPos()));
    if (!this.getLemma().equals("")) {
      out.append("\" lemma=\"" + Utility.cleanXMLString(this.getLemma()));
    }
    out.append("\" cstart=\"" + this.getOffset());
    out.append("\" cend=\"" + (this.getOffset() + this.getLength() - 1));
    out.append("\">" + Utility.cleanXMLString(this.getToken()));
    out.append("</t>\n");
    return out.toString();
  }
} // class
