/*
 * XMLDBAnnotationDatabase.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.hog.database;

import org.w3c.dom.*;
//import org.xmldb.api.base.Collection;
//import org.xmldb.api.base.ResourceIterator;
import java.io.*;
import org.apache.log4j.Logger;

/**
 * <p>The XML:DB-based database interface for the Deepthought annotation.
 * This class uses concepts like session and document (identified via
 * Strings and implemented as collections in the database)
 * that are used to structure the Deepthought annotation database.
 * In the Deepthought annotation database, a session comprises a collection of documents.
 * A document itself is a collection (!) of XML annotations that are
 * produced from different modules and can be accessed via annotation
 * layer names.</p>
 * For performance reasons, it is highly recommended to make use of
 * the addIndex() method for every collection that is added.
 *
 * @author <a href="mailto:ulrich.schaefer@dfki.de">Ulrich Schaefer</a>
 * @version 2003-09-18
 *
 */

public class XMLDBAnnotationDatabase extends AnnotationDatabase {

   static Logger logger = Logger.getLogger(XMLDBAnnotationDatabase.class);

	/** The method openDatabase() should be called after this empty constructor.
	 *
	 */
	public XMLDBAnnotationDatabase() {
		super();
	}

	/**
	 * @param databaseLocation an XML:DB URI like "xmldb:xindice://localhost:8080/db"
	 */
	public XMLDBAnnotationDatabase(String databaseLocation) {
				super(databaseLocation);
				this.openDatabase(databaseLocation);
	}

	/* (non-Javadoc)
	 * @see de.dfki.lt.hog.database.AnnotationDatabase#openDatabase(java.lang.String)
	 */
	public boolean openDatabase(String connectString) {
/*		try {
			Collection collection = XMLDBAccess.getCollection( connectString );
			if ( collection == null ) {
				System.err.println("ERROR : Collection " + connectString + " not found!");
				setDatabaseLocation(null);
				return false;
			} else return true;
		} catch (Exception e) {
			logger.error( e.toString());
			return false;
		}
*/
	    return true;
	}

	/* (non-Javadoc)
	 * @see de.dfki.lt.hog.database.AnnotationDatabase#closeDatabase()
	 */
	public void closeDatabase() {
		// nothing to do for Xindice ...

	}

	/* (non-Javadoc)
	 * @see de.dfki.lt.hog.database.AnnotationDatabase#addSession(java.lang.String, java.lang.String)
	 */
	public boolean addSession(String sessionID, String metaInfo) {
/*		try {
			String dbLocation = getDatabaseLocation();
			XMLDBAccess.addCollection(dbLocation, sessionID);
			if (metaInfo!=null) {
				if (!dbLocation.endsWith("/")) dbLocation+="/";
				XMLDBAccess.addDocumentFromString(dbLocation + sessionID, metaInfo, "metadata");
			}
			return true;
		} catch (Exception e) {
			logger.error( e.toString());
			return false;
		}
*/
	    return true;
	}

	/* (non-Javadoc)
	 * @see de.dfki.lt.hog.database.AnnotationDatabase#addSession(java.lang.String)
	 */
	public boolean addSession(String sessionID) {
/*		try {
			String dbLocation = getDatabaseLocation();
			XMLDBAccess.addCollection(dbLocation, sessionID);
			return true;
		} catch (Exception e) {
			logger.error( e.toString());
			return false;
		}
*/
	    return true;
	}

	/* (non-Javadoc)
	 * @see de.dfki.lt.hog.database.AnnotationDatabase#addSession(java.lang.String, org.w3c.dom.Node)
	 */
	public boolean addSession(String sessionID, Node metaInfo) {
/*		try {
			String dbLocation = getDatabaseLocation();
			XMLDBAccess.addCollection(dbLocation, sessionID);
			if (metaInfo!=null) {
				if (!dbLocation.endsWith("/")) dbLocation+="/";
				XMLDBAccess.addDocumentFromDOM(dbLocation + sessionID, (Document) metaInfo, "metadata");
			}
			return true;
		} catch (Exception e) {
			logger.error( e.toString());
			return false;
		}
*/
	    return true;
	}

	/* (non-Javadoc)
	 * @see de.dfki.lt.hog.database.AnnotationDatabase#deleteSession(java.lang.String)
	 */
	public boolean deleteSession(String sessionID) {
/*		try {
			XMLDBAccess.deleteCollection(sessionID);
			return true;
			} catch (Exception e) {
				logger.error( e.toString());
				return false;
			}
*/
	    return true;
	}

	/** Adds a new, empty annotation collection for a document.
	 * @see de.dfki.lt.hog.database.AnnotationDatabase#addDocument(java.lang.String, java.lang.String)
	 */
	public boolean addDocument(String sessionID, String docName) {
/*		try {
			String dbLocation = getDatabaseLocation();
			if (!dbLocation.endsWith("/")) dbLocation+="/";
			XMLDBAccess.addCollection(dbLocation + sessionID, docName);
			return true;
		} catch (Exception e) {
			logger.error( e.toString());
			return false;
		}
*/
	    return true;
	}

	/** delete an annotation collection for a document
	 * @see de.dfki.lt.hog.database.AnnotationDatabase#deleteDocument(java.lang.String, java.lang.String)
	 */
	public boolean deleteDocument(String sessionID, String docName) {
/*		try {
			String dbLocation = getDatabaseLocation();
			if (!dbLocation.endsWith("/")) dbLocation+="/";
			XMLDBAccess.deleteCollection(dbLocation + sessionID);
			return true;
			} catch (Exception e) {
				logger.error( e.toString());
				return false;
			}
*/
	    return true;
	}

	/* (non-Javadoc)
	 * @see de.dfki.lt.hog.database.AnnotationDatabase#addAnnotation(java.lang.String, java.lang.String, java.lang.String, java.lang.String)
	 */
	public boolean addAnnotation(
		String sessionID,
		String docName,
		String layerName,
		String annotation) {
/*
		try {
			String dbLocation = getDatabaseLocation();
			if (!dbLocation.endsWith("/")) dbLocation+="/";
			XMLDBAccess.addDocumentFromString(dbLocation + sessionID + "/" + docName, annotation, layerName);
			return true;
		} catch (Exception e) {
			logger.error( e.toString());
			return false;
		}
*/ return true;
	}

	/* (non-Javadoc)
	 * @see de.dfki.lt.hog.database.AnnotationDatabase#addAnnotation(java.lang.String, java.lang.String, java.lang.String, org.w3c.dom.Node)
	 */
	public boolean addAnnotation(
		String sessionID,
		String docName,
		String layerName,
		Node domTree) {
/*
		try {
			String dbLocation = getDatabaseLocation();
			if (!dbLocation.endsWith("/")) dbLocation+="/";
			XMLDBAccess.addDocumentFromDOM(dbLocation + sessionID + "/" + docName, (Document) domTree, layerName);
			return true;
		} catch (Exception e) {
			logger.error( e.toString());
			return false;
		}
*/
	    return true;
	}

	/* (non-Javadoc)
	 * @see de.dfki.lt.hog.database.AnnotationDatabase#getAnnotation(java.lang.String, java.lang.String, java.lang.String)
	 */
	public String getAnnotationAsString(
		String sessionID,
		String docName,
		String layerName) {
/*
		try {
			String dbLocation = getDatabaseLocation();
			if (!dbLocation.endsWith("/")) dbLocation+="/";
			return XMLDBAccess.retrieveDocumentAsString(dbLocation + sessionID + "/" + docName, layerName);
		} catch (Exception e) {
			logger.error( e.toString());
			return null;
		}
*/
	    return null;
	}

	/* (non-Javadoc)
	 * @see de.dfki.lt.hog.database.AnnotationDatabase#getAnnotationAsDOM(java.lang.String, java.lang.String, java.lang.String)
	 */
	public Node getAnnotationAsDOM(
		String sessionID,
		String docName,
		String layerName) {
/*
		try {
			String dbLocation = getDatabaseLocation();
			if (!dbLocation.endsWith("/")) dbLocation+="/";
			return XMLDBAccess.retrieveDocumentAsDOM(dbLocation + sessionID + "/" + docName, layerName);
		} catch (Exception e) {
			logger.error( e.toString());
			return null;
		}
*/
	    return null;
	}

	/* (non-Javadoc)
		 * @see de.dfki.lt.hog.database.AnnotationDatabase#XUpdateQuery(java.lang.String, java.lang.String)
		 */
	public long XUpdateQuery (String sessionID, String query) {
/*	try {
			String dbLocation = getDatabaseLocation();
			if (!dbLocation.endsWith("/")) dbLocation+="/";
			return XMLDBAccess.update(dbLocation + sessionID, query);
		} catch (Exception e) {
			logger.error( e.toString());
			return -1;
		}
*/
	    return -1;
	}

	/* (non-Javadoc)
	 * @see de.dfki.lt.hog.database.AnnotationDatabase#XPathQuery(java.lang.String, java.lang.String, java.lang.String, java.lang.String)
	 */

/*
	public ResourceIterator XPathQuery(
		String sessionID,
		String docName,
		String query,
		String namespaces) {

		try {
			String dbLocation = getDatabaseLocation();
			if (!dbLocation.endsWith("/")) dbLocation+="/";
		    return XMLDBAccess.XPathQuery(dbLocation + sessionID + "/" + docName, query, namespaces);
		} catch (Exception e) {
			logger.error( e.toString());
			return null;
		}
	    return null;
	}
*/

	/* (non-Javadoc)
	 * @param sessionID the name of the session (or subcollection) to be indexed.
	 * @param indexID the name for the index (must be unique within a collection)
	 * @param indexPattern a pattern for the index with shape elem, elem@attr, *, *@attr, elem@* or *@*
	 * @return true iff an indexer could be generated successfully
	 * @see de.dfki.lt.hog.database.AnnotationDatabase#addIndex(java.lang.String, java.lang.String, java.lang.String)
	 */
	public boolean addIndex(
		String sessionID,
		String indexID,
		String indexPattern) {
/*		try {
			String dbLocation = getDatabaseLocation();
			if (!dbLocation.endsWith("/")) dbLocation+="/";
			return XMLDBAccess.addIndexer(dbLocation + sessionID, indexID, indexPattern, null, null, null);
		} catch (Exception e) {
			logger.error( e.toString());
			return false;
		}
*/
	    return true;
	}

	/**
	 * Auxiliary method that reads an XML file into a String object.
	 * @param file The file to be opened
	 * @return String XML String with default encoding assumed (!)
	 * @throws IOException
	 */
	public String readFileFromDisk(File file, String charset)
		throws IOException {
			FileInputStream insr = new FileInputStream(file);
			byte[] fileBuffer = new byte[(int) file.length()];
			insr.read(fileBuffer);
			insr.close();
			return new String(fileBuffer, charset);
	}
}
