/*
 * testdb.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.hog.database;

import java.io.File;
import java.io.IOException;
import org.xmldb.api.base.ResourceIterator;
import org.xmldb.api.base.XMLDBException;

/**
 * @author Ulrich Schaefer
 * sample file for Annotation Database access.
 * @version 2003-09-19
 */
public class testdb {

  public static void main(String[] args) throws IOException {
    XMLDBAnnotationDatabase db;
    //args[0] = Xindice URI, e.g. "xmldb:xindice://clearing:8080/db/"
    String uri = args[0];
    if (uri==null) uri = "xmldb:xindice://localhost:8080/db";
    if (!uri.endsWith("/")) uri += "/";
    db = new XMLDBAnnotationDatabase(uri + System.getProperty("user.name"));
    String sSession = "i4711"; // session identifier
    String sDocument = "doc1"; // document name
    String sAnnotationLayer = "ANT"; // annotation layer name
    String xml = db.readFileFromDisk(new File("/project/cl/deepthought/CoreArchitecture/src/build.xml"),
           "UTF-8");

    boolean result = db.addSession(sSession, "<metadata created='just now'/>" ) // add a new session
        && db.addDocument(sSession, sDocument) // add a document node
        && db.addAnnotation(sSession, sDocument, sAnnotationLayer, xml) // add XML annotation
        && db.addIndex(sSession, "targetindex", "target");

    if (!result) System.out.println("something went wrong.");
    String anno = db.getAnnotationAsString(sSession, sDocument, sAnnotationLayer);
    System.out.println("Retrieved annotation:");
    System.out.println(anno);
    System.out.println("XPath query result:");
    ResourceIterator results = db.XPathQuery(sSession, sDocument, "path", null);
    try {
          while (results.hasMoreResources()) {
          System.out.println(results.nextResource().getContent());
        }
    } catch (XMLDBException xdbe) {
        System.err.println(xdbe.toString());
    }
    System.out.println("End of test.");
  }
}
