/*
 * CorcyModule.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.hog.modules;

import java.io.*;
import java.util.*;
import org.apache.log4j.Logger;
import org.w3c.dom.*;
import de.dfki.lt.hog.Metadata;
import de.dfki.lt.hog.MoCoMan;
import de.dfki.lt.hog.Module;
import de.dfki.lt.hog.exceptions.InitializationException;
import de.dfki.lt.hog.util.Utility;

/**
 * Module definition for Corcy commandline interface
 *
 * @author <a href=mailto:oeze@dfki.de>Oezguer Demir</a>
 * @version $Revision: 1.3 $
 * $Id: CorcyModule.java,v 1.3 2005/04/26 17:37:35 oeze Exp $
 */

/* Changes:
 * 2004-07-22 : US added empty result handling (metadata)
 * 2004-09-09 : US input + outputencoding properly passed to process streams
*/

public class CorcyModule
    extends Module {

  static Logger log = Logger.getLogger(CorcyModule.class);
  private BufferedReader procOut;
  private BufferedWriter procIn;
  private Runtime runtime;
  private Process process;
  private String corcyOptions = "";
  private String corcyScript = "";
  private String outputEncoding;
  private String inputEncoding;
  private static final String NEWLINE = System.getProperty("line.separator");
  private static final String DATA_ROOT_ELEMENT = "clusters";

  public CorcyModule(Properties tntModuleProps) throws InitializationException {
    try {
      super.init(tntModuleProps);
    }
    catch (InitializationException ie) {
      log.error(Utility.exception2String(ie));
    }
    corcyScript = getConfig().getProperty("corcy.script");
    corcyOptions = getConfig().getProperty("corcy.options");
    outputEncoding = getConfig().getProperty("corcy.outputencoding", "ISO-8859-1");
    inputEncoding = getConfig().getProperty("corcy.inputencoding", "ISO-8859-1");

    // Init comprises a quick test. If the test is positive, TnT will be
    // shutdown again (no server mode)
    log.info("Initializing (testing) Corcy ...");
    runtime = Runtime.getRuntime();

    //try {
    //process = runtime.exec(tntScript + " " + tntOptions);
    //log.debug("Invoked: " + tntScript + " " + tntOptions);
    //String lineIn;
    //procOut = new BufferedReader(
    //    new InputStreamReader(process.getInputStream()));
    //procIn = new BufferedWriter(
    //    new OutputStreamWriter(process.getOutputStream()));
    // test
    //if (procOut.ready()) {
    //  log.info("TnT initialized (tested).");
    //  procIn.close();
    //  procOut.close();
    //}
    //else {
    //  throw new InitializationException("TnT did not start");
    //} // else
    //} // try
    //catch (IOException ioe) {
    //  log.fatal(Utility.exception2String(ioe));
    //}
  } // TnTModule()

  public void shutdown() {
    log.info("Shutting down Corcy.");
    // nothing to do here so far
  }

  /* (non-Javadoc)
   * @see de.dfki.lt.hog.Module#process(java.lang.String)
   */
  public String process(String processString) {
	String duration = "00:00,000";
	long t1 = System.currentTimeMillis();
    MoCoMan mocoman = getMoCoMan();
    String rawText =
        Utility.getComputedAnnotation(mocoman, processString, "rawtext");
    Metadata md = new Metadata(processString);
    String sessionID = md.getSessionID();
    String acID = md.getAcID();
    String root = getModuleRootElement();
    // process text
    String lineIn;
    String metadata;
    StringBuffer sbResult = new StringBuffer();
    boolean inResult = false;
    try {
      process = runtime.exec(corcyScript + " " + corcyOptions);
      procOut =
          new BufferedReader(new InputStreamReader(process.getInputStream(), outputEncoding));
      procIn =
          new BufferedWriter(new OutputStreamWriter(process.getOutputStream(), inputEncoding));
      // feed the text to TnT
      procIn.write(rawText + NEWLINE);
      procIn.flush();
      procIn.close();
      // generate header
      StringBuffer corcyResult = new StringBuffer();
      while ( (lineIn = procOut.readLine()) != null) {
        corcyResult.append(lineIn + NEWLINE);
      } // while
      duration = Utility.formatDuration(System.currentTimeMillis()-t1);
      // check whether result is empty
      String corcy = corcyResult.toString();
      boolean empty = false;
      if (corcy.trim().equals("")) empty = true;

      sbResult.append("<" + root + ">");
      sbResult.append(Metadata.generateXml(sessionID,
                                           acID, duration,
                                           getModuleName(),
                                           "OK", empty,
                                           getConfig()));
      sbResult.append("<" + DATA_ROOT_ELEMENT + ">" + NEWLINE);
      // paste in results from Corcy
      sbResult.append(corcy);
      sbResult.append("</" + DATA_ROOT_ELEMENT + ">" + NEWLINE);
      sbResult.append("</" + root + ">");

      String sResult = sbResult.toString();
      log.debug("result: " + sResult);
      log.info("Processing time of CorcyModule: " + duration);
      return sResult;
    }
    catch (IOException ioe) {
      log.fatal(Utility.exception2String(ioe));
      return "<" + root + ">" +
          Metadata.generateXml(sessionID, acID, duration,
                               getModuleName(),
                               ioe.getMessage(),
                               true,
                               getConfig()) +
          "</" + root + ">";
    }
  }

  /* (non-Javadoc)
       * @see de.dfki.lt.hog.Module#processDOMTree(org.w3c.dom.Node)
   */
  public Node processDOMTree(Node domTree) {
    // TODO Auto-generated method stub
    // use a simple SAX parser to extract text from domTree
    // return a DOM from transformer DOMResult (i.e., copy most of process() except final DOM output)
    return null;
  }
} // class
