/*
 * JTokModule.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.hog.modules;

import java.io.*;
import java.util.*;
import org.apache.log4j.Logger;
import org.w3c.dom.*;
import de.dfki.lt.hog.Metadata;
import de.dfki.lt.hog.MoCoMan;
import de.dfki.lt.hog.Module;
import de.dfki.lt.hog.exceptions.InitializationException;
import de.dfki.lt.hog.util.Utility;
import de.dfki.lt.tools.tokenizer.*;
import de.dfki.lt.tools.tokenizer.annotate.*;
import de.dfki.lt.tools.tokenizer.output.*;

/** Module for JTok tokenizer and sentence splitter.
 *
 * @author <a href="mailto:ulrich.schaefer@dfki.de">Ulrich Schaefer</a>
 * @version $Revision: 1.4 $
 * $Id: JTokModule.java,v 1.4 2006/06/09 09:07:36 uschaefer Exp $
  */

 /* Created 2003-11-17
  * 2004-05-18: uschaefer xml header with encoding (new config property jtok.outputencoding) 
  * 2006-06-09: uschaefer removed call to cleanXMLstring and outputencoding for JTok V1.01
*/

public class JTokModule extends Module {

    static Logger log = Logger.getLogger(JTokModule.class);
    private JTok jtok;

    public JTokModule(Properties jTokModuleProps) {
        String jTokConfigFile = null;
        try {
            super.init(jTokModuleProps);
        } catch (InitializationException ie) {
            log.fatal(Utility.exception2String(ie));
        }

        log.info("Initializing JTok...");
        Properties jTokProps = new Properties();
        jTokConfigFile = getConfig().getProperty("jtok.configfile");
        try {
            FileInputStream in = new FileInputStream(jTokConfigFile);
            jTokProps.load(in);
            in.close();
        } catch (IOException e) {
            log.fatal(
                      "Couldn't open file: "
                      + jTokConfigFile
                      + "\n"
                      + Utility.exception2String(e));
        }
        jtok = new JTok(new Properties(jTokProps));
        log.info("JTok initialized.");
    }

    public void shutdown() {
        // do sproutShutdown --  no corresponding method found in InterpreterTest.
        log.info("shutting down jTok.");
        jtok = null;
    }


    /* (non-Javadoc)
     * @see de.dfki.lt.hog.Module#process(java.lang.String)
     */
    public String process(String processString) {
    	String duration = "00:00,000";
    	long t1 = System.currentTimeMillis();
        MoCoMan mocoman = getMoCoMan();
        String rawText = Utility.getComputedAnnotation(mocoman, processString, "rawtext");
        Metadata md = new Metadata(processString);
        String sessionID = md.getSessionID();
        String acID = md.getAcID();
        log.info("input text: \"" + rawText + "\"");
        log.debug("Module config: " +  Utility.conf2Xml(this.getConfig()));
        AnnotatedString as = jtok.tokenize(rawText, getModuleLanguage());
        String root = getModuleRootElement();
        log.info("processing time of JTok: " + duration);
        log.info("creating JTok XML output");
        String sXML = XMLOutputter.createXMLString(as,
            jtok.getLanguageResource(getModuleLanguage()));
        duration = Utility.formatDuration(System.currentTimeMillis()-t1);
        log.info("processing time for JTok XML generation: " + duration);
        String msg = "OK";
        int pos0=sXML.indexOf("<Document>");
        if (pos0<0) {
            msg = "Error: XML Format not recognized";
            log.error("XML Format not recognized in " + sXML);
        }
        int pos1 = sXML.indexOf("</Document>");

        StringBuffer sbResult = new StringBuffer();
        sbResult.append("<?xml version='1.0'?>");
        sbResult.append("<" + root + ">");
        md = new Metadata(sessionID, acID, duration,
                          getModuleName(), msg, false, getConfig());
        sbResult.append(md.getXmlString());
        sbResult.append(sXML.substring(pos0+10,pos1-1));
        sbResult.append("</" + root + ">");
        sXML=sbResult.toString();
        //sXML = cleanXMLString(sbResult);
        log.debug(sXML);
        return sXML;
    }

    /* (non-Javadoc)
     * @see de.dfki.lt.hog.Module#processDOMTree(org.w3c.dom.Node)
     */
    public Node processDOMTree(Node domTree) {
        // TODO Auto-generated method stub
        // tokenize as in process(), but use XMLOutputter.createXMLDocument() for output
        // the bad thing: JTok then outputs JDOM, but (directly) no JAXP DOM
        // so, use the jdomDOMoutputter:
        // org.jdom.output.DOMOutputter.output(org.jdom.Document) -> w3c-DOM
        // i.e., copy the whole process() and modifiy the two lines that generate XML Strings
        return null;
    }

}
