/*
 * PicModule.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.hog.modules;

import java.util.Properties;
import java.util.StringTokenizer;
import org.apache.log4j.Logger;
import org.w3c.dom.Node;
import de.dfki.lt.hog.Metadata;
import de.dfki.lt.hog.Module;
import de.dfki.lt.hog.exceptions.InitializationException;
import de.dfki.lt.hog.util.Utility;

/**
 * Module definition for Pet Input Chart generation from plain text
 * Created 2004-10-13 23:40
 * @author <a href="mailto:ulrich.schaefer@dfki.de">Ulrich Schaefer</a>
 * @version  $Id: PicModule.java,v 1.2 2005/04/26 17:37:37 oeze Exp $
 */

 /* Changes:
  *
 */

public class PicModule extends Module {

	static Logger log = Logger.getLogger(PicModule.class);
    private String sOutputAnnotationName = null;
    private String sInputAnnotationName = null;

	public PicModule(Properties picModuleProps) {
		try {
                  super.init(picModuleProps);
		} catch (InitializationException ie) {
                  log.fatal(Utility.exception2String(ie));
		}

                sInputAnnotationName = getConfig().getProperty("pic.inputtextannotation", "rawtext");
                sOutputAnnotationName = getConfig().getProperty("pic.outputannotation", "picraw");

		log.info("PicModule initialized.");
	}

	public void shutdown() {
		//  no corresponding method found in SproutInterpreter (Sprout API)
		log.info("shutting down PicModule.");
	}

	/* (non-Javadoc)
	 * @see de.dfki.lt.hog.Module#process(java.lang.String)
	 */
	public String process(String processString) {
            // get raw input text
            String rawText = Utility.getComputedAnnotation(getMoCoMan(), processString, sInputAnnotationName);
            String root = getModuleRootElement();
            Metadata md = new Metadata(processString);
            String sessionID = md.getSessionID();
            String acID = md.getAcID();

            StringBuffer sbPic = new StringBuffer();

            StringTokenizer st= new StringTokenizer(rawText, ",;.:'\"!()[]{}=/?\\|<>%&$^\uFFFD~*#_ \n\t", true);
            String token = null;
            int id = 0;
            int offset = 0;

            while (st.hasMoreTokens()) {
                token = st.nextToken();
                if (!(token.equals(" ")||token.equals("\t")||token.equals("\n"))) {
                	sbPic.append(toPiXml(id++, token, offset));
                }
                offset += token.length();
            }

            getMoCoMan().createAnnotation(sessionID, acID, sbPic.toString(),
                                         getModuleLanguage(), sOutputAnnotationName);

            StringBuffer sbResult = new StringBuffer();
            sbResult.append("<?xml version='1.0'?>");
            sbResult.append("<" + root + ">");
            md = new Metadata(sessionID, acID, "0.0",
                              getModuleName(), "OK", false, getConfig());
            sbResult.append(md.getXmlString());
            sbResult.append(sbPic);
            sbResult.append("</" + root + ">");
            return sbResult.toString();
        }


    /*
    String cat = null;
    String surface = null;
    String zn = null;
    if (!(token.equals(" ")||token.equals("\t")||token.equals("\n"))) {
        if (sbPic.length()>0) sbPic.append(" ");
        surface = token;
        cat = null;
        if (Utility.isInterpunct(token)) {
            zn = "zero";
            cat = token;
        } else {
              zn = "null";
        }
        if (surface.equals("\"")) {
            surface = cat = "\\\"";
            zn = "zero";
        }
        sbPic.append("( " + (id++) + ", " + offset + ", " + (offset + token.length()) +
                     ", 1, " + "\"" + surface +"\", 0, \"" + zn + "\"");
        if (cat!=null) sbPic.append(", \""+ cat + "\" 1.0");
        sbPic.append(" )");
    }
    */

    public String toPiXml(int id, String token, int offset) {
      	boolean isInterpunct = Utility.isInterpunct(token);
      	StringBuffer out = new StringBuffer(
      	        "<w id=\"PIC" + id + "\" cstart=\"" + offset +
      	        "\" cend=\"" + (offset + token.length() - 1) + "\"" +
    			( isInterpunct ? " constant=\"yes\"" : "" ) + ">\n" +
      		    "  <surface>" + Utility.cleanXMLString(token) + "</surface>\n");
      		if (isInterpunct)
      			out.append("  <pos tag=\"" + Utility.cleanXMLString(token) + "\" prio=\"1.0\"/>\n");
      		int number = -1;
      		try {
      		  number = Integer.parseInt(token);
      		} catch (NumberFormatException nfe) {};
      		if (number != -1) {
      			out.append("  <typeinfo id=\"PIT" + id + "\" baseform=\"no\">\n");
				out.append("    <stem>$generic_number</stem>\n");
			    out.append("  </typeinfo>\n");
      		}
      		out.append("</w>\n");
      	    return out.toString();
      }


        /* (non-Javadoc)
         * @see de.dfki.lt.hog.Module#processDOMTree(org.w3c.dom.Node)
	 */
	public Node processDOMTree(Node domTree) {
		// TODO Auto-generated method stub
		// use a simple SAX parser to extract text from domTree
		// return a DOM from transformer DOMResult (i.e., copy most of process() except final DOM output)
		return null;
	}

}
