/*
 * SdlModule.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.hog.modules;

import java.util.Properties;
import org.apache.log4j.Logger;
import org.w3c.dom.Node;
import de.dfki.lt.hog.Metadata;
import de.dfki.lt.hog.Module;
import de.dfki.lt.hog.container.AnnotationEncapsulator;
import de.dfki.lt.hog.exceptions.InitializationException;
import de.dfki.lt.hog.util.Utility;
import de.dfki.lt.sdl.*;

/**
 * Module definition for SDL runtime API.
 * Created 2004-07-19
 * @author <a href="mailto:ulrich.schaefer@dfki.de">Ulrich Schaefer</a>
 * @version $Revision: 1.4 $
 * $Id: SdlModule.java,v 1.4 2007/09/26 16:45:08 uschaefer Exp $
 */

/* Changes:
 * 2004-07-22 : added empty result handling (metadata)
 */

public class SdlModule extends Module {

    static Logger log = Logger.getLogger(SdlModule.class);
    private String sdlClassname = null;
    private String sdlInputAnnotationName = null;
    private Modules sdlObject = null;

    public SdlModule(Properties sdlModuleProps) {
        String sdlConfigFile = null;
        try {
            super.init(sdlModuleProps);
        } catch (InitializationException ie) {
            log.fatal(Utility.exception2String(ie));
        }
        this.sdlClassname = getConfig().getProperty("sdl.classname", null);
        this.sdlInputAnnotationName = getConfig().getProperty("sdl.inputannotation", "rawtext");
        log.info("Initializing SDL class " + this.sdlClassname + "...");

        try {
            // initialize SDL cascade
            this.sdlObject =  (Modules) Class.forName(this.sdlClassname).newInstance();
            log.info("SDL object " + this.sdlClassname + " initialized.");
        } catch (Throwable t) {
            log.error("Could not initialize SDL object " + this.sdlClassname + ": " + t.toString());
	}
    }

    public void shutdown() {
        //  no corresponding method found in InterpreterTest (Sprout API)
        log.info("shutting down Sdl.");
    }

    /* (non-Javadoc)
     * @see de.dfki.lt.hog.Module#process(java.lang.String)
     */
    public String process(String processString) {
    	String duration = "00:00,000";
    	long t1 = System.currentTimeMillis();
        String rawText = Utility.getComputedAnnotation(getMoCoMan(),
                                                    processString,
                                                    this.sdlInputAnnotationName);
        Metadata metadata = new Metadata(processString);
        AnnotationEncapsulator ae = new AnnotationEncapsulator(metadata, rawText);
        ae.setMoCoMan(getMoCoMan());
        ae.setLanguage(getModuleLanguage());
        log.debug("input text: \"" + rawText + "\"");
        log.info("running SDL object " +  this.sdlClassname +  "..." );
        String xmlresult = "";
        String diagnosis = "OK";
        try {
            // run SDL cascade
            AnnotationEncapsulator aeResult = (AnnotationEncapsulator) this.sdlObject.run(ae);
            xmlresult = (String) aeResult.getAnnotation();
        } catch (Throwable t) {
            log.error("Error in SDL object " + this.sdlClassname + ": " + t.toString());
            t.printStackTrace();
            diagnosis = t.toString();
        }
        duration = Utility.formatDuration(System.currentTimeMillis()-t1);
        log.info("processing time of SDL object " + this.sdlClassname + ":" + duration);
        boolean empty = false;
	int rmrsstart = 0;
	String result = "";
	if (xmlresult.indexOf("<rmrs")>=0) {
	    if (xmlresult.equals("") || xmlresult.indexOf("</rmrs>")==-1 || xmlresult.indexOf("<rmrs-list/>")>=0)
		empty = true;
	    rmrsstart = xmlresult.indexOf("<rmrs-list");
	    if (rmrsstart==-1) {
		xmlresult = "";
		rmrsstart = 0;
	    }
	    result =  "<" + getModuleRootElement() + ">" +
		Metadata.generateXml(metadata.getSessionID(), metadata.getAcID(),
				     duration, getModuleName(), diagnosis,
				     empty, getConfig()) + xmlresult.substring(rmrsstart) +
		"</" + getModuleRootElement() + ">";
	} else result = xmlresult;

        log.debug(result);
        return result;
    }


    /* (non-Javadoc)
     * @see de.dfki.lt.hog.Module#processDOMTree(org.w3c.dom.Node)
     */
    public Node processDOMTree(Node domTree) {
        // TODO Auto-generated method stub
        // use a simple SAX parser to extract text from domTree
        // return a DOM from transformer DOMResult (i.e., copy most of process() except final DOM output)
        return null;
    }

}
