/*
 * TnTModule.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.hog.modules;

import java.io.*;
import java.util.*;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

import org.apache.log4j.Logger;
import org.w3c.dom.*;
import de.dfki.lt.hog.Metadata;
import de.dfki.lt.hog.MoCoMan;
import de.dfki.lt.hog.Module;
import de.dfki.lt.hog.TransformationService;
import de.dfki.lt.hog.container.JToken;
import de.dfki.lt.hog.container.TnToken;
import de.dfki.lt.hog.exceptions.InitializationException;
import de.dfki.lt.hog.util.Utility;

/**
 * Module definition for TnT commandline interface
 * 
 * @author <a href=mailto:oeze@dfki.de>Oezguer Demir</a>
 * @version $Revision: 1.9 $ $Id: TnTModule.java,v 1.7 2005/08/24 17:43:42 oeze
 *          Exp $
 */

/*
 * Changes: 2004-05-18 : US xml header with encoding (new config property
 * tnt.outputencoding) 2004-07-22 : US added empty result handling (metadata)
 * 2004-09-09 : US input + outputencoding properly passed to process streams
 * 2004-11-22 : US generate PET input chart in XML format (obsoletes
 * TnTpicModule)
 * 2007-08-17: added smaf-output + options (default is pixml)
 */

public class TnTModule extends Module {

    static Logger log = Logger.getLogger(TnTModule.class);

    private BufferedReader procOut;

    private BufferedWriter procIn;

    private BufferedReader procErr;

    private Runtime runtime;

    private Process process;

    private String tntOptions = "";

    private String tntScript = "";

    private String outputEncoding;

    private String inputEncoding;

    private String tntPiXmlAnnotationName;

    private String tntPiXmlRootElement;

    private String tntSmafOutput;
    
    private String tntSmafAnnotationName;
    
    private String tntSmafStylesheet;
    private String sSmafOut;

    private static final String NEWLINE = System.getProperty("line.separator");

    private static final String DATA_ROOT_ELEMENT = "tokens";

    private static int lifeCycle = 0;

    private TransformationService transformationService = null;

    private Transformer transformer = null;

    public TnTModule(Properties tntModuleProps) throws InitializationException {
        try {
            super.init(tntModuleProps);
        } catch (InitializationException ie) {
            log.error(Utility.exception2String(ie));
        }
        tntScript = getConfig().getProperty("tnt.script");
        tntOptions = getConfig().getProperty("tnt.options");
        outputEncoding = getConfig().getProperty("tnt.outputencoding",
                "ISO-8859-1");
        inputEncoding = getConfig().getProperty("tnt.inputencoding",
                "ISO-8859-1");
        tntPiXmlAnnotationName = getConfig().getProperty(
                "tnt.piXMLoutputannotation", "TnTpiXML");
        tntPiXmlRootElement = getConfig().getProperty("tnt.piXMLrootelement",
                "pet-input-chart");
        // defaults to no, if not set in tnt.cfg
        tntSmafOutput = getConfig().getProperty("tnt.smafOutput", "0");
        tntSmafStylesheet = getConfig().getProperty("tnt.smafStylesheet");
        tntSmafAnnotationName = getConfig().getProperty("tnt.smafOutputAnnotation", "SMAF");

        start();
    } // TnTModule()

    public void start() {

        log.info("Initializing TnT ...");
        runtime = Runtime.getRuntime();

        String sname = tntScript + " " + tntOptions;
        log.debug("Invoking " + sname);

        try {
            process = runtime.exec(sname);
            procOut = new BufferedReader(new InputStreamReader(process
                    .getInputStream(), outputEncoding));
            procIn = new BufferedWriter(new OutputStreamWriter(process
                    .getOutputStream(), inputEncoding));
            procErr = new BufferedReader(new InputStreamReader(process
                    .getErrorStream(), outputEncoding));
        } catch (IOException ioe) {
            log.fatal(Utility.exception2String(ioe));
        }
        // try {
        // process = runtime.exec(tntScript + " " + tntOptions);
        // log.debug("Invoked: " + tntScript + " " + tntOptions);
        // String lineIn;
        // procOut = new BufferedReader(
        // new InputStreamReader(process.getInputStream()));
        // procIn = new BufferedWriter(
        // new OutputStreamWriter(process.getOutputStream()));
        // test
        // if (procOut.ready()) {
        // log.info("TnT initialized (tested).");
        // procIn.close();
        // procOut.close();
        // }
        // else {
        // throw new InitializationException("TnT did not start");
        // } // else
        // } // try
        // catch (IOException ioe) {
        // log.fatal(ioe.getMessage());
        // }
    }

    public void shutdown() {
        log.info("Shutting down TnT.");
        try {
            procIn.close();
            procOut.close();
            procErr.close();
        } catch (IOException ioe) {
            log.fatal(Utility.exception2String(ioe));
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.dfki.lt.hog.Module#process(java.lang.String)
     */
    public String process(String processString) {
        // lifeCycle++;
        // if (lifeCycle >= 1000) {
        // shutdown();
        // start();
        // lifeCycle = 0;
        // }
        String duration = "00:00,000";
        long t1 = System.currentTimeMillis();
        MoCoMan mocoman = getMoCoMan();
        String rawText = Utility.getComputedAnnotation(mocoman, processString,
                "rawtext");
        Metadata md = new Metadata(processString);
        String sessionID = md.getSessionID();
        String acID = md.getAcID();
        String root = getModuleRootElement();
        // get tokenized text
        ArrayList jtoks = JToken.parseJTokAnnotation(Utility
                .getComputedAnnotation(mocoman, processString, "JTok"));
        // split sentences according to JTok
        StringBuffer tokenBuffer = new StringBuffer();
        String lineSep;
        int numberOfTokens = jtoks.size();
        for (int i = 0; i < jtoks.size(); i++) {
            JToken tok = (JToken) jtoks.get(i);
            if (tok.getType().startsWith("PTERM")) {
                lineSep = NEWLINE + NEWLINE;
            } else {
                lineSep = NEWLINE;
            }
            tokenBuffer.append(tok.getToken() + lineSep);
        }
        String tokenizedStr = tokenBuffer.toString();
        // log.debug("JTok tokenized like this: " + tokenizedStr);
        // log.debug("JTok returned " + numberOfTokens + " tokens.");
        // process text
        String lineIn;
        String metadata;
        StringBuffer sbResult = new StringBuffer();
        boolean inResult = false;
        String diagnosis = "";
        try {
            // feed the text to TnT
            procIn.write(tokenizedStr + NEWLINE);
            procIn.flush();
            sbResult.append("<" + DATA_ROOT_ELEMENT + ">" + NEWLINE);
            // get results from TnT
            StringBuffer resultBuffer = new StringBuffer();
            // if (procOut.ready()) {
            // lineIn = procOut.readLine();
            // } else {
            // lineIn = "";
            // }
            lineIn = "";
            int tokenCounter = 0;
            while (tokenCounter != numberOfTokens) {
                lineIn = procOut.readLine();
                if (lineIn.length() > 0) {
                    tokenCounter++;
                }
                // log.debug("TnT out said: " + lineIn);
                // log.debug("TnT out said sth of length " + lineIn.length());
                // log.debug("numberOfTokens: " + numberOfTokens +
                // "tokenCounter: " + tokenCounter);
                resultBuffer.append(lineIn + NEWLINE);
            } // for i
            StringBuffer errBuffer = new StringBuffer();
            // log.debug("debug 1633");
            while (procErr.ready()) {
                lineIn = procErr.readLine();
                // log.debug("TnT err said: " + lineIn);
                // log.debug("TnT err said sth of length " + lineIn.length());
                errBuffer.append(lineIn);
            } // while
            if (errBuffer.toString().equals("")) {
                diagnosis = "OK";
            } else {
                diagnosis = errBuffer.toString();
                log.error(diagnosis);
            }
            // log.debug("debug 1634");
            // convert to XML
            String tntresult = resultBuffer.toString();
            // check whether result is empty
            boolean empty = false;
            if (tntresult.trim().equals("")) {
                empty = true;
            }
            // log.debug("debug 1635");
            log.debug("tntresult: " + tntresult);
            sbResult.append(tntokens2XML(parseTnT(tntresult, jtoks)));
            // log.debug("debug 1636");
            String sPiXML = (tntokens2PiXML(parseTnT(tntresult, jtoks)));
            // log.debug("debug 1637");
            duration = Utility.formatDuration(System.currentTimeMillis() - t1);
            // log.debug("debug 1638");

            // generate the header
            StringBuffer header = new StringBuffer();
            // log.debug("debug 1639");
            header.append("<?xml version='1.0'?>"); // encoding='" +
                                                    // outputEncoding + "'
            header.append("<" + root + ">");
            header.append(Metadata.generateXml(sessionID, acID, duration,
                    getModuleName(), diagnosis, empty, getConfig()));
            // log.debug("debug 1640");
            sbResult.append("</" + DATA_ROOT_ELEMENT + ">" + NEWLINE);
            sbResult.append("</" + root + ">");
            String sResult = header.toString().concat(sbResult.toString());
            // log.debug("result: " + sResult);

            String sPiXMLInput = "<?xml version='1.0'?><"
                    + tntPiXmlRootElement
                    + ">"
                    + Metadata.generateXml(sessionID, acID, duration,
                            tntPiXmlAnnotationName, diagnosis, empty,
                            getConfig()) + sPiXML + "</" + tntPiXmlRootElement
                    + ">";
            
            // if enabled, generate SMAF from PIXML using transformer+stylesheet
            if (tntSmafOutput.equals("1")) {
                
                if (this.transformationService == null) {
                    this.transformationService = new TransformationService(
                            getMoCoMan());
                }

                try {
                    transformer = this.transformationService
                            .createTransformer(tntSmafStylesheet);
                } catch (TransformerConfigurationException tce) {
                    log.error(Utility.exception2String(tce));
                }

                try {
                    sSmafOut = this.transformationService.transform(
                            transformer, sPiXMLInput);
                } catch (TransformerException te) {
                    log.error(Utility.exception2String(te));
                }

                log.debug("sPiXMLInput:\n" + sPiXMLInput);
                log.debug("smafOutput:\n" + sSmafOut);

                // store smaf in hog
                mocoman.createAnnotation(sessionID, acID, sSmafOut,
                        getModuleLanguage(), tntSmafAnnotationName);

            } else {
                // store pet input chart (XML)
                mocoman.createAnnotation(sessionID, acID, sPiXMLInput,
                        getModuleLanguage(), tntPiXmlAnnotationName);
            }
            log.info("Processing time of TnTModule: " + duration);
            // if (lifeCycle == 500) { log.error("spotCheck-500: " + sResult); }
            return sResult;
            
        } catch (IOException ioe) {
            log.fatal(Utility.exception2String(ioe));
            return "<" + root + ">" + Metadata.generateXml(sessionID, acID, duration,
                            getModuleName(), ioe.getMessage(), true,
                            getConfig()) + "</" + root + ">";
        }
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see de.dfki.lt.hog.Module#processDOMTree(org.w3c.dom.Node)
     */
    public Node processDOMTree(Node domTree) {
        // TODO Auto-generated method stub
        // use a simple SAX parser to extract text from domTree
        // return a DOM from transformer DOMResult (i.e., copy most of process()
        // except final DOM output)
        return null;
    }

    /**
     * Parses the -z0 output of TnT and returns an ArrayList of {@link TnToken}
     * instances.
     * 
     * @param tntOutput
     *            The output of TnT with switch -z0
     * @param jtokens
     *            An ArrayList of {@link JToken} instances
     * @return
     */
    private static ArrayList parseTnT(String tntOutput, ArrayList jtokens) {
        ArrayList tntokens = new ArrayList();
        ArrayList lines = Utility.splitLines(tntOutput);
        if (lines.size() == 0) {
            log.error("Token list received by TnT is empty");
            tntokens.add(new TnToken(new JToken(" ", 1, 0)));
            return tntokens;
        }
        for (int i = 0; i < jtokens.size(); i++) {
            TnToken tntoken = new TnToken((JToken) jtokens.get(i));
            ArrayList line = Utility.splitTabs((String) lines.get(i));
            for (int j = 1; j < line.size() - 1; j += 2) {
                if (!((j + 1) >= line.size())) {
                    tntoken.addTag((String) line.get(j), (String) line
                            .get(j + 1));
                }
            }
            tntokens.add(tntoken);
        }
        return tntokens;
    }

    // TODO: die beiden durchgänge für smaf-output
    // (hauptteil macht container/TnToken.java
    // NEU: TnToken.toPreSmafXml() (== .toXML() )

    private static String tntokens2XML(ArrayList tntokens) {
        StringBuffer out = new StringBuffer();
        for (int i = 0; i < tntokens.size(); i++) {
            out.append(((TnToken) tntokens.get(i)).toXML());
        }
        return out.toString();
    }

    private static String tntokens2PiXML(ArrayList tntokens) {
        StringBuffer out = new StringBuffer();
        for (int i = 0; i < tntokens.size(); i++) {
            out.append(((TnToken) tntokens.get(i)).toPiXML(i));
        }
        return out.toString();
    }

} // class
