/*
 * SampleRemoteTagger.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */


package de.dfki.lt.hog.modules.test;

import java.util.Properties;
import java.util.Vector;
import org.w3c.dom.Node;

import de.dfki.lt.hog.XmlRpcModule;
import de.dfki.lt.hog.exceptions.InitializationException;
import de.dfki.lt.hog.util.Utility;

/**
 * This module can only communicate with an adaptor, a remote module,
 * via XML-RPC. <code>process(String)</code> calls
 * <code>process(String, Vector)</code> specified by the
 * <code>XmlRpcClientInterface</code> which is implemented by the
 * abstract <code>XmlRpcModule</code> class.
 *
 * @see de.dfki.lt.hog.XmlRpcClientInterface
 * @see de.dfki.lt.hog.XmlRpcModule
 * @author <a href="mailto:robba@dfki.de">Robert Barbey</a>
 * @version 2003-11-04
 */
public class SampleRemoteTagger
    extends XmlRpcModule
{
    /**
     * Creates a <code>SampleRemoteTagger</code> configured by given
     * properties.
     *
     * @param remoteTaggerProps configures this
     *      <code>SampleRemoteTagger</code>
     */
    public SampleRemoteTagger(Properties remoteTaggerProps)
    {
	try {
	    super.init(remoteTaggerProps);
	} catch (InitializationException ie) {
	    xmlRpcModuleLog.fatal(Utility.exception2String(ie));
	}
    }

    public String process(String processString)
    {
	Vector params = new Vector();
	String handlerCall = getXmlRpcHandlerName()
	    + "."
	    + getXmlRpcHandlerMethod();

	params.add(processString);

	return super.process(handlerCall, params);
    }

    public Node processDOMTree(Node processTree)
    {
	return null;
    }

}
