/*
 * SampleTagger.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */


package de.dfki.lt.hog.modules.test;

import java.util.*;
import org.w3c.dom.*;

import de.dfki.lt.hog.Module;
import de.dfki.lt.hog.exceptions.InitializationException;

/**
 * This is a pseudo tagger just for demonstration purposes.
 * It gives a token a <tt>UPPER<tt> tag if this token starts
 * with an upper case character.
 *
 * @author <a href="mailto:robba@dfki.de">Robert Barbey</a>
 */

public class SampleTagger
extends Module {

	/**
	 * Creates a <code>Tagger</code> with a given <code>Properties</code>
	 * object.
	 * @param taggerProps configurates this <code>Tagger</code>
	 */
	public SampleTagger(Properties taggerProps) {
		try {
			this.init(taggerProps);
		} catch (InitializationException e) {
			System.err.println(e.getMessage());
		}
	}

	/**
	 * A sample tagger to illustrate communication with MoCoMan.
	 * Tags values in a given XML string with <tt>UPPER<tt> or <tt>LOWER<tt>
	 * if their first characters are upper or lower case, repectively.
	 * @param processString XML string to process
	 * @return XML string with <tt>pos<tt> attributes
	 */
	public String process(String processString) {
		StringBuffer output = new StringBuffer(processString);
		// contains position of next <W> tag
		int tagPosition = output.indexOf("<W");
		// contains current insert position for "pos"-attribute
		int insertPosition = 0;

		// while end of String is not reached
		while (tagPosition != -1) {
			int startReadPos, endReadPos;
			// current word to tag
			String temp, tag;

			insertPosition = output.indexOf(">", tagPosition);
			startReadPos = insertPosition + 1;
			endReadPos = output.indexOf("</W>", startReadPos - 1);

			temp = output.substring(startReadPos, endReadPos);
			if (Character.isUpperCase(temp.charAt(0))) {
				tag = "UPPER";
			} else {
				tag = "LOWER";
			}

			output.insert(insertPosition, " pos=\"" + tag + "\"");
			tagPosition = output.indexOf("<W", insertPosition);
		}

		return output.toString();
	}

	public Node processDOMTree(Node domTree)
	{
		return null;
	}

}
