/*
 * SampleTaggerAdaptor.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.hog.modules.test;

import java.util.*;
import de.dfki.lt.hog.Adaptor;

/**
 * @author <a href="mailto:robba@dfki.de">Robert Barbey</a>
 */
public class SampleTaggerAdaptor
extends Adaptor {

	public SampleTaggerAdaptor(Properties taggerProps) {
		super(taggerProps);
	}

	/**
	 * Tags an XML-String with "UPPER" or "LOWER" tags. Creates a StringBuffer,
	 * and inserts at rightmost position of <code>&lt;W ... &gt;</code> an attribute
	 * <code>pos=...</code> with appropriate tag.
	 * @param xmlString Document to tag
	 * @return tagged document
	 */
	public String process(String xmlString) {
		StringBuffer output = new StringBuffer(xmlString);
		// contains position of next <W> tag
		int tagPosition = output.indexOf("<W");
		// contains current insert position for "pos"-attribute
		int insertPosition = 0;

		// while end of String is not reached
		while (tagPosition != -1) {
			int startReadPos, endReadPos;
			// current word to tag
			String temp, tag;

			insertPosition = output.indexOf(">", tagPosition);
			startReadPos = insertPosition + 1;
			endReadPos = output.indexOf("</W>", startReadPos - 1);

			temp = output.substring(startReadPos, endReadPos);
			if (Character.isUpperCase(temp.charAt(0))) {
				tag = "UPPER";
			} else {
				tag = "LOWER";
			}

			output.insert(insertPosition, " pos=\"" + tag + "\"");
			tagPosition = output.indexOf("<W", insertPosition);
		}


		return output.toString();
	}

	public void shutdown()
	{
		super.shutdown();
	}



}
