/*
 * ModuleTest.java
 *
 * (C) 2007 DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.hog.test;

import java.io.*;
import java.util.Properties;
import java.util.regex.*;

import javax.xml.xpath.*;

import org.apache.xmlrpc.*;

import de.dfki.lt.hog.Application;
import de.dfki.lt.hog.MoCoMan;
import de.dfki.lt.hog.util.Utility;

import static org.junit.Assert.*;
import org.junit.Test;
import org.xml.sax.InputSource;


/**
 * junit Test for a mocoman application using pet or tnt
 * 
 * @version
 * @author <a href="mailto:daniel.contag@dfki.de">Daniel Contag</a>
 */
public class ModuleTest extends Application {

  public ModuleTest(Properties applProps) {
    try {
      super.init(applProps);
    } catch (IOException ioe) {
      applicationLog.error(Utility.exception2String(ioe));
    }
  }

  public ModuleTest() {
  }

  @Test
  public void testModule() throws java.net.MalformedURLException,
      XmlRpcException, FileNotFoundException {
    Properties applProps = new Properties();
    try {
      FileInputStream fis = new FileInputStream("conf/test/application.cfg");
      applProps.load(fis);
      fis.close();
    } catch (IOException ioe) {
      applicationLog.error(Utility.exception2String(ioe));
    }

    ModuleTest moduleApp = new ModuleTest(applProps);

    // create a MoCoMan
    MoCoMan mocoman = MoCoMan.getInstance();
    String mtCfg = System.getProperty("moduletest.cfg");
    String sessionId = mocoman.createSession(mtCfg);

    // invoke analysis
    String input = System.getProperty("test.input");

    String annCollId = mocoman.createAnnotationCollection(sessionId);

    String annotationId = mocoman.createInitialAnnotation(sessionId, annCollId,
        input, "en", "ModuleTestApp");

    String output = mocoman.analyse(sessionId, annCollId, annotationId, 0, 0,
        100);
    
    // testing pet
    if ((System.getProperty("test.mode")).equals("pet")) {

      int number_readings = Integer.parseInt(System
          .getProperty("number.readings"));
      
      Pattern rp = Pattern.compile("\\\\*<pet readings=\"(\\w+)\">\\\\*");
      Matcher rpMatcher = rp.matcher(output);
      
      // test fails here if no match is found
      assertTrue(rpMatcher.find());

      String strNbr = rpMatcher.group(1);
      int nr = Integer.parseInt(strNbr);
      
      assertTrue(nr > number_readings);

    }
    
    else
      // testing tnt
      if ((System.getProperty("test.mode")).equals("tnt")) {
      
      XPath xpath = XPathFactory.newInstance().newXPath();
      
      // xpath pattern to test if 2nd element exists
      String test_element = "/tnt/tokens";
      
      // xpath pattern to test if 2. element has children
      // ('*' selects all children if any)
      String test_children = "/tnt/tokens/*";
      
      StringReader r = new StringReader(output);
      InputSource is = new InputSource(r);
      
      try {
        Boolean t1 = (Boolean) xpath.evaluate(test_element, is, XPathConstants.BOOLEAN);
        
        r = new StringReader(output);
        is = new InputSource(r);
        Boolean t2 = (Boolean) xpath.evaluate(test_children, is, XPathConstants.BOOLEAN);
        
        assertTrue(t1);
        assertTrue(t2);
        
      } catch (XPathExpressionException e) {
        e.getMessage();
        e.printStackTrace();
      }
      
    }
    else 
      // no testmode selected, so we fail here
      assertTrue(false);

    mocoman.shutdown();

  }
}
