/*
 * SampleApplication.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.hog.test;

import java.io.*;
import java.util.Properties;
import org.apache.log4j.Logger;
import de.dfki.lt.hog.Application;
import de.dfki.lt.hog.MoCoMan;


/**
 * An application for test purposes.
 *
 * @author <a href=mailto:oeze@dfki.de>Oezguer Demir</a>
 * @version $Revision: 1.2 $
 * $Id: SampleApplication.java,v 1.2 2005/04/26 17:37:44 oeze Exp $
 */
public class SampleApplication
    extends Application {

  /**
   * A <code>Logger</code> for information and debugging.
   */
  static Logger log = Logger.getLogger(SampleApplication.class);

  public SampleApplication(Properties applProps) {
    try {
      super.init(applProps);
    }
    catch (IOException ioe) {
      System.err.println(ioe.getMessage());
    }
  }

  public static void main(String[] args) {
    Properties applProps = new Properties();
    try {
      FileInputStream fis = new FileInputStream("conf/test/application.cfg");
      applProps.load(fis);
      fis.close();
    }
    catch (IOException ioe) {
      System.err.println(ioe.getMessage());
    }
    SampleChunkApplication sappl = new SampleChunkApplication(applProps);
    System.err.println("This is a test application for HoG.");
    MoCoMan mocoman = MoCoMan.getInstance();
    String sessionId = mocoman.createSession("conf/test/testsession.cfg");
    System.err.println("Session " + sessionId + " created.");
    String doc =
        //"(((foo & ''')((";
        "Dies ist ein Test";
        //"This is a test";
    String annCollId = mocoman.createAnnotationCollection(sessionId);
    String annotationId =
        mocoman.createInitialAnnotation(sessionId, annCollId, doc, "de",
                                        "sampleapp");
    String output =
        mocoman.analyse(sessionId, annCollId, "xmltext", 0, 0, 100);
    System.err.println(output);
    System.err.println(
        mocoman.xml2htmlTransformer(sessionId, annCollId, "TnT",
                                    "xsl/html/xml2html.xsl", "no"));
    mocoman.shutdown();
    System.err.println("Test application finished.");
  }
}
