/*
 * SampleXmlRpcApplication.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.hog.test;

import java.util.Properties;
import java.util.Vector;
import java.io.IOException;
import java.io.FileInputStream;
import org.apache.xmlrpc.XmlRpcException;
import de.dfki.lt.hog.MoCoManServer;
import de.dfki.lt.hog.XmlRpcApplication;
import de.dfki.lt.hog.util.Utility;

/**
 * This class demonstrates how to connect create and connect to a
 * <tt>MoCoManServer<tt> and how to use that server to invoke methods
 * remotely via XML-RPC.
 *
 * @author <a href"mailto:robba@dfki.de">Robert Barbey</a>
 * @version $Revision: 1.2 $
 * $Id: SampleXmlRpcApplication.java,v 1.2 2005/04/26 17:37:45 oeze Exp $
 */

public class SampleXmlRpcApplication
    extends XmlRpcApplication
{
    public SampleXmlRpcApplication(Properties sampleProps)
    {
	super.init(sampleProps);
    }

    public static void main(String argv[])
    {
	// an answer of XmlRpc calls
	String xmlRpcAnswer = null;
	String xmlRpcSessionId = null;
	String xmlRpcAnnCollId = null;
	String xmlRpcAnnotationId = null;

        MoCoManServer mocomanServer = null;
        try {
          mocomanServer = new MoCoManServer();
        }
        catch (IOException ioe) {
          String eMessage = "Couldn't set up XML-RPC WebServer on default port."
              + "\n Error: " + Utility.exception2String(ioe);
          System.err.println(eMessage);
          System.exit(-1);
        } // catch

//        mocomanServer.wait();

	Properties sxraProps = new Properties();

	try {
	    FileInputStream fis =
		new FileInputStream("conf/test/xmlapp.cfg");
	    sxraProps.load(fis);
	    fis.close();
	} catch (IOException ioe) {
	    applicationLog.error(Utility.exception2String(ioe));
	}

	SampleXmlRpcApplication sxra =
	    new SampleXmlRpcApplication(sxraProps);

	// create a session
	Vector params = new Vector();
	params.add("conf/xmlrpcsession.cfg");
        //applicationLog.info("giving " + params + " to ");////DEBUG
	try {
	    xmlRpcSessionId =
		(String) sxra.getXmlRpcApplicationClient().
		execute(sxra.getXmlRpcHandlerName()
			+ ".createSession", params);
	} catch (IOException ioe) {
	    applicationLog.error(Utility.exception2String(ioe));
	} catch (XmlRpcException xre) {
	    applicationLog.error(Utility.exception2String(xre));
	}

	applicationLog.info("Session created with ID: "
			    + xmlRpcSessionId);

	//create AnnotationCollection
	params.clear();
	params.add(xmlRpcSessionId);
	try {
	    xmlRpcAnnCollId =
		(String) sxra.getXmlRpcApplicationClient().
		execute(sxra.getXmlRpcHandlerName()
			+ ".createAnnotationCollection",
			params);
	} catch (IOException ioe) {
	    applicationLog.error(Utility.exception2String(ioe));
	} catch (XmlRpcException xre) {
	    applicationLog.error(Utility.exception2String(xre));
	}

	applicationLog.info("AnnotationCollection created with ID: "
			    + xmlRpcAnnCollId);

	//create AnnotationCollection
	params.clear();
	params.add(xmlRpcSessionId);
	params.add(xmlRpcAnnCollId);
	params.add("Dies ist ein XML-RPC Beispieldokument, "
		   + "um den Programmflu\uFFFD zu illustrieren.");
	params.add("de");
	params.add("rawtext");
	try {
	    xmlRpcAnnotationId =
		(String) sxra.getXmlRpcApplicationClient().
		execute(sxra.getXmlRpcHandlerName()
			+ ".createAnnotation",
			params);
	} catch (IOException ioe) {
	    applicationLog.error(Utility.exception2String(ioe));
	} catch (XmlRpcException xre) {
	    applicationLog.error(Utility.exception2String(xre));
	}

	applicationLog.info("Annotation created with ID: "
			    + xmlRpcAnnotationId);

	// analyse
	params.clear();
	params.add(xmlRpcSessionId);
	params.add(xmlRpcAnnCollId);
	params.add(xmlRpcAnnotationId);
	params.add(new Integer(0));
	params.add(new Integer(0));
	params.add(new Integer(1));
	try {
	    xmlRpcAnswer =
		(String) sxra.getXmlRpcApplicationClient().
		execute(sxra.getXmlRpcHandlerName()
			+ ".analyse",
			params);
	} catch (IOException ioe) {
	    applicationLog.error(Utility.exception2String(ioe));
	} catch (XmlRpcException xre) {
	    applicationLog.error(Utility.exception2String(xre));
	}

	applicationLog.info("Analysis output: "
			    + xmlRpcAnswer);

        // shutdown mocomanServer
        mocomanServer.shutdown();
        applicationLog.info("MoCoManServer was shut down");

	// shutdown mocoman
	params.clear();
	try {
	    xmlRpcAnswer =
		(String) sxra.getXmlRpcApplicationClient().
		execute(sxra.getXmlRpcHandlerName()
			+ ".shutdown", params);
	} catch (IOException ioe) {
	    applicationLog.error(Utility.exception2String(ioe));
	} catch (XmlRpcException xre) {
	    applicationLog.error(Utility.exception2String(xre));
	}

	applicationLog.info("MoCoMan was shut down");
    }
}
