/*
 * SproutPetApplication.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.hog.test;

import java.io.*;
import java.util.Properties;
import org.apache.xmlrpc.*;
import de.dfki.lt.hog.Application;
import de.dfki.lt.hog.Metadata;
import de.dfki.lt.hog.MoCoMan;
import de.dfki.lt.hog.util.Utility;

/**
 * Test for Mocoman application based on Sprout and JTok
 *
 * @version $Revision: 1.2 $
 * $Id: SproutPetApplication.java,v 1.2 2005/04/26 17:37:45 oeze Exp $
 * @author <a href="mailto:ulrich.schaefer@dfki.de">Ulrich Schaefer</a>
 */
public class SproutPetApplication
    extends Application
{

    public SproutPetApplication(Properties applProps)
    {
	try {
            super.init(applProps);
	} catch (IOException ioe) {
	    applicationLog.error(Utility.exception2String(ioe));
	}
    }

    public static void main(String[] args)
	throws java.net.MalformedURLException,
	       XmlRpcException,
	       FileNotFoundException
    {
	Properties applProps = new Properties();
	try {
	    FileInputStream fis =
		new FileInputStream("conf/test/application.cfg");
	    applProps.load(fis);
	    fis.close();
	} catch (IOException ioe) {
	    applicationLog.error(Utility.exception2String(ioe));
	}

	SproutPetApplication sappl = new SproutPetApplication(applProps);

	applicationLog.info("This is a test application for thre modules: JTok (depth 0), Sprout(depth 1) and Pet (depth 2).");

        // create a MoCoMan
        MoCoMan mocoman = MoCoMan.getInstance();
        String sessionId = mocoman.createSession("conf/test/sproutpetapp.cfg");
        applicationLog.info("Session " + sessionId + " created.");

        // invoke analysis
        String doc = "The manager in New York who evaluated his programmers is more competent than the one who had consultants evaluate them.";
        //doc = Utility.readStringFromFile("log/test.txt", "iso-8859-1");

        String annCollId =
            mocoman.createAnnotationCollection(sessionId);  // TODO : XML quote doc !!!! (&amp; &quot; &lt; &gt; &apos;)

        String xmldoc = "<?xml version=\"1.0\"?><text>" +
            Metadata.generateXml(sessionId, annCollId, "0.0", "SproutPetApp", "", false, applProps) + doc + "</text>";

        String annotationId =
            mocoman.createAnnotation(sessionId, annCollId, doc, "en", "rawtext");

        annotationId =
            mocoman.createAnnotation(sessionId, annCollId, xmldoc, "en", "xmltext");

        String output =
            mocoman.analyse(sessionId, annCollId, "xmltext", 0, 0, 100);

        // log output
        applicationLog.debug(output);

        //Utility.writeStringToFile("log/result.xml", output );

        // this also terminates all modules.
        mocoman.shutdown();

        //XMLDBAnnotationDatabase xdb = new XMLDBAnnotationDatabase("xmldb:xindice://clavinova:8080/db/");
        //applicationLog.info(xdb.getAnnotationAsString(sessionId, annCollId, "result"));

        applicationLog.info("Sprout test application finished");

    }
}
