/*
 * TestApplication.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.hog.test;

import java.io.*;
import java.util.Properties;
import org.apache.xmlrpc.*;

import de.dfki.lt.hog.Application;
import de.dfki.lt.hog.Metadata;
import de.dfki.lt.hog.MoCoMan;
import de.dfki.lt.hog.TransformationService;
import de.dfki.lt.hog.util.Utility;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;
/**
 * Test for Mocoman application
 *
 * @version 2004-01-08
 * @author <a href="mailto:ulrich.schaefer@dfki.de">Ulrich Schaefer</a>
 */
public class TestApplication
    extends Application
{

    public TestApplication(Properties applProps)
    {
	try {
            super.init(applProps);
	} catch (IOException ioe) {
	    applicationLog.error(Utility.exception2String(ioe));
	}
    }

    public static void main(String[] args)
	throws java.net.MalformedURLException,
	       XmlRpcException,
	       FileNotFoundException
    {
	Properties applProps = new Properties();
	try {
	    FileInputStream fis =
		new FileInputStream("conf/test/application.cfg");
	    applProps.load(fis);
	    fis.close();
	} catch (IOException ioe) {
	    applicationLog.error(Utility.exception2String(ioe));
	}

	TestApplication sappl = new TestApplication(applProps);

	applicationLog.info("This is a test application.");

        if (args.length==0) {
            System.out.println("usage: TestApplication filename [ languagecode [ encoding ] ]");
            System.exit(1);
        }
        String lang = "en";
        if (args.length>1) lang = args[1];
        String encoding = "ISO-8859-1";
        if (lang.equals("ja")) encoding = "EUC-JP";
        if (args.length>2) encoding = args[2];
        String lpref = "";
        if (!lang.equals("en")) lpref = lang + "/";

        // create a MoCoMan
        MoCoMan mocoman = MoCoMan.getInstance();
        String sessionId = mocoman.createSession("conf/" + lpref +  "xmlrpcsession.cfg");
        applicationLog.info("Session " + sessionId + " created.");

        // invoke analysis
        //String doc = "The manager in New York who evaluated his programmers is more competent than the one who had consultants evaluate them.";
        String doc = Utility.readStringFromFile(args[0], encoding);

        String annCollId =
            mocoman.createAnnotationCollection(sessionId);  // TODO : XML quote doc !!!! (&amp; &quot; &lt; &gt; &apos;)

        String xmldoc = "<?xml version=\"1.0\" encoding=\"" + encoding + "\"?><text>" +
            Metadata.generateXml(sessionId, annCollId, "0.0", "TestApp", "", false, applProps) + doc + "</text>";

        String annotationId =
            mocoman.createAnnotation(sessionId, annCollId, doc, lang, "rawtext");

        annotationId =
            mocoman.createAnnotation(sessionId, annCollId, xmldoc, lang, "xmltext");

        String output =
            mocoman.analyse(sessionId, annCollId, "xmltext", 0, 0, 100);

        if (lang.equals("ja")) {
            String pic =
                mocoman.getAnnotation(sessionId, annCollId, "ChaSen");

            Utility.writeStringToFile("log/pic.xml", pic );
        }

        // log output
        applicationLog.debug(output);

        Utility.writeStringToFile("log/result.xml", output );
        TransformationService ts = new TransformationService(mocoman);
        Transformer myTransformer=null;
        try {
        myTransformer = ts.createTransformer("xsl/html/rmrs2html.xsl");
        } catch (TransformerConfigurationException tce) {
            applicationLog.error(Utility.exception2String(tce));
        }
        myTransformer.setParameter("encoding", encoding);
        String result=null;
        try {
        result = ts.transform(myTransformer, output);
        } catch (TransformerException te) {
            applicationLog.error(Utility.exception2String(te));
        }
        Utility.writeStringToFile("log/result.html", result );

        // this also terminates all modules.
        mocoman.shutdown();

        //XMLDBAnnotationDatabase xdb = new XMLDBAnnotationDatabase("xmldb:xindice://clavinova:8080/db/");
        //applicationLog.info(xdb.getAnnotationAsString(sessionId, annCollId, "result"));

        applicationLog.info("Test application finished");

    }
}
