/*
 * TestApplication.java
 *
 * (C) 2007  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.hog.test;

import static org.junit.Assert.*;
import org.junit.Before;
import org.junit.Test;

import java.util.*;
import de.dfki.lt.hogclient.HogXmlRpcClient;

/**
 * JUnit tests for de.dfki.lt.hogclient.HogXmlRpcClient.
 * based on de.dfki.lt.hogclient.cmdline.TestApplication by Oezgur Demir
 *
 * @author <a href="mailto:dcontag@dfki.de">Daniel Contag</a>
 * @version
 */
 
public class TestXmlRpc {
  
  String hostname = "localhost";
  int port = 8411;
  HogXmlRpcClient client = new HogXmlRpcClient(hostname, port);
  
  @Test public void TestXmlRpcNeu () {
    // Server should answer "Hello"
    String answer = client.sayHello();
    assertEquals("Hello", answer);
    
    // Test creating a session with default values
    String sid = client.createSession();
    assertNotSame("", sid);
    
    // Test createSession(String, String)
    sid = client.createSession("en", "xmlrpcsession.cfg");
    assertNotSame("", sid);
    
    // Test createAnnotationCollection()
    String cid = client.createCollection();
    assertNotSame("", cid);
    
    // Test createAnnotationCollection(String)
    cid = client.createCollection(sid);
    assertNotSame("", cid);
    
    // Test createAnnotation(String) using defaults
    String aid = client.createAnnotation("This is a test.");
    assertNotSame("", aid);
    
    // Test createAnnotation(String, String, String, String, String)
    aid = client.createAnnotation(sid, cid, "This is a test", "en",
                                client.getEncoding());
    assertNotSame("", aid);
    
    // Test createAnnotation(String, String, String, String, String, String)
    aid = client.createAnnotation(sid, cid, "This is a test.", "en",
                                "fooClient", client.getEncoding());
    assertNotSame("", aid);
      
    // Test getAnnotation()
    String annotation = client.getAnnotation();
    assertNotSame("", annotation);
    
    // Test getAnnotation(String, String, String)
    annotation = client.getAnnotation(sid, cid, "xmltext");
    assertNotSame("", annotation);
    
    // Test getAnnotationAsHTML()
    annotation = client.getAnnotationAsHTML();
    assertNotSame("", annotation);
    
    // Test getAnnotationAsHTML(String, String, String)
    annotation = client.getAnnotationAsHTML("session2", "collection2",
                                            "xmltext");
    // TODO
    // schlägt immer fehl!
    // assertNotSame("", annotation);
    
    // Test analyze()
    String result = client.analyze();
    
    // schlägt immer fehl!
    // assertNotSame("", annotation);
    // ?? result wird verwendet, aber dann annotation verglichen
    // siehe code hier unten:
    /*
    if (annotation.equals("")) {
      System.out.println("  Analysis failed.");
    }
    else {
      System.out.println("  Retrieved: " + result);
    }
    */
    
    
    // Test analyze(String, String, String)
    result = client.analyze("session2", "collection2", "xmltext");
    // schlägt immer fehl
    // assertNotSame("", annotation);
    // ?? dasselbe wie oben
    /*
    if (annotation.equals("")) {
      System.out.println("  Analysis failed.");
    }
    else {
      System.out.println("  Retrieved: " + result);
    }
    */
    
    // Test analyze(String, String, String, int)
    result = client.analyze("session2", "collection2", "xmltext", 10);
    // schlägt immer fehl
    // assertNotSame("", annotation);
    // ?? Dasselbe wie oben
    /*
    if (annotation.equals("")) {
      System.out.println("  Analysis failed.");
    }
    else {
      System.out.println("  Retrieved: " + result);
    }
    */
    
    
    // Test error handling of analyze(String, String, String)
    result = client.analyze("session999_", "collection999_", "xmltext");
    assertEquals("", annotation);
    // ?? wie oben?
    /*
    if (annotation.equals("")) {
      System.out.println("  Analysis failed.");
    }
    else {
      System.out.println("  Retrieved: " + result);
    }
    */
    
    // Test error handling of getAnnotationAsHTML(String, String, String)
    annotation = client.getAnnotationAsHTML("session999_", "collection999_",
                                            "xmltext_");
    assertEquals("", annotation);
    
    // Test error handling of getAnnotation(String, String, String)
    annotation = client.getAnnotation("session999_", "collection999_",
                                      "rawtext_");
    assertEquals("", annotation);
    
    // Test error handling of
    //   createAnnotation(String, String, String, String, String)
    aid = client.createAnnotation("session999_", "collection999_",
                                  "This is a test", "en", client.getEncoding());
    assertEquals("", aid);
    
    // Test error handling of createAnnotationCollection(String)
    cid = client.createCollection("session999_");
    assertEquals("", cid);
    
    // Test error handling of deleteSession(String)
    String cr = client.deleteSession("session999_");
    assertNotSame("ok", cr);
    
    // Test server status
    String status = client.getServerStatusString();
    assertNotSame("", status);
    
    // Test shutdown()
    String s = client.shutdown();
    assertEquals("ok", s);
  }

  
}
