/*
 * Shutdown.java
 *
 * (C) 2006  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.hogclient.cmdline;

import java.util.*;
import de.dfki.lt.hogclient.HogXmlRpcClient;

/**
 * Shutdown Mocoman via commandline (e.g., from ant)
 *
 * @author <a href=mailto:uschaefer@dfki.de>Ulrich Schaefer</a>
 * @version $Id: Shutdown.java,v 1.1 2006/06/08 14:02:00 uschaefer Exp $
 */

public class Shutdown {

  public static void main(String args[]) {

    HogXmlRpcClient client;
    String hostname = args[0];
    int port = Integer.parseInt(args[1]);

    System.out.println("Connecting to " + hostname + ":" + port);

    try {
      client = new HogXmlRpcClient(hostname, port);
    }
    catch (ExceptionInInitializerError ex0) {
      System.err.println("HogXmlRpcClient.main: "
                         + "Could not initialize client connecting to "
                         + hostname + ":" + port);
      System.err.println("HogXmlRpcClient.main: "
                         + "Server might be down or unreachable.");
      System.err.print("HogXmlRpcClient.main: StackTrace: ");
      ex0.printStackTrace();
      return;
    }

    System.out.println("  Created new client: " + client);
    System.out.println("  Client is connected to " + client.getServerURL());
    System.out.println("Saying 'Hello' to server");
    String answer = client.sayHello();
    System.out.println("  Server said: " + answer);


    // Print the server status
    System.out.print("Retrieving server status");
    String status = client.getServerStatusString();
    if (!status.equals("")) {
      System.out.println(status);
    }
    else {
      System.out.println("  Retrieval of server status failed");
    }

    // Test shutdown()
    System.out.print("Shutting the server down");
    if (client.shutdown().equals("ok")) {
      System.out.println("  Server shut down");
    }
    else {
      System.out.println("  Server shut down failed");
    }
  }
}
