/*
 * TestApplication.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.hogclient.cmdline;

import java.util.*;
import de.dfki.lt.hogclient.HogXmlRpcClient;

/**
 * Test application for de.dfki.lt.hogclient.HogXmlRpcClient.
 *
 * @author <a href=mailto:oeze@dfki.de>Oezguer Demir</a>
 * @version $Id: TestApplication.java,v 1.5 2006/06/08 15:44:40 uschaefer Exp $
 */

public class TestApplication {

  public TestApplication() {
    // (for static use)
  }

  // MAIN

  public static void main(String args[]) {

    HogXmlRpcClient client;
    String hostname = args[0];
    int port = Integer.parseInt(args[1]);

    System.out.println("Connecting to " + hostname + ":" + port);
    System.out.print(" (Should succeed)");

    try {
      client = new HogXmlRpcClient(hostname, port);
    }
    catch (ExceptionInInitializerError ex0) {
      System.err.println("HogXmlRpcClient.main: "
                         + "Could not initialize client connecting to "
                         + hostname + ":" + port);
      System.err.println("HogXmlRpcClient.main: "
                         + "Server might be down or unreachable.");
      System.err.print("HogXmlRpcClient.main: StackTrace: ");
      ex0.printStackTrace();
      return;
    }

    System.out.println("  Created new client: " + client);
    System.out.println("  Client is connected to " + client.getServerURL());
    System.out.println("Saying 'Hello' to server");
    String answer = client.sayHello();
    System.out.println("  Server said: " + answer);

    // Test createSession()
    System.out.print("Initializing session (default values)");
    System.out.println(" (Should succeed)");
    System.out.println("  Current values: (" + client.getLanguage() + ", "
                       + client.getConfiguration() + ")");
    String sid = client.createSession();
    if (sid.equals("")) {
      System.out.println("  Initialization failed");
    }
    else {
      System.out.println("  Initialized " + sid);
    }

    // Test createSession(String, String)
    System.out.print("Initializing session (en, xmlrpcsession.cfg)");
    System.out.println(" (Should succeed)");
    sid = client.createSession("en", "xmlrpcsession.cfg");
    if (sid.equals("")) {
      System.out.println("  Initialization failed");
    }
    else {
      System.out.println("  Initialized " + sid);
    }

    // Test createAnnotationCollection()
    System.out.print("Creating new annotation collection");
    System.out.println(" (Should succeed)");
    String cid = client.createCollection();
    if (cid.equals("")) {
      System.out.println("  Creation failed.");
    }
    else {
      System.out.println("  Created " + cid);
    }

    // Test createAnnotationCollection(String)
    System.out.print("Creating new annotation collection");
    System.out.println(" (Should succeed)");
    cid = client.createCollection(sid);
    if (cid.equals("")) {
      System.out.println("  Creation failed.");
    }
    else {
      System.out.println("  Created " + cid);
    }

    // Test createAnnotation(String)
    System.out.print("Creating new annotation collection using defaults");
    System.out.println(" (Should succeed)");
    String aid = client.createAnnotation("This is a test.");
    if (aid.equals("")) {
      System.out.println("  Creation failed.");
    }
    else {
      System.out.println("  Created " + aid);
    }

    // Test createAnnotation(String, String, String, String, String)
    System.out.print("Creating new annotation collection using custom");
    System.out.println(" (Should succeed)");
    aid = client.createAnnotation(sid, cid, "This is a test", "en",
                                  client.getEncoding());
    if (aid.equals("")) {
      System.out.println("  Creation failed.");
    }
    else {
      System.out.println("  Created " + aid);
    }

    // Test createAnnotation(String, String, String, String, String, String)
    System.out.print(
        "Creating new annotation collection using custom + custom");
    System.out.println(" (Should succeed)");
    aid = client.createAnnotation(sid, cid, "This is a test.", "en",
                                  "fooClient", client.getEncoding());
    if (aid.equals("")) {
      System.out.println("  Creation failed.");
    }
    else {
      System.out.println("  Created " + aid);
    }

    // Test getAnnotation()
    System.out.print(
        "Retrieving annotation");
    System.out.println(" (Should succeed)");
    String annotation = client.getAnnotation();
    if (annotation.equals("")) {
      System.out.println("  Retrieval failed.");
    }
    else {
      System.out.println("  Retrieved: " + annotation);
    }

    // Test getAnnotation(String, String, String)
    System.out.print(
        "Retrieving annotation <sid, cid, aid>");
    System.out.println(" (Should succeed)");
    annotation = client.getAnnotation(sid, cid, "xmltext");
    if (annotation.equals("")) {
      System.out.println("  Retrieval failed.");
    }
    else {
      System.out.println("  Retrieved: " + annotation);
    }

    // Test getAnnotationAsHTML()
    System.out.print(
        "Retrieving annotation as HTML");
    System.out.println(" (Should succeed)");
    annotation = client.getAnnotationAsHTML();
    if (annotation.equals("")) {
      System.out.println("  Retrieval failed.");
    }
    else {
      System.out.println("  Retrieved: " + annotation);
    }

    // Test getAnnotationAsHTML(String, String, String)
    System.out.print(
        "Retrieving annotation <sid, cid, aid> as HTML");
    System.out.println(" (Should succeed)");
    annotation = client.getAnnotationAsHTML("session2", "collection2",
                                            "xmltext");
    if (annotation.equals("")) {
      System.out.println("  Retrieval failed.");
    }
    else {
      System.out.println("  Retrieved: " + annotation);
    }

    // Test analyze()
    System.out.print(
        "Analyzing annotation");
    System.out.println(" (Should succeed)");
    String result = client.analyze();
    if (annotation.equals("")) {
      System.out.println("  Analysis failed.");
    }
    else {
      System.out.println("  Retrieved: " + result);
    }

    // Test analyze(String, String, String)
    System.out.print(
        "Analyzing annotation <sid, cid, aid>");
    System.out.println(" (Should succeed)");
    result = client.analyze("session2", "collection2", "xmltext");
    if (annotation.equals("")) {
      System.out.println("  Analysis failed.");
    }
    else {
      System.out.println("  Retrieved: " + result);
    }

    // Test analyze(String, String, String, int)
    System.out.print(
        "Analyzing annotation <sid, cid, aid> + custom depth");
    System.out.println(" (Should succeed)");
    result = client.analyze("session2", "collection2", "xmltext", 10);
    if (annotation.equals("")) {
      System.out.println("  Analysis failed.");
    }
    else {
      System.out.println("  Retrieved: " + result);
    }

    // Test error handling of analyze(String, String, String)
    System.out.print(
        "Analyzing annotation <sid, cid, aid>");
    System.out.println(" (Should fail)");
    result = client.analyze("session999_", "collection999_", "xmltext");
    if (annotation.equals("")) {
      System.out.println("  Analysis failed.");
    }
    else {
      System.out.println("  Retrieved: " + result);
    }

    // Test error handling of getAnnotationAsHTML(String, String, String)
    System.out.print(
        "Retrieving annotation <sid, cid, aid> as HTML");
    System.out.println(" (Should fail)");
    annotation = client.getAnnotationAsHTML("session999_", "collection999_",
                                            "xmltext_");
    if (annotation.equals("")) {
      System.out.println("  Retrieval failed.");
    }
    else {
      System.out.println("  Retrieved: " + annotation);
    }

    // Test error handling of getAnnotation(String, String, String)
    System.out.print(
        "Retrieving annotation <sid, cid, aid>");
    System.out.println(" (Should fail)");
    annotation = client.getAnnotation("session999_", "collection999_",
                                      "rawtext_");
    if (annotation.equals("")) {
      System.out.println("  Retrieval failed.");
    }
    else {
      System.out.println("  Retrieved: " + annotation);
    }

    // Test error handling of
    //   createAnnotation(String, String, String, String, String)
    System.out.print("Creating new annotation collection using custom");
    System.out.println(" (Should fail)");
    aid = client.createAnnotation("session999_", "collection999_",
                                  "This is a test", "en", client.getEncoding());
    if (aid.equals("")) {
      System.out.println("  Creation failed.");
    }
    else {
      System.out.println("  Created " + aid);
    }

    // Test error handling of createAnnotationCollection(String)
    System.out.print("Creating new annotation collection in session999_");
    System.out.println(" (Should fail)");
    cid = client.createCollection("session999_");
    if (cid.equals("")) {
      System.out.println("  Creation failed.");
    }
    else {
      System.out.println("  Created " + cid);
    }

    // Test error handling of deleteSession(String)
    System.out.print("Closing session999_");
    System.out.println(" (Should fail)");
    if (client.deleteSession("session999_").equals("ok")) {
      System.out.println("  Closed session999_");
    }
    else {
      System.out.println("  Closing session999_ failed");
    }

    // Print all exception stack traces
    System.out.println("==================================================");
    System.out.println("Printing client's exceptions stack (don't panic): ");
    System.out.println("==================================================");
    Stack exceptions = client.getAllExceptionCauses();
    while (!exceptions.isEmpty()) {
      Throwable ex = (Throwable) exceptions.pop();
      System.out.println("------------------Message--------------------");
      System.out.println(ex.getMessage());
      System.out.println("------------------StackTrace-----------------");
      ex.printStackTrace();
      System.out.println("---------------------------------------------");
    }

    // Print the server status
    System.out.print("Retrieving server status");
    System.out.println(" (Should succeed)");
    String status = client.getServerStatusString();
    if (!status.equals("")) {
      System.out.println(status);
    }
    else {
      System.out.println("  Retrieval of server status failed");
    }

    // Test shutdown()
    System.out.print("Shutting the server down");
    System.out.println(" (Should succeed)");
    if (client.shutdown().equals("ok")) {
      System.out.println("  Server shut down");
    }
    else {
      System.out.println("  Server shut down failed");
    }
  }
}
