/*
 * IMediator.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.sdl;

/**
 * this is the interface which every mediator must fulfill!!
 *
 * @see Mediators
 * @see IModule
 * @see Modules
 *
 * @author (C) Hans-Ulrich Krieger
 * @since JDK 1.3
 * @version 1.1.2, Mon May 10 15:08:07 CEST 2004
 */
public interface IMediator {

  /**
   * given two modules, sequence mediates the input to module2,
   * computed from the output of module1;
   * @see Mediators#seq for a possible implementation
   */
  public Object seq(IModule module1, IModule module2);

  /**
   * given an array of modules, parallelism constructs the final
   * result of a (quasi-)parallel independent computation of an
   * arbitrary number of modules, from the output of these modules;
   * @see Mediators#par for a possible implementation
   */
  public Object par(IModule[] modules);

  /**
   * given a single module, fixpoint construct a final result
   * by feeding the module with its own output; this computation,
   * of course, must not terminate;
   * @see Mediators#fix for a possible implementation
   */
  public Object fix(IModule module);

}
