/*
 * SproutModules.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.sdl.sprout;

import de.dfki.lt.sdl.*;
import de.dfki.lt.sprout.runtime.interpreter.SproutInterpreter;


/**
 * this abstract class extends Modules for the purpose of interfacing
 * SProUT with SDL;
 * SproutModules wraps an InterpreterTest object (= empty SProUT) and
 * initializes this interpreter with a SProUT config file, using the
 * init() method
 *
 * @author (C) Hans-Ulrich Krieger
 * @since JDK 1.3
 * @version 1.1, Tue Jul 20 17:49:50 CEST 2004
 */
public abstract class SproutModules extends Modules {

  /**
   * holds a reference to the SProUT interpreter
   */
  protected SproutInterpreter sprout;
  protected String annotationName;

  /**
   * creates an empty interpreter
   */
  public SproutModules() {
    super();
    this.sprout = new SproutInterpreter();
  }

  /**
   * the init argument given to modules which inherit from SproutModules
   * is interpreted to be the SProUT config filename
   */
  public void init(String[] initArgs) {
    boolean status = this.sprout.initialize(initArgs[0]);
    this.annotationName = initArgs[1];
    if (status == false) {
      System.err.println("can not initialize module (" + initArgs[0] + ")");
      System.exit(1);
    }
  }

}
