/*
 * SproutModulesTextDomEncapsulated.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.sdl.sprout;

import java.util.ArrayList;
import org.w3c.dom.Node;

import de.dfki.lt.hog.container.AnnotationEncapsulator;
import javax.xml.parsers.ParserConfigurationException;
import de.dfki.lt.sprout.runtime.XmlTransformer;
import org.apache.log4j.Logger;

/**
 * this class extends SproutModules by defining the run() method
 * and assuming that the input value is a text string that has to be analyzed
 *
 * @author (C) Hans-Ulrich Krieger, Ulrich Schaefer (encapsulation)
 * @version $Revision: 1.1 $
 * $Id: SproutModulesTextDomEncapsulated.java,v 1.1 2005/07/01 11:50:24 uschaefer Exp $
 */
public class SproutModulesTextDomEncapsulated extends SproutModules {

  private static Logger log = Logger.getLogger(SproutModulesTextDomEncapsulated.class);

  private boolean bDuplicateCorefs = false;
  private String sOutputPath = null;

  /**
   * nothing to do here
   */
  public SproutModulesTextDomEncapsulated() {
    super();
  }

  /**
   * the init argument given to modules which inherit from SproutModules
   * is interpreted to be the SProUT config filename
   */
  public void init(String[] initArgs) {
    boolean status = this.sprout.initialize(initArgs[0]);
    this.annotationName = initArgs[1];
    if (status == false) {
      log.error("can not initialize module (" + initArgs[0] + ")");
    }
    if (initArgs.length>2)
        bDuplicateCorefs = initArgs[2].trim().equalsIgnoreCase("true");
    if (initArgs.length>3)
        sOutputPath = initArgs[3].trim();
  }


  /**
   * calls SProUT on the input text;
   * output is an instance of class MatchWriterImpl
   */
  public Object run(Object input) {
    // clear module and set input field
    this.clear();
    this.setInput(input);
    // input is assumed to be of type String in this class, encoding the
    // text which we want to analyze
    AnnotationEncapsulator aeIn = (AnnotationEncapsulator) input;
    String inputText = (String) aeIn.getAnnotation();
    // set input of SProUT
    this.sprout.setNewInputTextfromString(inputText);
    // encodes a sequence of array lists of match infos, each equipped
    // with positional information and a list of alternative TFSs
    long t1 = System.currentTimeMillis();
    ArrayList output = this.sprout.RunInterpreter();
    Node dom = null;
    try {
        dom = XmlTransformer.buildResultDOM(output, bDuplicateCorefs, sOutputPath);
    } catch (ParserConfigurationException pce) {
        log.error(pce);
    }
    log.info("SProUT interpreter runtime (" + this.annotationName + "): " + (System.currentTimeMillis() - t1) + " milliseconds");
    // send computed annotation to MoCoMan
    //    if (!this.annotationName.equals(""))
    //    	aeIn.getMoCoMan().createAnnotation(aeIn.getSessionID(), aeIn.getAnnotationCollectionID(),
    //                                       xmlString, aeIn.getLanguage(), this.annotationName);
    // log.debug(xmlString);
    AnnotationEncapsulator aeOut = aeIn.clone(dom);
    // set output field of this module
    this.setOutput(aeOut);
    return aeOut;
  }

}
