/*
 * SproutModulesXmlXml.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.sdl.sprout;

import java.util.ArrayList;
import java.io.*;
import de.dfki.lt.sprout.runtime.XmlTransformer;

/**
 * this abstract class extends SproutModules by defining the run() method and
 * assuming that the input value is an XML string that has to be analyzed
 *
 * @author (C) Hans-Ulrich Krieger
 * @since JDK 1.3
 * @version 1.0.1, Thu May 13 15:47:32 CEST 2004
 */
public class SproutModulesXmlXml extends SproutModules {

  private boolean bDuplicateCorefs = false;
  private String sOutputPath = null;

  /**
   * nothing to do here
   */
  public SproutModulesXmlXml() {
    super();
  }

  /**
   * the init argument given to modules which inherit from SproutModules
   * is interpreted to be the SProUT config filename
   */
  public void init(String[] initArgs) {
    boolean status = this.sprout.initialize(initArgs[0]);
    this.annotationName = initArgs[1];
    if (status == false) {
      System.err.println("can not initialize module (" + initArgs[0] + ")");
    }
    if (initArgs.length>2)
        bDuplicateCorefs = initArgs[2].trim().equalsIgnoreCase("true");
    if (initArgs.length>3)
        sOutputPath = initArgs[3].trim();
  }

  /**
   * calls SProUT on the input XML string;
   * output is the printed representation (a string) of an instance of class
   * MatchWriterImpl
   */
  public Object run(Object input) {
    // clear module and set input field
    this.clear();
    this.setInput(input);
    // input is assumed to be of type String in this class, encoding the
    // filename
    String xmlInString = (String)input;
    // set input for SProUT
    // set standard input text stream to empty string
    //this.sprout.setNewInputTextfromString(" ");  // or better ""
    this.sprout.setNewXMLInputTextfromString(xmlInString);
    // encodes a sequence of array lists of match infos, each equipped
    // with positional information and a list of alternative TFSs
    ArrayList output = this.sprout.RunInterpreter();
    StringWriter stw = new StringWriter();
    PrintWriter pw = new PrintWriter(stw);
    XmlTransformer.printResult(pw, output, bDuplicateCorefs, sOutputPath);
    pw.flush();
    pw.close();
    String xmlOutString = stw.toString();
    // set output field of this module
    this.setOutput(xmlOutString);
    return xmlOutString;
  }

}
