/*
 * MaxMediator.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.sdl.test;

import de.dfki.lt.sdl.*;

/**
 * this class is a subclass of Mediator and its purpose is solely to
 * define an additional specialized parallelism mediator for signature
 * <Integer, <Randomize, Randomize>>
 *
 * note that this is the first time that I extend a class which is not part of
 * the package of the superclass:
 *   o superclass Mediator, package de.dfki.lt.sdl
 *   o subclass MaxMediator, package de.dfki.lt.sdl.test
 *
 * @author (C) Hans-Ulrich Krieger
 * @since JDK 1.3
 * @version 1.0.1, Tue Nov 12 14:49:51 CET 2002
 */
public class MaxMediator extends Mediators {

  /**
   * only calls the super constructor, viz., Mediator()
   */
  public MaxMediator() {
    super();
  }

  /**
   * compute the maximum from the output of an arbitrary number of  modules
   * and wrap this number in an Integer object;
   * I assume that atLEAST one module is given to par(), but I do NOT check
   * for this!!
   */
  public Object par(IModule[] modules) {
    int[] outs = new int[modules.length];
    int i;
    for (i = 0; i < modules.length; i++)
      outs[i] = ((Integer)(modules[i].getOutput())).intValue();
    // assume at least one module
    int max = outs[0];
    for (i = 0; i < outs.length; i++)
      if (outs[i] > max)
	max = outs[i];
    System.out.print("<");
    for (i = 0; i < outs.length; i++){
      System.out.print(outs[i]);
      if ((i + 1) != outs.length)
	System.out.print(", ");
    }
    System.out.print(">");
    System.out.println(" --> MaxMediator --> " + max);
    return new Integer(max);
  }

}
