/*
 * Randomize.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.sdl.test;

import java.util.*;
import de.dfki.lt.sdl.*;

/**
 * purpose of this class:  testing SDL;
 * functionality inherited from abstract class Modules which implements the
 * interface IModule;
 * run(_) is the only method which I MUST define
 *
 * @author (C) Hans-Ulrich Krieger
 * @since JDK 1.3
 * @version 1.0.3, Thu Dec  5 15:17:41 CET 2002
 */
public class Randomize extends Modules implements Runnable {

  /**
   * Randomize is an int random number generator
   */
  public Randomize() {
    super();
  }

  /**
   * since run(_) is defined to take an Object argument, I must wrap
   * the input integer into an Integer object; this int is used to generate int
   * random numbers between 0 and int - 1
   */
  public Object run(Object obj) {
    if (obj instanceof Integer) {
      int i = ((Integer)obj).intValue();
      if (i == 0) {
	System.out.println("0 --> Randomize --> 0");
	return new Integer(0);
      }
      else {
	Random rand = new Random();
	// random numbers between 0 and (i - 1) incl.
	int res = Math.abs(rand.nextInt()) % i;
	System.out.println(i + " --> Randomize --> " + res);
	return new Integer(res);
      }
    }
    else
      throw new ModuleRunError("Module Randomize has been given a wrong class");
  }

  /**
   *
   */
  public void run() {
    setOutput(run (getInput()));
  }

  /**
   * I overwrite the (useless) init() method from class Modules;
   * however, it does not perform any useful things, but only prints
   * the content of initArgs[0] to System.out
   */
  public void init(String[] initArgs) {
    System.out.println(initArgs[0]);
  }

}

