/*
 * Test.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.sdl.test;

import de.dfki.lt.sdl.*;

/**
 * purpose of this class:  testing SDL;
 * this class tests the following system specification:
 *          ------
 *     --> | rnd1 |       ----        ----         -----
 *  --|--> | rnd2 | M--> | +1 | M--> | +1 | M--> | 542 | <--M-->
 *     --> | rnd3 |       ----        ----         -----
 *          ------
 * rnd is realized by class Randomize, +1 by class Increment, 542 by class
 * Incr5UpTo42, and concurrency, sequence and fixpoint iteration through
 * ModOps with the help of the corresponding Mediator methods
 *
 * @author (C) Hans-Ulrich Krieger
 * @since JDK 1.3
 * @version 1.1.3, Thu Dec  5 15:18:12 CET 2002
 */
public class Test {

  /**
   * realizes the system behavior from the class header:
   *   (rand1 | rand2 | rand3) inc1 inc2 (i5ut42 *)
   * I do always generate a new instance for every module to avoid side
   * effects
   */
  public Test(int i) {
    IMediator mmed = new MaxMediator();
    Randomize rand1 = new Randomize();
    rand1.init(new String[] {"\n***this is the init string to first randomizer***\n"});
    Randomize rand2 = new Randomize();
    Randomize rand3 = new Randomize();
    Increment inc1 = new Increment();
    Increment inc2 = new Increment();
    Incr5UpTo42 i5ut42 = new Incr5UpTo42();
    // (rand1 | rand2 | rnd3) inc1 inc2 (i5ut42 *)
    /*
    ModOps.fix(i5ut42,
	       mmed,
	       ModOps.seq(inc1,
			  inc2,
			  mmed,
			  ModOps.par(new IModule[] {rand1, rand2, rand3},
				     mmed,
				     new Integer(i))));
				     */
  }

  /**
   * call this main method with an integer number to provide the above
   * specified behavior
   */
  public static void main(String[] args) {
    new Test(Integer.parseInt(args[0]));
  }

}
