/*
 * XsltModules.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.sdl.xslt;

import de.dfki.lt.hog.TransformationService;
import de.dfki.lt.sdl.*;
import javax.xml.transform.*;
import org.apache.log4j.Logger;

/**
 * XSLT transformation Module for HoG
 * @author Ulrich Schaefer
 * @version $Revision: 1.3 $
 * $Id: XsltModules.java,v 1.3 2005/04/26 17:37:59 oeze Exp $
 */
public abstract class XsltModules extends Modules {

    private static Logger log = Logger.getLogger(XsltModules.class);
    Transformer transformer;
    TransformationService tservice;
    String annotationName;

    /**
     * creates an empty interpreter
     */
    public XsltModules() {
        super();
    }

    /**
     * Create XSLT transformer with optional parameters, where<br>
     * initArgs[0] = stylesheet filename<br>
     * initArgs[1] = annotation name<br>
     * initArgs[2n] = parameter name n<br>
     * initArgs[2n+1] = parameter value n<br>
     */
    public void init(String[] initArgs) throws ModuleInitError {
        String stylesheetFilename = initArgs[0];
        this.annotationName = initArgs[1];
        this.tservice = new TransformationService();
        try {
            this.transformer = this.tservice.createTransformer(stylesheetFilename);
        } catch (TransformerConfigurationException tce) {
            log.error(tce);
            throw new ModuleInitError(tce.toString());
        }
        for (int i=2; i<initArgs.length; i++) {
            this.transformer.setParameter(initArgs[i], initArgs[++i]);
        }
    }

}
