/*
 * XsltModulesDomStringEncapsulated.java
 *
 * (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.sdl.xslt;

import de.dfki.lt.sdl.*;
import javax.xml.transform.*;
import org.w3c.dom.Node;
import org.apache.log4j.Logger;
import de.dfki.lt.hog.container.AnnotationEncapsulator;

/**
 * Transforms input org.w3c.dom.Node to a String using the configured stylesheet.
 * @author Ulrich Schaefer
 * @version $Revision: 1.3 $
 * $Id: XsltModulesDomStringEncapsulated.java,v 1.3 2005/07/01 11:50:25 uschaefer Exp $
 */
public class XsltModulesDomStringEncapsulated extends XsltModules {

    private static Logger log = Logger.getLogger(XsltModulesDomStringEncapsulated.class);
  /**
   * Creates an empty interpreter.
   */
  public XsltModulesDomStringEncapsulated() {
    super();
  }


  /**
   * Calls XSLT transformer on the input org.w3c.dom.Node;
   * output is an instance of class org.w3c.dom.Node
   */
  public Object run(Object input) throws ModuleRunError {
    // clear module and set input field
    this.clear();
    this.setInput(input);
    // set input of transformer to org.w3c.dom.Node
    AnnotationEncapsulator aeIn = (AnnotationEncapsulator) input;
    this.tservice.setMoCoMan(aeIn.getMoCoMan());
    Node inputDoc = (Node)aeIn.getAnnotation();
    String output;
    // transform to String
    try {
        String sid = aeIn.getSessionID();
        String acid = aeIn.getAnnotationCollectionID();
        String aid = (String)this.transformer.getParameter("aid");
        this.transformer.setParameter("uri", "hog://" + sid + "/" + acid + "/" + aid);
        // transform to String
        long t1 = System.currentTimeMillis();
        output = this.tservice.transformToString(this.transformer, inputDoc);
        log.info("XSLT transformation runtime (" + this.annotationName + "): " + (System.currentTimeMillis() - t1) + " milliseconds");
    } catch (TransformerException te) {
        log.error(te);
        throw new ModuleRunError(te.toString());
    }
        // send computed annotation to MoCoMan
    //if (!this.annotationName.equals(""))
    //	aeIn.getMoCoMan().createAnnotation(aeIn.getSessionID(), aeIn.getAnnotationCollectionID(),
    //                                   output, aeIn.getLanguage(), this.annotationName);
    log.debug(output);
    AnnotationEncapsulator aeOut = aeIn.clone(output);
    // set output field of this module
    this.setOutput(aeOut);
    return aeOut;
  }


}
