/*
 * XsltModulesStringDomEncapsulated.java
 *
 * (C) 2006  DFKI Language Technology Lab http://www.dfki.de/lt
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */

package de.dfki.lt.sdl.xslt;

import de.dfki.lt.hog.container.AnnotationEncapsulator;
import de.dfki.lt.sdl.*;
import javax.xml.transform.*;
import org.w3c.dom.Node;
import org.apache.log4j.Logger;

/**
 * Transforms input XML String to an output String using the configured stylesheet.
 * @author Ulrich Schaefer
 * @version $Revision: 1.1 $
 * $Id: XsltModulesStringDomEncapsulated.java,v 1.1 2006/05/04 19:34:44 uschaefer Exp $
 */
public class XsltModulesStringDomEncapsulated extends XsltModules {


  private static Logger log = Logger.getLogger(XsltModulesStringDomEncapsulated.class);

  /**
   * Creates an empty interpreter.
   */
  public XsltModulesStringDomEncapsulated() {
    super();
  }


  /**
   * Calls transformer on the input XML String using the configured stylesheet;
   * output is a String.
   */
  public Object run(Object input) throws ModuleRunError {
    // clear module and set input field
    this.clear();
    this.setInput(input);
    // set input of transformer (XML String)
    AnnotationEncapsulator aeIn = (AnnotationEncapsulator) input;
    this.tservice.setMoCoMan(aeIn.getMoCoMan());
    String inputString = (String)aeIn.getAnnotation();
    Node output;
    try {
        String sid = aeIn.getSessionID();
        String acid = aeIn.getAnnotationCollectionID();
        String aid = (String)this.transformer.getParameter("aid");
        this.transformer.setParameter("uri", "hog://" + sid + "/" + acid + "/" + aid);
        // transform to String
        long t1 = System.currentTimeMillis();
        output = this.tservice.transformToDom(this.transformer, inputString);
        log.info("XSLT transformation runtime (" + this.annotationName + "): " + (System.currentTimeMillis() - t1) + " milliseconds");
    } catch (TransformerException te) {
        log.error(te);
        throw new ModuleRunError(te.toString());
    }
        // send computed annotation to MoCoMan
    //if (!this.annotationName.equals(""))
    //	aeIn.getMoCoMan().createAnnotation(aeIn.getSessionID(), aeIn.getAnnotationCollectionID(),
    //                                   output, aeIn.getLanguage(), this.annotationName);
    log.debug(output);
    AnnotationEncapsulator aeOut = aeIn.clone(output);
    // set output field of this module
    this.setOutput(aeOut);
    return aeOut;
  }


}
