#! /usr/bin/env python2.3
# -*- coding: latin-1 -*-

# author: Andreas Eisele, based on work by zgr Demir
# started: Tue Jan 13 11:48:38 2004

# purpose of file:
# test my understanding of the HOG/MoCoMan architecture
# make some of the functionality accessible from the command line
# second attempt: show results from JTok


__author__ = 'Andreas Eisele <eisele@dfki.de>'
__date__ = '$Date: 2004/02/06 15:21:04 $'.strip("$")
__version__ = '$Revision: 1.2 $'.strip("$")
__project__ = 'Deepthought/HeartOfGold'

import sys
from xml.dom.minidom import parseString


def showResults(analysis, oFormat):
    dom = parseString(analysis)
    tuList = dom.getElementsByTagName("tu")
    for tu in tuList:
        for token in  tu.getElementsByTagName("Token"):
            print token.getAttribute("string")
        print
        

def main():
    # shuffle some command-line options into local variables
    from myOptParse import myOptParse
    from mocomanproxy import connect
    (o, args) = myOptParse(
        version=__version__+"("+__date__+")",
        usage="usage: %prog [options] [sentence]\n\nsend sentence(s) to JTok tokenizer and display results",
        Host="localhost",
        port=8411,
        lang="en",
        depth=0,
        session=0,   # create a new one
        oFormat="WPL|SPL",
        verbosity=0
        )
    for k in o: exec k+"=o[k]"
    del o, k


    # connect to server
    srv = connect(Host, port, verbosity=verbosity)

    # create session, annotationCollection, rawtext and xmltext annotation
    if session==0:
        sid = srv.mocoman.createSession("conf/jtokapp.cfg")
    else:
        sid = "session%s" % session
    

    input = " ".join(args)

    if input:
        sentenceIterator=[input].__iter__()
    else:
        sentenceIterator=sys.stdin

    for sentence in sentenceIterator:
        sentence=sentence.strip()

        cid = srv.mocoman.createAnnotationCollection(sid)
        aid = srv.mocoman.createAnnotation(sid, cid, sentence, lang, "rawtext")

        # not sure I understand why this is needed...
        #metadata = srv.metadata.createMetadataAsString(sid, cid, "0.0", "pythonClientApp")
        metadata = srv.metadata.generateXml(sid, cid, "0.0", "pythonClientApp", "")
        xmldoc = "<?xml version='1.0'?><text>" + metadata + sentence + "</text>"
        aid2 = srv.mocoman.createAnnotation(sid, cid, xmldoc, lang, "xmltext")

        analysis = srv.mocoman.analyse(sid, cid, aid2, 0, 0, depth)

        if verbosity>0:
            print >> sys.stderr, "\n\nAnalysis of: `` %s ''" % sentence
            print >> sys.stderr, "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
        try:
            showResults(analysis, oFormat)
        except:
            print >> sys.stderr, "exception ignored"
            raise

    # clean up session (only if it was started in this script)
    if session==0:
        srv.mocoman.closeSession(sid)
    

if __name__ == "__main__":
    main()



