#! /usr/bin/env python2.3
# -*- coding: latin-1 -*-

# author: Andreas Eisele, based on work by zgr Demir
# started: Tue Jan 13 11:48:38 2004

# purpose of file:
# test my understanding of the HOG/MoCoMan architecture
# make some of the functionality accessible from the command line
# first attempt: feed the commandline arguments into a new pet session and show results

# current inconsistency: allows to specify depth and lang, but uses English
# Pet-only session


__author__ = 'Andreas Eisele <eisele@dfki.de>'
__date__ = '$Date: 2004/02/06 15:15:09 $'.strip("$")
__version__ = '$Revision: 1.8 $'.strip("$")
__project__ = 'Deepthought/HeartOfGold'

import sys
from xml.dom.minidom import parseString


def showResults(analysis, show):    
    # show results (somewhat preliminary)
    pred2reading={}
    readingScores=[]
    try:
        dom = parseString(analysis)
        readings=int(dom.getElementsByTagName("pet")[0].getAttribute("readings"))
        metadata=dom.getElementsByTagName("metadata")[0]
        diagnosis=[node.getAttribute("value") for node in metadata.getElementsByTagName("entry") if node.getAttribute("name")=="diagnosis"]
        assert len(diagnosis)==1
        diagnosis=diagnosis[0]
        print readings, "reading(s) found"
        print "diagnosis:", diagnosis

        if readings==0:
            print analysis
        else:
            rmrsList=dom.getElementsByTagName("rmrs")
            for rmrs in rmrsList:
                # individual readings
                confidence=float(rmrs.getAttribute("confidence"))
                readingID=rmrs.getAttribute("reading")
                readingScores.append((confidence,readingID))
                print "reading",readingID, ", confidence",confidence
                predList = [d.toxml() 
                            for tag in "ep rarg ing hcons".split()
                            for d in rmrs.getElementsByTagName(tag)]
                if show=="all": print "\n".join(predList)
                for pred in predList:
                    if pred in pred2reading:
                        pred2reading[pred]+= (readingID,)
                    else:
                        pred2reading[pred] = (readingID,)

        if show=="best" and len(readingScores)>0:
            readingScores.sort()
            bestID=readingScores[-1][1]
            print "best reading:%s, score:%f" % ( bestID, readingScores[-1][0])
            bestPreds=[]
            for (p,r) in pred2reading.items():
                if bestID in r:
                    bestPreds.append(p)
            bestPreds.sort()
            print "\n".join(bestPreds)
            
        if show=="synopsis":
            print "\nSynopsis"
            if len(readingScores)>0:
                sets2preds={}
                keys=pred2reading.keys()
                keys.sort()
                for k in keys:
                    set = pred2reading[k]
                    if set in sets2preds:
                        sets2preds[set] += [k]
                    else:
                        sets2preds[set] = [k]
                print "common preds:"
                print "\n".join(sets2preds.pop(tuple([str(i) for i in range(readings)])))
                for set in sets2preds:
                    print "\nonly in %s:" % "|".join(set)
                    print "\n".join(sets2preds[set])

    except:
        print "problem interpreting output"
        print analysis
        raise


def main():
    # shuffle some command-line options into local variables
    from myOptParse import myOptParse
    from mocomanproxy import connect
    (o, args) = myOptParse(
        version=__version__+"("+__date__+")",
        usage="usage: %prog [options] [sentence]\n\nsend sentence(s) to PET parser and display RMRS(s)",
        Host="localhost",
        port=8411,
        lang="en",
        depth=2,
        session=0,   # create a new one
        readings="best|all|synopsis|none",
        verbosity=0
        )
    for k in o: exec k+"=o[k]"
    del o, k


    # connect to server
    srv = connect(Host, port, verbosity=verbosity)

    # create session, annotationCollection, rawtext and xmltext annotation
    if session==0:
        sid = srv.mocoman.createSession("conf/petapp.cfg")
    else:
        sid = "session%s" % session
    

    input = " ".join(args)

    if input:
        sentenceIterator=[input].__iter__()
    else:
        sentenceIterator=sys.stdin

    for sentence in sentenceIterator:
        sentence=sentence.strip()

        cid = srv.mocoman.createAnnotationCollection(sid)
        aid = srv.mocoman.createAnnotation(sid, cid, sentence, lang, "rawtext")

        # not sure I understand why this is needed...
        #metadata = srv.metadata.createMetadataAsString(sid, cid, "0.0", "pythonClientApp")
        metadata = srv.metadata.generateXml(sid, cid, "0.0", "pythonClientApp", "")
        xmldoc = "<?xml version='1.0'?><text>" + metadata + sentence + "</text>"
        aid2 = srv.mocoman.createAnnotation(sid, cid, xmldoc, lang, "xmltext")

        analysis = srv.mocoman.analyse(sid, cid, aid2, 0, 0, depth)

        if verbosity>0:
            print >> sys.stderr, "\n\nAnalysis of: `` %s ''" % sentence
            print >> sys.stderr, "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
        try:
            showResults(analysis, readings)
        except:
            print "exception ignored"
            raise

    # clean up session (only if it was started in this script)
    if session==0:
        srv.mocoman.closeSession(sid)
    

if __name__ == "__main__":
    main()



