# -*- coding: utf-8 -*-
import os
import xmlrpclib
from urlparse import urlunsplit

class HogClientError(Exception):
    pass


class Formats(object):
    XML = 0
    HTML = 1
    LaTeX = 2

    
class HogClient(object):
    NAME = "pythonClientApp"
    def __init__(self, server):
        self._server = server
        self._session_id = None
        self._lang = None

    def start_session(self, language, session_name="default.cfg"):
        """Starts a new annotation session with the configuration `session_name`.

        Each HoG client can only have one active session, and the session must be closed
        properly.
        """
        assert self._session_id is None
        config_file = os.path.join("conf", language, "sessions", session_name)
        self._session_id = self.mocoman.createSession(config_file)
        self._coll_id = self.mocoman.createAnnotationCollection(self._session_id)
        self._lang = language

    def close(self):
        """Closes the active session."""
        if self._session_id:
            self.mocoman.closeSession(self._session_id)
            self._session_id = None
            

    def analyze(self, text, depth = 100):
        """Analyze `text` to a specified `depth`.

        Returns the deepest annotation.
        """
        aid = self.mocoman.createInitialAnnotation(
            self._session_id, self._coll_id, xmlrpclib.Binary(text.encode('UTF-8')), self._lang, HogClient.NAME)

        result = self.mocoman.analyseBinary(self._session_id, self._coll_id, aid, 0, 0, depth)
        return (aid, result)

    def get_analysis_result(self, annotation_id, format=Formats.XML, insert_surface=True):
        """Returns the result of a specific annotator.

        Valid formats are listed in `Formats`.
        """
        result = None
        if format == Formats.XML:
            result = self.mocoman.getAnnotationBinary(self._session_id, self._coll_id, annotation_id)
            if insert_surface and result.data.find("rmrs") > -1 and result.data.find("cfrom") > -1:
                result = self.mocoman.xmlTransformerBinary(
                    self._session_id, self._coll_id, annotation_id, "xsl/rmrs/insertsurface.xsl", "no")


        elif format == Formats.HTML:
            result = self.mocoman.xmlTransformerBinary(self._session_id, self._coll_id, annotation_id,
                                                       "xsl/html/xml2html.xsl", "no")
            if insert_surface and result.data.find("rmrs") > -1 and result.data.find("cfrom") > -1:
                result = self.mocoman.xmlTransformerBinary(self._session_id, self._coll_id, annotation_id,
                                                          "xsl/html/rmrs2html.xsl", "yes")

        elif format == Formats.LaTeX:
            result = self.mocoman.xmlTransformerBinary(self._session_id, self._coll_id, annotation_id,
                                                       "xsl/latex/rmrs2latex.xsl", "yes")

        else:
            raise HogClientError, "Unknown format id %i." % (format,)
        
        return unicode(result.data, "UTF-8")    

    def get_annotation_ids(self):
        """Returns all annotation ids defined in the last analysis.

        Annotations are ordered by depth.
        """
        anns = self.mocoman.getAnnotationIds(self._session_id, self._coll_id)
        return filter(None, anns.split(","))


    server = property(lambda s: s._server)
    mocoman = property(lambda s: s._server.mocoman)


def connect(hostname, port):
    """Creates a connection to a HoG server.

    Parameters: 
      - `hostname`: the host the HoG runs on
      - `port`: the port of the HoG server

    Returns an instance of `HogClient`.
    """
    server_loc = urlunsplit(("http", "%s:%i" % (hostname, port), "", "", ""))
    xmlrpc_server = xmlrpclib.Server(server_loc)
    xmlrpc_server.mocoman.sayHello()
    return HogClient(xmlrpc_server)
