#!/usr/bin/env python

"""
This script demonstrates how to use the Deepthought
ModuleManager's server via XML-RPC.

USAGE

1. Make sure that

   de.dfki.lt.hog.MoCoManServer

   is running. Use the ant target 'mocomanserver'.

2. Run

   [python] [./]sampleRpcClient.py http://localhost:8411
"""

__author__ = 'Oezguer Demir <oeze@dfki.de>'
__date__ = '$Date: 2006/06/08 15:44:32 $'
__version__ = '$Revision: 1.21 $'
__project__ = 'Deepthought/HeartOfGold'

import xmlrpclib
import string
import sys

conf = 'xmlrpcsession.cfg'
lang = 'en'
depth = 110

srv = None
srvLoc = None
lastSID = None
lastCID = None
lastAID = None
result = None
encoding = "latin-1"

def printHelp():
    """Tell the user how to use this script."""
    print " cs     create a new session"
    print "  cc     create a new collection"
    print "  ca     create a new annotation"
    print "  ds     delete/close a session"
    print "  a      analyze an annotation"
    print "  g      get a computed annotation"
    print "  p      print result buffer"
    print "  s      print status report"
    print "  d      set analysis depth"
    print "  x      shutdown mocoman"
    print "  q      quit"

def printUsage():
    """Tell the user how to start this script and exit."""
    print 'Usage: [python] [./]sampleRpcClient.py [hostname port_number]'
    print '       default hostname: localhost'
    print '       default port_number: 8411'
    sys.exit()

def parseCmdLine():
    """Read, interpret and react to the command line arguments."""
    if len(sys.argv) == 1:
        hostname = 'localhost'
        port = 8411
    elif len(sys.argv) == 3:
        hostname = sys.argv[1]
        port = sys.argv[2]
    else:
        printUsage()
    return (hostname, port)

def connect(hostname, port):
    """Connect to a given XML-RPC server - return the Server reference."""
    global srv, srvLoc
    srvLoc = 'http://' + hostname + ':' + str(port)
    srv = xmlrpclib.Server(srvLoc)
    if not checkConnection():
        sys.exit()
    try:
        print 'Message from MoCoManServer on %s: %s'\
              %(hostname,srv.mocoman.sayHello())
        print 'Connected to server %s on port %s'\
              %(hostname, port)
    except:
        print 'Connection failed!'
        print 'Please make sure that MoCoManServer is',
        print 'running at', srvLoc
        sys.exit()
    return srv

def checkConnection():
    """Test if the connection to the server is available."""
    global srv, srvLoc
    try:
        srv.mocoman.sayHello()
        return 1
    except:
        print 'Connection failed!'
        print 'Please make sure that MoCoManServer is',
        print 'running at', srvLoc
        return 0

def createSession():
    """Create a new session and print its id."""
    global srv, lang, conf

    print 'Language [%s]: '%(lang),
    tmp = string.rstrip(sys.stdin.readline())
    if not tmp == '':
        lang = tmp
    if not checkConnection(): return
    print "Creating session ...",
    sid = srv.mocoman.createSession('conf/%s/%s'%(lang,conf))
    print "\r New session created: id = '%s'"%(sid)
    return sid

def deleteSession():
    """Delete/close a session, print whether the operation was successful."""
    global srv, lastSID
    if not checkConnection(): return
    print 'Session ID [%s]: '%(lastSID),
    sid = string.rstrip(sys.stdin.readline())
    if sid == '':
        sid = lastSID
    else:
        lastSID = sid
    try:
        if srv.mocoman.closeSession(sid):
            print "Deleted and closed session '%s'"%(sid)
        else:
            print "Server returned FALSE: ",
            print "Session '%s' was probably not deleted"\
                  %(sid)
    except:
        print "Apparently, there is no session with id '%s'"%(sid)

def createAnnoColl():
    """Create a new annotation collection and print its id."""
    global srv, lastSID
    if not checkConnection(): return
    print 'Session ID [%s]: '%(lastSID),
    sid = string.rstrip(sys.stdin.readline())
    if sid == '':
        sid = lastSID
    else:
        lastSID = sid
    try:
        cid = srv.mocoman.createAnnotationCollection(sid)
        print "New annotation collection added to '%s': '%s'"\
              %(sid, cid)
        return cid
    except:
        print "Apparently, there is no session with id '%s'"%(sid)

def createAnno():
    """Process the string by invoking .analyse() -> creates a new annotation"""
    global srv, lastSID, lastCID, lastAID, lang, encoding
    if not checkConnection(): return
    print 'Session ID [%s]: '%(lastSID),
    sid = string.rstrip(sys.stdin.readline())
    if sid == '':
        sid = lastSID
    else:
        lastSID = sid
    print 'Collection ID [%s]: '%(lastCID),
    cid = string.rstrip(sys.stdin.readline())
    if cid == '':
       cid = lastCID
    else:
        lastCID = cid
    print 'Input (text): ',
    input = unicode(string.rstrip(sys.stdin.readline()), encoding)
    try:
        aid = srv.mocoman.createInitialAnnotation(sid, cid, xmlrpclib.Binary(input.encode('utf8')), lang,
                                                  "pythonClientApp")
        print "New initial annotations created and added to '%s.%s'"\
              %(sid, cid)
        lastAID = aid
        return aid
    except:
        print "Could not create annotation in collection '%s.%s'"\
              %(sid, cid) 
        return lastAID

def getannotation():
    """Return a computed annotation."""
    global srv, lastSID, lastCID, lastAID
    if not checkConnection(): return
    print 'Session ID [%s]: '%(lastSID),
    sid = string.rstrip(sys.stdin.readline())
    if sid == '':
        sid = lastSID
    else:
        lastSID = sid
    print 'Collection ID [%s]: '%(lastCID),
    cid = string.rstrip(sys.stdin.readline())
    if cid == '':
        cid = lastCID
    else:
        lastCID = cid
    print 'Annotation ID [%s]: '%(lastAID),
    aid = string.rstrip(sys.stdin.readline())
    if aid == '':
        aid = lastAID
    else:
        lastAID = aid
    try:
        return srv.mocoman.getAnnotationBinary(sid, cid, aid)
    except:
        print "Could not get annotated data from '%s.%s.%s'"%(sid,cid,aid)

 
def analyse():
    """Analyse a given annotation."""
    global srv, lastSID, lastCID, lastAID, depth
    if not checkConnection(): return
    print 'Session ID [%s]: '%(lastSID),
    sid = string.rstrip(sys.stdin.readline())
    if sid == '':
       sid = lastSID
    else:
       lastSID = sid
    print 'Collection ID [%s]: '%(lastCID),
    cid = string.rstrip(sys.stdin.readline())
    if cid == '':
       cid = lastCID
    else:
        lastCID = cid
    print 'Annotation ID [%s]: '%(lastAID),
    aid = string.rstrip(sys.stdin.readline())
    if aid == '':
       aid = lastAID
    else:
       lastAID = aid
    return srv.mocoman.analyseBinary(sid, cid, aid, 0, 0, depth)


def setDepth():
    """Set global var depth."""
    global depth
    print 'Depth currently set to %i'%(depth)
    print 'New depth [%i]: '%(depth),
    depth = int(string.rstrip(sys.stdin.readline()))
    
def printStatus():
    """Output MoCoMan's current data status."""
    global srv, lastSID, lastCID, lastAID, depth
    if not checkConnection():
        print "----- Status (not connected) -----"
        print "Currently not connected"
        print "Last connection:"
        print "  %s"%(srv)
        return
    print "----- Status (connected) -----"
    print "Client:"
    print "  Connected to '%s'"%(srv)
    print "  Current analysis depth: %i"%(depth)
    if lastSID: print "  Last session: '%s'"%(lastSID)
    if lastCID: print "  Last annotation collection: '%s'"%(lastCID)
    if lastAID: print "  Last annotation: '%s'"%(lastAID)
    if result: print "  Resultbuffer contains %d bytes ('p' to print)"\
                     %(len(result))
    print "Server:"
    print srv.mocoman.getStatus()

def shutdown():
    srv.mocoman.shutdown()

def html():
    """Return a annotation as html."""
    global srv, lastSID, lastCID, lastAID
    if not checkConnection(): return
    print 'Session ID [%s]: '%(lastSID),
    sid = string.rstrip(sys.stdin.readline())
    if sid == '':
        sid = lastSID
    else:
        lastSID = sid
    print 'Collection ID [%s]: '%(lastCID),
    cid = string.rstrip(sys.stdin.readline())
    if cid == '':
        cid = lastCID
    else:
        lastCID = cid
    print 'Annotation ID [%s]: '%(lastAID),
    aid = string.rstrip(sys.stdin.readline())
    if aid == '':
        aid = lastAID
    else:
        lastAID = aid
    return srv.mocoman.xmlTransformerBinary(sid, cid, aid,
                                        "xsl/html/xml2html.xsl", "no")

    
def main():
    """Init and run the main loop."""
    global srv, lastSID, lastCID, lastAID, result, encoding
    hostname, port = parseCmdLine()
    srv = connect(hostname, port)
    result = None

    input = ''
    while input != 'q':
        #command = 1
        if input == 'h':
            printHelp()
        elif input == 'cs':
            lastSID = createSession()
        elif input == 'cc':
            lastCID = createAnnoColl()
        elif input == 'ca':
            lastAID = createAnno()
        elif input == 'ds':
            deleteSession()
        elif input == 'a':
            result = unicode(analyse().data,"utf-8").encode(encoding)
            if result != None:
                print "Result ready - 'p' to print"
        elif input == 'g':
            anno = getannotation()
            if anno != None:
                print unicode(anno.data,"utf-8").encode(encoding)
        elif input == 'p':
            if result: print result
            else: print '<Result buffer is empty>'
        elif input == 'd':
            setDepth()
        elif input == 's':
            printStatus()
        elif input == 'ss':
            print srv.mocoman.getStatusXml()
        elif input == 'x':
            if srv.mocoman.shutdown():
                print 'Mocoman was shutdown - server offline'
        elif input == 'html':
            print unicode(html().data,"utf-8").encode(encoding)
        elif input != '':
            print "Unknown command"
        print "Command (h for help): ",
        input = string.rstrip(sys.stdin.readline())
        

#
# MAIN 
#
if __name__=='__main__':
    main()
