#!/usr/bin/env python

"""
This script shuts down the Deepthought ModuleManager's server via XML-RPC.

 ./

USAGE

1. Make sure that

   de.dfki.lt.hog.MoCoManServer

   is running. Use the ant target 'mocomanserver'.

2. Run

   [python] [./]shutdownserver.py [hostname port]

   Example
   shutdownserver.py localhost 8411
"""

__author__ = 'Ulrich Schaefer <ulrich.schaefer@dfki.de>, Oezguer Demir <oeze@dfki.de>'
__date__ = '$Date: 2005/03/11 22:47:53 $'
__version__ = '$Revision: 1.2 $'
__project__ = 'Deepthought/HeartOfGold'

import xmlrpclib
import string
import sys
import os

srv = None
srvLoc = None

def printUsage():
    """Tell the user how to start this script and exit."""
    print 'Usage: [python] [./]shutdownserver.py [hostname port_number]'
    print '       default hostname: localhost'
    print '       default port_number: 8411'
    print 'Example: analyzeGUI.py localhost 8411'
    sys.exit()

def parseCmdLine():
    """Read, interpret and react to the command line arguments."""
    if len(sys.argv) == 1:
        hostname = 'localhost'
        port = 8411
    elif len(sys.argv) == 3:
        hostname = sys.argv[1]
        port     = sys.argv[2]
    else:
        printUsage()
    return (hostname, port)

def connect(hostname, port):
    """Connect to a given XML-RPC server - return the Server reference."""
    global srv, srvLoc
    srvLoc = 'http://' + hostname + ':' + str(port)
    srv = xmlrpclib.Server(srvLoc)
    if not checkConnection():
        sys.exit()
    try:
        print 'Connected to server %s on port %s'\
              %(hostname, port)
        print 'Message from MoCoManServer on %s: %s'\
              %(hostname,srv.mocoman.sayHello())
    except:
        print 'Connection failed!'
        print 'MoCoManServer seems to be shut down or'
        print 'running somewhere else than ', srvLoc
        sys.exit()
    return srv

def checkConnection():
    """Test if the connection to the server is available."""
    global srv, srvLoc
    try:
        srv.mocoman.sayHello()
        return 1
    except:
        print 'Connection failed!'
        print 'MoCoManServer seems to be shut down or'
        print 'running somewhere else than ' + srvLoc
        return 0

def main():
    """Init and shutdown the server."""
    global srv
    hostname, port = parseCmdLine()
    srv = connect(hostname, port)

    if srv.mocoman.shutdown():
       print 'Mocoman was shutdown - server offline'

#
# MAIN 
#
if __name__=='__main__':
    main()
