<?xml version="1.0" encoding="utf-8"?>

<!-- converts pixml to FSC -->	

<!-- 
   (C) 2007 DFKI Language Technology Lab http://www.dfki.de/lt
   author Ulrich.Schaefer@dfki.de
   derived from pic2smaf.xsl by Daniel Contag, daniel.contag@dfki.de

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  
  <xsl:output method="xml" indent="yes"/>
  <xsl:strip-space elements="*"/>
  
  <!-- generates <!DOCTYPE smaf SYSTEM 'fsc.dtd'> in output-->
  <xsl:output doctype-system="fsc.dtd"/>
  
  
  <xsl:template match = "pet-input-chart">
    <fsc version="1.0" >
      <chart id="converted from PIXML">
       
        <!-- extract text but not again from named entity entries (which have typeinfos) -->
        <!-- this is only accurate if spaces have been normalized in input text -->
        <text>
          <xsl:for-each select = "w[not(typeinfo)]">
            <xsl:value-of select = "surface/text()" />
            <xsl:if test="not(position()=last())"><xsl:text> </xsl:text></xsl:if>
          </xsl:for-each>
        </text>
            
        <xsl:variable name = "numberOfTokens"><xsl:value-of select = "count(w[not(typeinfo)])"/></xsl:variable>
        <lattice>
          <xsl:attribute name = "init">v0</xsl:attribute>
          <xsl:attribute name = "final">
            <xsl:value-of select="concat('v', $numberOfTokens)" />
          </xsl:attribute>

          <!-- iterate over w's and convert to 3 types of edges -->
          <xsl:for-each select = "w[not(typeinfo)]">
            <xsl:call-template name = "edge_token" />
          </xsl:for-each>

          <xsl:for-each select = "w[not(typeinfo)]">
            <xsl:call-template name = "edge_pos" /> 
          </xsl:for-each>

          <!-- named entities -->
          <xsl:for-each select = "w[typeinfo]">
            <xsl:call-template name = "edge_ne" />
          </xsl:for-each>
        </lattice>
      </chart>
    </fsc>
  </xsl:template>


  <!-- generates token edges with text -->
  <xsl:template name = "edge_token" >
    <edge>        
      <xsl:attribute name = "source">
        <xsl:value-of select = "concat('v', position() - 1)" />
      </xsl:attribute>

      <xsl:attribute name = "target">
        <xsl:value-of select = "concat('v', position())" />
      </xsl:attribute>

      <fs type="+tok">
        <f name="+IDS" org="list">
          <str><xsl:value-of select="concat('Tok_', position() - 1, '_', position())"/></str>
        </f>
        <f name="+FORM">
          <str><xsl:value-of select = "surface/text()" /></str>
        </f>
        <f name="+CFROM">
          <str><xsl:value-of select = "@cstart" /></str>
        </f>
        <f name="+CTO">
          <str><xsl:value-of select = "@cend + 1" /></str>
        </f>
      </fs>
    </edge>
  </xsl:template>


  <!-- generates edges with slots (slots contain pos-tags) -->
  <xsl:template name = "edge_pos" >
    <edge>
      <xsl:attribute name = "source">
        <xsl:value-of select = "concat('v', position() - 1)" />
      </xsl:attribute>
      
      <xsl:attribute name = "target">
        <xsl:value-of select = "concat('v', position())" />
      </xsl:attribute>

      <fs type="+tok">
        <f name="+IDS" org="list">
          <str><xsl:value-of select="concat('PoS_', position() - 1, '_', position())"/></str>
        </f>
        <f name="+FORM">
          <str><xsl:value-of select = "surface/text()" /></str>
        </f>
        <f name="+CFROM">
          <str><xsl:value-of select = "@cstart" /></str>
        </f>
        <f name="+CTO">
          <str><xsl:value-of select = "@cend + 1" /></str>
        </f>
        <f name="+TNT">
            <fs type="+tnt">
              <f name="+TAGS" org="list">
                <xsl:for-each select="pos">
                  <str><xsl:value-of select = "@tag" /></str>  
                </xsl:for-each>
              </f>
              <f name="+PRBS" org="list">
                <xsl:for-each select="pos">
                  <str><xsl:value-of select = "@prio" /></str>
                </xsl:for-each>
              </f>
            </fs>
          </f>
        </fs>
   
    </edge>
  </xsl:template>


  <!-- handle named entities - this is not yet adapted to FSC (still SMAF) -->
  <xsl:template name = "edge_ne" >

    <edge>
      <xsl:attribute name = "type">namedEntity</xsl:attribute>    

      <xsl:attribute name = "id">
        <xsl:value-of select = "concat('n', position())" />
      </xsl:attribute>
     
      <xsl:attribute name = "cfrom">
        <xsl:value-of select = "@cstart" />
      </xsl:attribute>
    
      <xsl:attribute name = "cto">
        <xsl:value-of select = "@cend + 1" />
      </xsl:attribute>

      <xsl:attribute name = "source">
        <xsl:value-of select = "concat('v', position() - 1)" />
      </xsl:attribute>

      <xsl:attribute name = "target">
        <xsl:value-of select = "concat('v', position())" />
      </xsl:attribute>

      <xsl:text>
        </xsl:text>

      <xsl:if test = "@prio">
        <slot>
          <xsl:attribute name = "name">weight</xsl:attribute>
          <xsl:value-of select = "@prio" />
        </slot>
      </xsl:if>

      <xsl:text>
        </xsl:text>

      <slot>    
        <xsl:attribute name = "name">surface</xsl:attribute>
        <xsl:value-of select = "surface/text()" />
      </slot>

      <xsl:text>
        </xsl:text>

      <fs>
        <xsl:attribute name = "type" >
          <xsl:value-of select = "typeinfo/stem/text()" />
        </xsl:attribute>
      </fs>

      <xsl:text>
      </xsl:text>

    </edge>

  </xsl:template>

</xsl:stylesheet>
