<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <!-- RMRS2HTML Transformation Stylesheet version 2004-03-26
       Input DTD: A subset of rmrs.dtd by Ann Copestake
       plus additional input characters in a <text> element below <rmrs>,
       e.g., <text><c>t</c><c>e</c><c>x</c><c>t</c></text>
       Output (HTML)
       Processing: e.g. xsltproc rmrs2html.xsl inputrmrs.xml > rmrs.html
       
       Developed by Thomas Kloecker, Ulrich Schaefer,
       Project QUETAL, quetal@dfki.de

       (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt

       This program is free software; you can redistribute it and/or
       modify it under the terms of the GNU Lesser General Public
       License 2.1 as published by the Free Software Foundation.

       This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
       Lesser General Public License for more details.
       
       You should have received a copy of the GNU Lesser General Public
       License along with this library; if not, write to the Free Software
       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
       
       CHANGES
       2004-05-04 uschaefer title added for HoG module names
       2004-07-07 uschaefer parameter $encoding added
       2004-07-21 uschaefer variable attributes in small font added (see $varsep)
       2005-04-21 uschaefer removed global parameter encoding (incompatible with Xalan 2.6)
       2006-05-04 uschaefer variable span with and w/o morph features corrected
-->
<!-- input DTD -->
<!-- The root element can have an arbitrary name (here rmrs-list); however, that's not -->
<!-- expressible with DTD, one would have to switch to using Schemas instead. -->
<!--
<!DOCTYPE rmrs-list [
    <!ELEMENT rmrs-list (rmrs*)>
    <!ELEMENT rmrs (text?, label, ep*, rarg*, ing*, hcons*)>
    <!ELEMENT text (c*)>
    <!ELEMENT c (#PCDATA)>
    <!ELEMENT label EMPTY>
    <!ELEMENT ep ((gpred | realpred), label, var)>
    <!ELEMENT gpred (#PCDATA)>
    <!ELEMENT realpred EMPTY>
    <!ELEMENT rarg (rargname, label, (var | constant))>
    <!ELEMENT rargname (#PCDATA)>
    <!ELEMENT var EMPTY>
    <!ELEMENT constant (#PCDATA)>
    <!ELEMENT ing (ing-a, ing-b)>
    <!ELEMENT ing-a (var)>
    <!ELEMENT ing-b (var)>
    <!ELEMENT hcons (hi, lo)>
    <!ELEMENT hi (var)>
    <!ELEMENT lo (label | var)>
    <!ATTLIST rmrs     cfrom CDATA #IMPLIED
                cto   CDATA #IMPLIED >
    <!ATTLIST ep    cfrom CDATA #REQUIRED
                  cto CDATA #REQUIRED >
    <!ATTLIST label   vid CDATA #REQUIRED >
    <!ATTLIST var    sort CDATA #REQUIRED
                  vid CDATA #REQUIRED >
    <!ATTLIST realpred lemma CDATA #REQUIRED
          pos (v | n | j | r | p | q | c | x | u) #REQUIRED
          sense CDATA #IMPLIED >
    <!ATTLIST hcons hreln (qeq | lheq | outscopes) #REQUIRED >
]>

  -->
  <xsl:output method="html" encoding="ISO-8859-1"/>
  <xsl:variable name="varsep"><xsl:text> </xsl:text></xsl:variable>
  <!-- color variables used both in the stylesheet and in the JavaScript-->
  <xsl:variable name="unmarked"><xsl:text>black</xsl:text></xsl:variable>     <!-- black letters and lines -->
  <xsl:variable name="background"><xsl:text>#FFFF99</xsl:text></xsl:variable> <!-- light - yellow background -->
  <xsl:variable name="arg0name"><xsl:text>argzero</xsl:text></xsl:variable>   <!-- name for arg0 if "arg0" is taken -->
  <xsl:variable name="lineBreakAfterThisManyPreds" select="number(10)" />
  <xsl:variable name="dirtyHack"><xsl:text disable-output-escaping = "yes">&lt;/tr&gt;&lt;tr&gt;</xsl:text></xsl:variable>
  <!-- ================                   /                           ============= -->
  <!-- top template just calls all rmrs descendants of the root (root-element can have any name) -->
  <xsl:template match="/">
    <html>
      <xsl:call-template name="head"/>
      <body onload="bodyonload();">
        <table>
          <tbody>
            <xsl:apply-templates select="//rmrs"/>
          </tbody>
        </table>
      </body>
    </html>
  </xsl:template>
  <!-- =========                                       HEAD                                   ======== -->
  <xsl:template name="head">
    <head>
      <style type="text/css">
        table.rmrsMrs {
        padding: 0.2em;
        border: solid;
        border-width: thin;
        color: <xsl:value-of select="$unmarked"/>;
        background: <xsl:value-of select="$background"/>;
        }
        body {
        margin-left: 5%; margin-right: 5%;
        font-weight: normal; font-family: helvetica, arial, sans-serif;
        color: <xsl:value-of select="$unmarked"/>;
        }
        td {
        font-weight: normal; font-family: helvetica, arial, sans-serif;
        }
        table.rmrsRelsContainer {
        }
        table.rmrsRelation {
        padding-right: 2px;
        padding-left: 1px;
        color: <xsl:value-of select="$unmarked"/>;
        border-left: solid;
        border-right: solid;
        border-left-width: thin;
        border-right-width: thin;
        border-top: none;
        border-bottom: none;
        }
        td.rmrsFeatureTop, td.rmrsFeatureIndex,
        td.rmrsFeatureRels, td.rmrsFeatureHcons {
        font-size: 10pt;
        }
        td.rmrsValueTop, td.rmrsValueIndex,
        td.rmrsValueRels, td.rmrsValueHcons {
        font-size: 10pt;
        }
        td.rmrsPredicate {
        font-size: 10pt; font-style: italic;
        }
        td.rmrsLabel {
        font-size: 10pt; font-style: normal; text-align: left;
        }
        td.rmrsValue {
        font-size: 10pt; font-style: italic; text-align: right;
        }
        span.rmrsBracket {
        color: <xsl:value-of select="$unmarked"/>;
        font-size: 16pt;
        }
      </style>
      <title><xsl:call-template name="getModuleName"/> RMRS</title>
      <!-- was link type="text/css" rel="stylesheet" href="rmrs.css"/ -->
      <xsl:call-template name="scriptGen"/>
    </head>
  </xsl:template>
  <!-- ========                                        GETMODULE                                ======= -->
  <xsl:template name="getModuleName">
    <xsl:value-of select='//metadata/conf/entry[@name="module.name"]/@value'/>
  </xsl:template>
  <!-- ========                                        SCRIPTGEN                                ======= -->
  <xsl:template name="scriptGen">
    <script language="javascript">
      <xsl:text>
        var unmarked = '</xsl:text><xsl:value-of select="$unmarked"/><xsl:text>';
        var marked = 'red';
        var background = '</xsl:text><xsl:value-of select="$background"/><xsl:text>';
        var chars = new Array();
        var rmrsVars = new Object();
        function bodyonload(){

        <!-- When the html is loaded, a two-dimensional array (indexed by the
             number of the rmrs (1 or 2 in the example), and the position in
             the text) is created, pointing to span-wrappers of the characters
             of the text. In the example, e.g. char[1][0] is a DOM-object
             representing the span-tag that encloses the letter "W", because
             the first rmrs-text ("Where is the Nokia...") starts with
             "W".document.getElementById(..) retrieves the object in question.
             Those objects are then used to change foreground and background
             colors (see functions setColor, setBackground, hilite, lolite)
             of the text when the mouse moves over a predicate. (But there
             is no predicate-highlighting when the mouse moves over the text,
             due to non-uniqueness.) The cfrom and cto attributes of
             ep-element in the input xml are passed to charSelect
             (charUnselect), when the mouse (un)selects a predicate.
         -->
      </xsl:text>
      <xsl:for-each select="//rmrs">
        <xsl:variable name="rmrsNum" select="position()"/>
        <xsl:text>chars[</xsl:text>
        <xsl:value-of select="$rmrsNum"/>
        <xsl:text>] = new Array();
        </xsl:text>
        <xsl:for-each select="text/c">
          <xsl:text>chars[</xsl:text>
          <xsl:value-of select="$rmrsNum"/>
          <xsl:text>][</xsl:text>
          <xsl:value-of select="position()-1"/>
          <xsl:text>] = document.getElementById('char</xsl:text>
          <xsl:value-of select="$rmrsNum"/>
          <xsl:text>_</xsl:text>
          <xsl:value-of select="position()-1"/>
          <xsl:text>');
          </xsl:text>
        </xsl:for-each>
      </xsl:for-each>
      <xsl:text>
      <!-- similar to the above, the DOM-wrappers of the rmrs-variables
           are retrieved here for later use in highlighting. The global
           object rmrsVars holds an associative array mapping each
           rmrs-variable to the array of occurances of the variable in
           the document. For example,
           rmrsVars['1_u19'] = rmrsVars.1_u19 is the array of the two
           occurances of the variable u19 in the first rmrs.   
           Corresponding mouse-event will trigger rmrsVariable(un)Select.
      -->    
        var all = document.getElementsByTagName('span');
        for (var i = 0; all.length > i; i++) {
        var className = all[i].className;
        if ( !rmrsVars[className] ) { rmrsVars[className] = new Array(); }
        var elems = rmrsVars[className];
        var l = elems.length;
        elems[l] = all[i];
        }
        }
        function setColor(elem, color) {
        elem.style.color = color;
        }    
        function setBackground(elem, color) {
        elem.style.background = color;
        }    
        function hilite(elem) {
        setColor(elem, background);
        setBackground(elem, marked);
        }    
        function lolite(elem) {
        setColor(elem, unmarked);
        setBackground(elem, background);
        }    
        function charSelect(rmrsNum, posFrom, posTo) {
        for (var position = posFrom; posTo >= position; position++)
        hilite(chars[rmrsNum][position]);
        }    
        function charUnselect(rmrsNum, posFrom, posTo) {
        for (var position = posFrom; posTo >= position; position++)
        lolite(chars[rmrsNum][position]);
        }    
        function varSelect(tags) {
        for (var i = 0; tags.length > i; i++)
        hilite(tags[i]);
        }    
        function varUnselect(tags) {
        for (var i = 0; tags.length > i; i++)
        lolite(tags[i]);
        }    
        function rmrsVariableSelect(name) {
        varSelect(rmrsVars['rmrsVariable' + name]);
        }
        function rmrsVariableUnselect(name) {
        varUnselect(rmrsVars['rmrsVariable' + name]);
        }
      </xsl:text>
    </script>
  </xsl:template>
  <!-- ===================           RMRS            ============================ -->
  <!-- just call the templates for the 5 rows of the main table (text, top,
       rels, hcons, ing), each with the rmrs-number as a parameter -->
  <xsl:template match="rmrs">
    <xsl:variable name="rmrsNum" select="position()"/>
    <tr>
      <td>
        <table class="rmrsMrs">
          <tbody>
            <xsl:call-template name="txt">
              <xsl:with-param name="rmrsNum">
                <xsl:value-of select="$rmrsNum"/>
              </xsl:with-param>
            </xsl:call-template>
            <xsl:call-template name="top">
              <xsl:with-param name="rmrsNum">
                <xsl:value-of select="$rmrsNum"/>
              </xsl:with-param>
            </xsl:call-template>
            <xsl:call-template name="rels">
              <xsl:with-param name="rmrsNum">
                <xsl:value-of select="$rmrsNum"/>
              </xsl:with-param>
            </xsl:call-template>
            <xsl:call-template name="hcons">
              <xsl:with-param name="rmrsNum">
                <xsl:value-of select="$rmrsNum"/>
              </xsl:with-param>
            </xsl:call-template>
            <xsl:call-template name="ingroup">
              <xsl:with-param name="rmrsNum">
                <xsl:value-of select="$rmrsNum"/>
              </xsl:with-param>
            </xsl:call-template>
          </tbody>
        </table>
      </td>
    </tr>
  </xsl:template>
  <!-- ==================                        TXT                       ================= -->
  <!-- new class in stylesheet might be good -->
  <xsl:template name="txt">
    <xsl:param name="rmrsNum">42</xsl:param>
    <tr>
      <td class="rmrsFeatureTop">TEXT
      </td>
      <td class="rmrsValueTop">
        <xsl:for-each select="text/c">
          <xsl:call-template name="c">
            <xsl:with-param name="rmrsNum">
              <xsl:value-of select="$rmrsNum"/>
            </xsl:with-param>
          </xsl:call-template>
        </xsl:for-each>
      </td>
    </tr>
  </xsl:template>
  <!-- =================                C                      ============== -->
  <!-- individual letters of the rext -->
  <xsl:template name="c">
    <xsl:param name="rmrsNum">777</xsl:param>
    <span>
      <xsl:attribute name="style"><xsl:text>color:</xsl:text><xsl:value-of select="$unmarked"/></xsl:attribute>
      <xsl:attribute name="id"><xsl:text>char</xsl:text><xsl:value-of select="$rmrsNum"/><xsl:text>_</xsl:text><xsl:value-of select="position()-1"/></xsl:attribute>
      <xsl:variable name="character">
        <xsl:value-of select="."/>
      </xsl:variable>
      <xsl:value-of select="$character"/>
    </span>
  </xsl:template>
  <!-- ==============                           TOP                           ============ -->
  <!-- the label-daughter of the rmrs-element gives the top-handle -->
  <xsl:template name="top">
    <xsl:param name="rmrsNum">42</xsl:param>
    <tr>
      <td class="rmrsFeatureTop">TOP</td>
      <td class="rmrsValueTop">
        <xsl:call-template name="varspan">
          <xsl:with-param name="varname">
            <xsl:apply-templates select="label"/>
          </xsl:with-param>
          <xsl:with-param name="rmrsNum">
            <xsl:value-of select="$rmrsNum"/>
          </xsl:with-param>
        </xsl:call-template>
      </td>
    </tr>
  </xsl:template>
  <!-- =================                  RELS                ============= -->
  <!-- The relations (main entry). Calls ep and freeArgs, and puts all that
       into {} -->
  <xsl:template name="rels">
    <xsl:param name="rmrsNum">42</xsl:param>
    <tr>
      <td class="rmrsFeatureRels">RELS</td>
      <td class="rmrsValueRels">
        <table class="rmrsRelsContainer">
          <tbody>
            <tr>
              <td valign="middle">
                <span class="rmrsBracket">{</span>
              </td>
              <td>
                <table class="rmrsRelsContainer">
                  <tbody>
                    <tr>
                      <xsl:call-template name="ep">
                        <xsl:with-param name="rmrsNum">
                          <xsl:value-of select="$rmrsNum"/>
                        </xsl:with-param>
                      </xsl:call-template>
                      <xsl:call-template name="freeArgs">
                        <xsl:with-param name="rmrsNum">
                          <xsl:value-of select="$rmrsNum"/>
                        </xsl:with-param>
                      </xsl:call-template>
                    </tr>
                  </tbody>
                </table>
              </td>
              <td valign="middle">
                <span class="rmrsBracket">}</span>
              </td>
            </tr>
          </tbody>
        </table>
      </td>
    </tr>
  </xsl:template>
  <!-- =================                              EP                       ============ -->
  <xsl:template name="ep">
    <xsl:param name="rmrsNum">42</xsl:param>
    <xsl:variable name = "argAfterLbl">
    	<xsl:choose>
    		<xsl:when test="count(rarg/rargname[text()='ARG0'])=0">ARG0</xsl:when>
    		<xsl:otherwise><xsl:value-of select="$arg0name" /></xsl:otherwise>
    	</xsl:choose>
    </xsl:variable>
    <xsl:for-each select="ep">
      <!-- remember handle for later putting the rmrs-vars to the right
           place -->
      <xsl:variable name="handle">
        <xsl:value-of select="label/@vid"/>
      </xsl:variable>
      <!-- arguments that mouse-event must send to JavaScript text-hiliting
           function -->
      <xsl:variable name="mouseArgs">
        <xsl:value-of select="$rmrsNum"/>
        <xsl:text>, </xsl:text>
        <xsl:value-of select="@cfrom"/>
        <xsl:text>, </xsl:text>
        <xsl:value-of select="@cto"/>
        <xsl:text>);</xsl:text>
      </xsl:variable>
      <td>
        <table class="rmrsRelation">
          <tbody>
            <tr> <!-- rels-row 1: the predicate -->
              <td class="rmrsPredicate" colspan="2">
                <xsl:attribute name="onmouseover"><xsl:text>charSelect(</xsl:text><xsl:value-of select="$mouseArgs"/></xsl:attribute>
                <xsl:attribute name="onmouseout"><xsl:text>charUnselect(</xsl:text><xsl:value-of select="$mouseArgs"/></xsl:attribute>
                <xsl:apply-templates select="gpred | realpred"/>
              </td>
            </tr>
            <tr> <!-- rels-row 2: the label / handle -->
              <td class="rmrsLabel">LBL</td>
              <td class="rmrsValue">
                <xsl:call-template name="varspan">
                  <xsl:with-param name="varname">
                    <xsl:apply-templates select="label"/>
                  </xsl:with-param>
                  <xsl:with-param name="rmrsNum">
                    <xsl:value-of select="$rmrsNum"/>
                  </xsl:with-param>
                </xsl:call-template>
              </td>
            </tr>
            <tr> <!-- rels-row 3: ARG0 -->
              <td class="rmrsLabel"><xsl:value-of select="$argAfterLbl" /></td>
              <td class="rmrsValue">
                <xsl:call-template name="varspan">
                  <xsl:with-param name="varname">
                    <xsl:apply-templates select="var"/>
                  </xsl:with-param>
                  <xsl:with-param name="rmrsNum">
                    <xsl:value-of select="$rmrsNum"/>
                  </xsl:with-param>
                </xsl:call-template>
              </td>
            </tr>
            <!-- rels-rows greater 3: the other arguments / variables
             "other" arguments of current ep come from the older
                 rarg-sisters, having a label with vid = the vid of  
                 the current ep, which was stored into variable
                 $handle above. "Parent" goes back up from the label
                 to the rarg element, because that's what has to be
                 selected / looped -->
            <xsl:for-each select="following-sibling::rarg/label[@vid=$handle]/parent::*">
              <tr>
                <td class="rmrsLabel">
                  <xsl:value-of select="rargname"/>
                </td>
                <td class="rmrsValue">
                  <xsl:call-template name="varspan">
                    <xsl:with-param name="varname">
                      <xsl:apply-templates select="var | constant"/>
                    </xsl:with-param>
                    <xsl:with-param name="rmrsNum">
                      <xsl:value-of select="$rmrsNum"/>
                    </xsl:with-param>
                  </xsl:call-template>
                </td>
              </tr>
            </xsl:for-each>
          </tbody>
        </table>
      </td>
	  <xsl:if test = "position() mod $lineBreakAfterThisManyPreds = 0">
	  <!-- now insert </tr><tr> for table-linebreak; that's ill-formed xml, so let's play dirty!
	       (clean is possible but A LOT more work) -->
   		<td><xsl:value-of disable-output-escaping="yes" select="$dirtyHack" /></td>
 	  </xsl:if>
    </xsl:for-each>
  </xsl:template>
  <!-- =================                    FREEARGS                       ============ -->
  <xsl:template name="freeArgs">
    <xsl:param name="rmrsNum">42</xsl:param>
    <xsl:for-each select="rarg">
      <xsl:variable name="label">
        <xsl:value-of select="label/@vid"/>
      </xsl:variable>
      <!-- if the argument (rarg) has an ep sister with same label@vid, then
           the argument is not free -->
      <xsl:variable name="numWitnessesOfNonFreedom">
        <xsl:value-of select="count(parent::*/ep/label[@vid=$label])"/>
      </xsl:variable>
      <!-- if the argument has no such sisters, it's free. -->
      <xsl:if test="$numWitnessesOfNonFreedom=0">
        <td>
          <table class="rmrsRelation">
            <tbody>
              <tr>
                <td class="rmrsPredicate" colspan="2">
                  <xsl:text>free_args</xsl:text>
                </td>
              </tr>
              <tr>
                <td class="rmrsLabel">
                  <xsl:value-of select="rargname"/>
                </td>
                <td class="rmrsValue">
                  <xsl:call-template name="varspan">
                    <xsl:with-param name="varname">
                      <xsl:apply-templates select="var | constant"/>
                    </xsl:with-param>
                    <xsl:with-param name="rmrsNum">
                      <xsl:value-of select="$rmrsNum"/>
                    </xsl:with-param>
                  </xsl:call-template>
                </td>
              </tr>
            </tbody>
          </table>
        </td>
      </xsl:if>
    </xsl:for-each>
  </xsl:template>
  <!-- ================                           GPRED                  ================== -->
  <!-- predicate names are text from gpred elements -->
  <xsl:template match="gpred">
    <xsl:value-of select="."/>
  </xsl:template>
  <!-- ===========                            REALPRED                           =============== -->
  <!-- To distinguish gpreds from realpreds, the latter start
       with "_". E.g. the verb-realpred "use" is output _use_v. -->
  <xsl:template match="realpred">
    <xsl:text>_</xsl:text>
    <xsl:value-of select="@lemma"/>
    <xsl:text>_</xsl:text>
    <xsl:value-of select="@pos"/>
  </xsl:template>
  <!-- =======                                     HCONS                              ================ -->
  <xsl:template name="hcons">
    <xsl:param name="rmrsNum">42</xsl:param>
    <tr>
      <td class="rmrsFeatureHcons">HCONS</td>
      <td class="rmrsValueHcons">
        <span class="rmrsBracket">{</span>
        <xsl:for-each select="hcons">
          <xsl:call-template name="varspan">
            <xsl:with-param name="varname">
              <xsl:apply-templates select="hi"/>
            </xsl:with-param>
            <xsl:with-param name="rmrsNum">
              <xsl:value-of select="$rmrsNum"/>
            </xsl:with-param>
          </xsl:call-template>
          <xsl:text> </xsl:text>
          <xsl:value-of select="@hreln"/>
          <xsl:text> </xsl:text>
          <xsl:call-template name="varspan">
            <xsl:with-param name="varname">
              <xsl:apply-templates select="lo"/>
            </xsl:with-param>
            <xsl:with-param name="rmrsNum">
              <xsl:value-of select="$rmrsNum"/>
            </xsl:with-param>
          </xsl:call-template>
          <xsl:if test="not (position()=last())">
            <xsl:text>, </xsl:text>
          </xsl:if>
        </xsl:for-each>
        <span class="rmrsBracket">}</span>
      </td>
    </tr>
  </xsl:template>
  <!-- ========                               HI                     =================== -->
  <xsl:template match="hi">
    <xsl:apply-templates select="var"/>
  </xsl:template>
  <!-- ========                               LO                     =================== -->
  <xsl:template match="lo">
    <xsl:apply-templates select="label"/>
  </xsl:template>
  <!-- ========                            INGROUP                  =================== -->
  <xsl:template name="ingroup">
    <xsl:param name="rmrsNum">42</xsl:param>
    <tr>
      <td class="rmrsFeatureHcons">ING</td>
      <td class="rmrsValueHcons">
        <span class="rmrsBracket">{</span>
        <xsl:for-each select="ing">
          <xsl:call-template name="varspan">
            <xsl:with-param name="varname">
              <xsl:apply-templates select="ing-a"/>
            </xsl:with-param>
            <xsl:with-param name="rmrsNum">
              <xsl:value-of select="$rmrsNum"/>
            </xsl:with-param>
          </xsl:call-template>
          <xsl:text> ing </xsl:text>
          <xsl:call-template name="varspan">
            <xsl:with-param name="varname">
              <xsl:apply-templates select="ing-b"/>
            </xsl:with-param>
            <xsl:with-param name="rmrsNum">
              <xsl:value-of select="$rmrsNum"/>
            </xsl:with-param>
          </xsl:call-template>
          <xsl:if test="not (position()=last())">
            <xsl:text>,</xsl:text>
          </xsl:if>
        </xsl:for-each>
        <span class="rmrsBracket">}</span>
      </td>
    </tr>
  </xsl:template>
  <!-- ========                               ING-A                     =================== -->
  <xsl:template match="ing-a">
    <xsl:apply-templates select="var"/>
  </xsl:template>
  <!-- ========                               ING-B                     =================== -->
  <xsl:template match="ing-b">
    <xsl:apply-templates select="var"/>
  </xsl:template>
  <!-- ============               VAR                      ============ -->
  <xsl:template match="var">
    <xsl:value-of select="@sort"/>
    <xsl:value-of select="@vid"/>
    <xsl:for-each select="@*">
      <xsl:if test='name(.)!="sort" and name(.)!="vid"'>
        <xsl:value-of select="$varsep"/>
        <xsl:value-of select="name(.)"/>
        <xsl:text>=</xsl:text>
        <xsl:value-of select="."/>
      </xsl:if>
    </xsl:for-each>
  </xsl:template>
  <!-- ==========                            LABEL                        =============== -->
  <!-- labels always start with "h", followed by their vid -->
  <xsl:template match="label">
    <xsl:text>h</xsl:text>
    <xsl:value-of select="@vid"/>
  </xsl:template>
  <!-- ==========                            CONSTANT                        =============== -->
  <xsl:template match="constant">
    <xsl:value-of select="."/>
  </xsl:template>
  <!-- ==========                         VARSPAN                            ============ -->
  <xsl:template name="varspan">
    <xsl:param name="varname">error</xsl:param>
    <xsl:param name="rmrsNum">0</xsl:param>
    <xsl:variable name="composedVarName">
      <xsl:value-of select="$rmrsNum"/>
      <xsl:value-of select="$varsep"/>
      <xsl:choose>
        <xsl:when test='contains($varname, $varsep) and contains($varname, "=")'>
          <xsl:value-of select='substring-before($varname, $varsep)'/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$varname"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <span>
      <xsl:attribute name="style"><xsl:text>color:</xsl:text><xsl:value-of select="$unmarked"/></xsl:attribute>
      <xsl:attribute name="class"><xsl:text>rmrsVariable</xsl:text><xsl:value-of select="$composedVarName"/></xsl:attribute>
      <xsl:attribute name="onmouseover"><xsl:text>rmrsVariableSelect('</xsl:text><xsl:value-of select="$composedVarName"/><xsl:text>')</xsl:text></xsl:attribute>
      <xsl:attribute name="onmouseout"><xsl:text>rmrsVariableUnselect('</xsl:text><xsl:value-of select="$composedVarName"/><xsl:text>')</xsl:text></xsl:attribute>
      <xsl:text/>
      <xsl:choose>
        <xsl:when test='contains($varname, $varsep) and contains($varname, "=")'>
          <xsl:value-of select='substring-before($varname, $varsep)'/>
          <span style="font-size: 6pt">
            <xsl:value-of select="$varsep"/>
            <xsl:value-of select='substring-after($varname, $varsep)'/>
          </span>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$varname"/>
        </xsl:otherwise>
    </xsl:choose>
    </span>
  </xsl:template>
</xsl:stylesheet>