<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
                version="1.0">

  <!-- Transform LKB sppp format (http://wiki.delph-in.net/moin/LkbSppp) -->
  <!-- to PET XML input chart format -->
  <!-- (http://wiki.delph-in.net/moin/PetInput, XML input) -->

  <!-- Version: $Id: sppp2pic.xsl,v 1.3 2006/06/29 13:04:02 uschaefer Exp $ -->

  <!-- 
   (C) 2006 DFKI Language Technology Lab http://www.dfki.de/lt
   Author: Ulrich Schaefer, Ulrich.Schaefer@dfki.de

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
-->


<!-- 

  SPPP DTD:

  <!ELEMENT segment ( token )* >

  <!ELEMENT token ( analysis )* >
  <!ATTLIST token form CDATA #REQUIRED 
                  from NMTOKEN #REQUIRED
                  to NMTOKEN #REQUIRED >

  <!ELEMENT analysis ( rule )* >
  <!ATTLIST analysis stem CDATA #REQUIRED
                     probability NMTOKEN #IMPLIED
                     pos NMTOKEN #IMPLIED >
            
  <!ELEMENT rule EMPTY >
  <!ATTLIST rule id NMTOKEN #REQUIRED
                 form CDATA #REQUIRED >


  sample input:

  <segment>
    <token form="kim" from="0" to="2">
      <analysis stem="kim"/>
    </token>
    <token form="sleeps." from="4" to="10">
      <analysis stem="sleep">
        <rule id="plur_noun_infl_rule" form="sleeps"/>
        <rule id="punct_period_rule" form="sleeps."/>
      </analysis>
      <analysis stem="sleep" probability="0.5" pos="VP">
        <rule id="third_sg_fin_verb_infl_rule" form="sleeps"/>
        <rule id="punct_period_rule" form="sleeps."/>
      </analysis>
    </token>
  </segment>


  PIC DTD:

  <!ELEMENT pet-input-chart ( w | ne )* >

  <!ELEMENT w ( surface, path*, pos*, typeinfo* ) >
  <!ATTLIST w       id ID      #REQUIRED
                cstart NMTOKEN #REQUIRED
                  cend NMTOKEN #REQUIRED
                  prio CDATA   #IMPLIED
                constant (yes | no) "no" >

  <!ELEMENT surface ( #PCDATA ) >

  <!ELEMENT path EMPTY >
  <!ATTLIST path     num NMTOKEN #REQUIRED >

  <!ELEMENT typeinfo ( stem, infl*, fsmod* ) >
  <!ATTLIST typeinfo  id ID    #REQUIRED
                    prio CDATA #IMPLIED
                 baseform (yes | no) "yes" >

  <!ELEMENT stem ( #PCDATA ) >

  <!ELEMENT infl   EMPTY >
  <!ATTLIST infl   name  CDATA #REQUIRED >

  <!ELEMENT fsmod  EMPTY >
  <!ATTLIST fsmod  path  CDATA #REQUIRED
                   value CDATA #REQUIRED >

  <!ELEMENT pos  EMPTY >
  <!ATTLIST pos  tag     CDATA #REQUIRED
                 prio    CDATA #IMPLIED >

  <!ELEMENT ne  ( ref+, pos*, typeinfo+ )  >
  <!ATTLIST ne        id ID    #REQUIRED
                    prio CDATA #IMPLIED >

  <!ELEMENT ref  EMPTY >
  <!ATTLIST ref      dtr IDREF #REQUIRED >

  sample output (goal; from TnT+Sprout preprocessing!):

  <pet-input-chart>
    <w id="TNT0" cstart="0" cend="2">
      <surface>Did</surface>
      <pos tag="VBD" prio="1.000000e+00"/>
    </w>
    <w id="TNT1" cstart="4" cend="6">
      <surface>Dan</surface>
      <pos tag="NNP" prio="1.000000e+00"/>
    </w>
    <w id="TNT2" cstart="8" cend="17">
      <surface>Flickinger</surface>
      <pos tag="NNP" prio="1.000000e+00"/>
    </w>
    <w id="TNT3" cstart="19" cend="24">
      <surface>travel</surface>
      <pos tag="NN" prio="8.104886e-01"/>
      <pos tag="VBP" prio="1.084933e-01"/>
      <pos tag="VB" prio="8.101807e-02"/>
    </w>
    <w id="TNT4" cstart="26" cend="27">
      <surface>to</surface>
      <pos tag="TO" prio="1.000000e+00"/>
    </w>
    <w id="TNT5" cstart="29" cend="31">
      <surface>ACL</surface>
      <pos tag="NNP" prio="9.015200e-01"/>
      <pos tag="VB" prio="9.847996e-02"/>
    </w>
    <w id="TNT6" cstart="33" cend="36">
      <surface>2003</surface>
      <pos tag="CD" prio="1.000000e+00"/>
      <typeinfo id="TYI6" baseform="no">
        <stem>$generic_number</stem>
      </typeinfo>
    </w>
    <w id="TNT7" cstart="37" cend="37" constant="yes">
      <surface>?</surface>
      <pos tag="?" prio="1.0"/>
    </w>
    <w id="SPR2.1" cstart="4" cend="17" constant="yes" prio="0.5">
      <surface>Dan Flickinger</surface>
      <typeinfo id="TIN2.1" baseform="no">
        <stem>$generic_name</stem>
      </typeinfo>
    </w>
    <w id="SPR3.1" cstart="29" cend="36" constant="yes" prio="0.5">
      <surface>ACL 2003</surface>
      <typeinfo id="TIN3.1" baseform="no">
        <stem>$generic_name</stem>
      </typeinfo>
    </w>
  </pet-input-chart>

-->

  <xsl:output method="xml" omit-xml-declaration="no" indent="yes"/>
  <xsl:strip-space elements="*"/>

  <xsl:param name="metadata"><metadata/></xsl:param>

  <xsl:template match="/">
    <xsl:element name="pet-input-chart">
      <xsl:copy-of select="$metadata"/>
      <xsl:for-each select="@*">
        <xsl:copy-of select="."/>
      </xsl:for-each>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>

  <xsl:template match="token">
      <xsl:apply-templates select="analysis">
        <xsl:with-param name="cstart" select="@from"/>
        <xsl:with-param name="cend" select="@to"/>
        <xsl:with-param name="tokenno" select="position()"/>
        <xsl:with-param name="surface" select="@form"/>
      </xsl:apply-templates>
  </xsl:template>

  <xsl:template match="analysis">
    <xsl:param name="cstart" select="-1"/>
    <xsl:param name="cend" select="-1"/>
    <xsl:param name="tokenno" select="0"/>
    <xsl:param name="surface" select="'SURFACE'"/>
    <xsl:param name="idsuffix" select="concat($tokenno,'.',position())"/>

    <w id="{concat('FLW', $idsuffix)}" cstart="{$cstart}" cend="{$cend}">

      <xsl:choose>
        <xsl:when test="count(*)=0 and not(@pos)">
          <xsl:attribute name="constant">
            <xsl:value-of select="'yes'"/>
          </xsl:attribute>
          <surface><xsl:value-of select="$surface"/></surface>
          <typeinfo id="{concat('FLT',$tokenno,'.',position())}" baseform="no">
            <stem><xsl:value-of select="@stem"/></stem>
          </typeinfo>
        </xsl:when>
        <xsl:otherwise>
          <surface><xsl:value-of select="$surface"/></surface>
        </xsl:otherwise>
      </xsl:choose>

      <xsl:if test="@tag">
        <pos tag="{@tag}">
          <xsl:if test="@probability">
            <xsl:attribute name="prio">
              <xsl:value-of select="@probability"/>
            </xsl:attribute>
          </xsl:if>
        </pos>
      </xsl:if>

      <xsl:apply-templates select="rule">
        <xsl:with-param name="idsuffix" select="$idsuffix"/>
      </xsl:apply-templates>

    </w>

  </xsl:template>


  <xsl:template match="rule">
    <xsl:param name="idsuffix" select="0"/>
    <xsl:param name="stem" select="NOSTEM"/>
    <typeinfo id="{concat('FLR',$idsuffix, '.', position())}" baseform="no">
      <stem><xsl:value-of select="@form"/></stem>
      <infl name="{@id}"/>
    </typeinfo>
  </xsl:template>


  <xsl:template match="text()"/>


</xsl:stylesheet>

