<?xml version="1.0" encoding="iso-8859-1"?>

<!-- morphposfilter.xsl
*    Inserting Chunkie-CAT for POS-filtering:
*    - traverse and print the entire XML structure
*    - for all words (by span info) match all their M-SYN structures (i.e all readings)
*        - retrieve chunkie-cat for given span
*        - add feature CAT with type chunkie-cat  
*        - print all remaining features of M-SYN.
*
* (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
*   Authors: Anette Frank, Anette.Frank@dfki.de
*            Kathrin Spreyer, Kathrin.Spreyer@dfki.de
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Lesser General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public
*   License along with this library; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
-->


<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output method="xml" indent="yes"/>
<xsl:strip-space elements="*"/>
<xsl:param name="uri">file:/tmp/kspreyer/hog/Chunkie.xml</xsl:param>
<xsl:param name="lang" select="'en'"/>


<xsl:template match="*">
  <xsl:choose>
    <xsl:when test="@type='lex'">
      <xsl:variable name="matchid" select="../@id"/>

      
      <xsl:variable name="chunkie-cat">
        <xsl:call-template name="get-chunkie-cat">
          <xsl:with-param name="begcat" select="../@cstart"/>
          <xsl:with-param name="endcat" select="../@cend"/>
        </xsl:call-template>
      </xsl:variable>

      <!-- define new FS M-SYN
           rewrite type m-syn as map-morph-<chunkie-cat> 
           add feature CAT = <chunkie-cat> (for all readings) 
           copy all features  -->

   <xsl:element name="FS">
      <xsl:attribute name="type">  
        <xsl:value-of select="'lex'"/>
      </xsl:attribute>  
      <xsl:element name="F">
        <xsl:attribute name="name">
           <xsl:value-of select="'NODE'"/>
        </xsl:attribute>
        <xsl:element name="FS">
          <xsl:attribute name="type">
            <xsl:value-of select="'node'"/>
          </xsl:attribute>
          <xsl:element name="F">
             <xsl:attribute name="name">
                <xsl:value-of select="'ID'"/>
              </xsl:attribute>
              <xsl:element name="FS">
                <xsl:attribute name="type">
                    <xsl:value-of select="concat('&quot;',../@cstart,'_',../@cend,'&quot;')"/>
              </xsl:attribute>
             </xsl:element>
          </xsl:element>
        </xsl:element>
      </xsl:element>

      <xsl:element name="F">
          <xsl:attribute name="name">
             <xsl:value-of select="'M-SYN'"/>
          </xsl:attribute>
          <xsl:element name="FS">
             <xsl:attribute name="type">
                <xsl:value-of select="'m-syn'"/>
                <!--xsl:value-of select="concat('map-morph-',translate($chunkie-cat,'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz'))"/-->
             </xsl:attribute>

             <xsl:element name="F">
               <xsl:attribute name="name">
                 <xsl:value-of select="'CAT'"/>
               </xsl:attribute>
               <xsl:element name="FS">
                 <xsl:attribute name="type">
                   <xsl:choose>
                     <xsl:when test="$lang='en'">
                       <xsl:value-of select="translate(concat('_',$chunkie-cat),'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')"/>
                     </xsl:when>
                     <xsl:otherwise>
                       <xsl:value-of select="translate($chunkie-cat,'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')"/>
                     </xsl:otherwise>
                   </xsl:choose>   

                 </xsl:attribute>
               </xsl:element>
             </xsl:element>

             <xsl:for-each select='F[@name="M-SYN"]/FS/F[@name!="CAT"]'>
                <xsl:copy-of select="."/>
             </xsl:for-each> 

        </xsl:element>
      </xsl:element>

      <xsl:for-each select="F[@name!='M-SYN' and @name!='NODE']">
            <xsl:copy-of select="."/>
      </xsl:for-each> 
      </xsl:element>
        
      </xsl:when>
      
      <xsl:otherwise>

        <xsl:copy>
          <xsl:for-each select="@*">
              <xsl:copy-of select="."/>
          </xsl:for-each>
          <xsl:apply-templates/>
        </xsl:copy>

      </xsl:otherwise>
    </xsl:choose>

  </xsl:template>
  

  <!--go to chunke-xml, get + return cat value for corresponding span -->
  <xsl:template name="get-chunkie-cat">
    <xsl:param name="begcat"/>
    <xsl:param name="endcat"/>
    <xsl:value-of select="document($uri)//w[@cstart=$begcat and @cend=$endcat]/@pos"/>
  </xsl:template>

</xsl:stylesheet>
