<?xml version="1.0" encoding="iso-8859-1"?>

<!-- nodeinfo.xsl
*    Inserting node info: 
*    local ID (defined by spans), mother ID and CAT as defined by Chunkie
*    - traverse and print the entire XML structure
*    - for all synsem/lex/phrase types and CAT structures (with identifying span) (in all readings)
*        - retrieve mother node's category and spans from Chunkie
*        - define NODE feature: local ID, M-ID, M-CAT
*        - print all remaining features of synsem/lex/phrase
*
* (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
*   Authors: Anette Frank, Anette.Frank@dfki.de
*            Kathrin Spreyer, Kathrin.Spreyer@dfki.de
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Lesser General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public
*   License along with this library; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
-->

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output method="xml" indent="yes"/>
<xsl:strip-space elements="*"/>
<xsl:param name="uri">file:/tmp/kspreyer/hog/Chunkie.xml</xsl:param>

<xsl:template match="*">
<xsl:choose>

    <xsl:when test="(substring(@type,string-length(@type)-2,string-length(@type))='lex') or (@type='phrase')">
 
      <xsl:variable name="matchid" select="../@id"/>

       <xsl:variable name="synsem">
          <xsl:choose>
            <xsl:when test="@type='phrase'">    <!-- doesn#t match -->
              <xsl:value-of select="@type"/>
            </xsl:when>
            <xsl:otherwise>
               <xsl:value-of select="'lex'"/>            
            </xsl:otherwise>
          </xsl:choose>
       </xsl:variable>

      <xsl:variable name="chunkie-node-mid">
        <xsl:call-template name="get-chunkie-mid">
          <xsl:with-param name="begspan" select="../@cstart"/>
          <xsl:with-param name="endspan" select="../@cend"/>
        </xsl:call-template>
      </xsl:variable>

      <xsl:variable name="chunkie-node-mcat">
        <xsl:call-template name="get-chunkie-mcat">
          <xsl:with-param name="begspan" select="../@cstart"/>
          <xsl:with-param name="endspan" select="../@cend"/>
        </xsl:call-template>
      </xsl:variable>

     <xsl:variable name="nodeid">
        <xsl:value-of select="F[@name='NODE']/FS/F[@name='ID']/FS/@type"/>
     </xsl:variable>

     <xsl:element name="FS">
          <xsl:attribute name="type">

            <xsl:value-of select="@type"/>   <!-- insert variable value here -->
          </xsl:attribute>
          <xsl:element name="F">
            <xsl:attribute name="name">
              <xsl:value-of select="'NODE'"/>
            </xsl:attribute>
            <xsl:element name="FS">
              <xsl:attribute name="type">
                <xsl:value-of select="'node'"/>
              </xsl:attribute>

              <xsl:element name="F">
                <xsl:attribute name="name">
                   <xsl:value-of select="'ID'"/>
                </xsl:attribute>
                <xsl:element name="FS">
                   <xsl:attribute name="type">
                       <xsl:value-of select="$nodeid"/>
                   </xsl:attribute>
                </xsl:element>
              </xsl:element>

              <xsl:element name="F">
                <xsl:attribute name="name">
                   <xsl:value-of select="'M-ID'"/>
                </xsl:attribute>
                <xsl:element name="FS">
                   <xsl:attribute name="type">
                       <xsl:value-of select="concat('&quot;',$chunkie-node-mid,'&quot;')"/>
                   </xsl:attribute>
                </xsl:element>
              </xsl:element>

              <xsl:element name="F">
                <xsl:attribute name="name">
                   <xsl:value-of select="'M-CAT'"/>
                </xsl:attribute>
                <xsl:element name="FS">
                   <xsl:attribute name="type">
                     <xsl:copy-of select="$chunkie-node-mcat"/>
                   </xsl:attribute>
                </xsl:element>
              </xsl:element>
          </xsl:element>
      </xsl:element>

      <xsl:for-each select='F'>
          <xsl:if test="./@name!='NODE'">
                <xsl:copy-of select="."/>
          </xsl:if>
      </xsl:for-each> 

    </xsl:element>


    </xsl:when>

    <xsl:otherwise>
        <xsl:copy>
          <xsl:for-each select="@*">
            <xsl:copy-of select="."/>
          </xsl:for-each>
          <xsl:apply-templates/>
        </xsl:copy>
    </xsl:otherwise>
</xsl:choose>
</xsl:template>

  <!-- go to chunkie-xml, find chunk with matching local-beg, local-end (and local-cat),
         retrieve its mother-chunks beg, end and cat 
         return as result of this template call-->

  <xsl:template name="get-chunkie-mid">
    <xsl:param name="begspan"/>
    <xsl:param name="endspan"/>

    <xsl:variable name="mothernode" select="document($uri)//*[@cstart=$begspan and @cend=$endspan]/.."/>

    <xsl:value-of select="concat($mothernode/@cstart,'_',$mothernode/@cend)"/>
  </xsl:template>

  <xsl:template name="get-chunkie-mcat">
    <xsl:param name="begspan"/>
    <xsl:param name="endspan"/>

    <xsl:variable name="mothernode" select="document($uri)//*[@cstart=$begspan and @cend=$endspan]/.."/>

    <xsl:value-of select="translate($mothernode/@cat, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ', 'abcdefghijklmnopqrstuvwxyz')"/>
  </xsl:template>

</xsl:stylesheet>
