<?xml version="1.0" encoding="iso-8859-1"?>

<!-- rmrsfs2rmrsxml_sets.xsl
*    Convert SProUT XML output to XML RMRS
*    (This stylesheet is based on code created by Ulrich.Schaefer@dfki.de)
*   
* (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
*   Authors: Anette Frank, Anette.Frank@dfki.de
*            Kathrin Spreyer, Kathrin.Spreyer@dfki.de
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Lesser General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public
*   License along with this library; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
-->


<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
        
  <xsl:output method="xml" indent="yes"/>
  <xsl:strip-space elements="*"/>

  <xsl:template match='/SPROUTPUT'>
    <!-- preserve root node, but downcase element name -->

    <xsl:element name="rmrs-list">

    <!-- extract minimal and maximal character pos for delimiting the span of complete rmrs -->
       <xsl:attribute name="cfrom">
          <xsl:for-each select="//MATCHINFO">
              <xsl:sort select="@cstart" data-type="number" order="ascending"/>
              <xsl:if test="position() = 1">
                  <xsl:value-of select="@cstart" />
              </xsl:if>
          </xsl:for-each>
       </xsl:attribute>

       <xsl:attribute name="cto">
          <xsl:for-each select="//MATCHINFO">
              <xsl:sort select="@cend" data-type="number" order="ascending"/>
              <xsl:if test="position() = last()">
                  <xsl:value-of select="@cend" />
              </xsl:if>
          </xsl:for-each>
       </xsl:attribute>

       <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>

  <xsl:template match='metadata'>
      <xsl:copy-of select="."/>
  </xsl:template>

  <xsl:template match='DISJ'>
    <!-- preserve disjunction embedding, but downcase element name -->
    <xsl:element name="disj">
      <xsl:apply-templates/>
    </xsl:element> 
  </xsl:template>

  <xsl:template match='MATCHINFO'> 
    <!-- dispatch according to rmrs ep type -->
    <xsl:variable name="cfrom" select="@cstart"/>
    <xsl:variable name="cto" select="@cend"/>
    <xsl:element name="rmrs">
      <xsl:attribute name="cfrom">
           <xsl:value-of select="$cfrom"/>
      </xsl:attribute>
      <xsl:attribute name="cto">
           <xsl:value-of select="$cto"/>
      </xsl:attribute>
      <xsl:call-template name="epfslist">
        <xsl:with-param name="node" select='FS/F[@name="RMRS"]/FS/F[@name="RELS"]/SET/FS'/>
        <xsl:with-param name="listroot" select='FS/F[@name="RMRS"]/FS/F[@name="RELS"]/SET'/>
        <xsl:with-param name="counter" select="'1'"/>
      </xsl:call-template>
      <xsl:call-template name="hconsfslist">
        <xsl:with-param name="cfrom" select="$cfrom"/>
        <xsl:with-param name="cto" select="$cto"/>
        <xsl:with-param name="node" select='FS/F[@name="RMRS"]/FS/F[@name="CONS"]/SET/FS'/>
        <xsl:with-param name="listroot" select='FS/F[@name="RMRS"]/FS/F[@name="CONS"]/SET'/>
        <xsl:with-param name="listroot_rels" select='FS/F[@name="RMRS"]/FS/F[@name="RELS"]/SET'/>
        <xsl:with-param name="counter" select="'1'"/>
      </xsl:call-template>
      <xsl:call-template name="ingfslist">
        <xsl:with-param name="cfrom" select="$cfrom"/>
        <xsl:with-param name="cto" select="$cto"/>
        <xsl:with-param name="node" select='FS/F[@name="RMRS"]/FS/F[@name="ING"]/SET/FS'/>
        <xsl:with-param name="listroot" select='FS/F[@name="RMRS"]/FS/F[@name="ING"]/SET'/>
        <xsl:with-param name="listroot_rels" select='FS/F[@name="RMRS"]/FS/F[@name="RELS"]/SET'/>
        <xsl:with-param name="counter" select="'1'"/>
      </xsl:call-template>
    </xsl:element>
  </xsl:template>

  <xsl:template name="epfslist">
    <xsl:param name="node"/>
    <xsl:param name="counter"/>
    <xsl:param name="listroot"/>

    <xsl:for-each select="$node">

      <!-- output next element -->
      <xsl:call-template name="epfsbranch">
        <xsl:with-param name="node" select='.'/>
        <xsl:with-param name="counter" select="position()"/>
        <xsl:with-param name="listroot" select="$listroot"/>
      </xsl:call-template>
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="epfsbranch">
    <xsl:param name="node"/>
    <xsl:param name="counter"/>
    <xsl:param name="listroot"/>
    <xsl:variable name="cfrom" select='$node/F[@name="CFROM"]/FS/@type'/>
    <xsl:variable name="cto" select='$node/F[@name="CTO"]/FS/@type'/>
    <xsl:variable name="lb" select='$node/F[@name="LB"]'/>
    <xsl:variable name="var" select='$node/F[@name="VAR"]'/>
    <xsl:variable name="rel" select='$node/F[@name="REL"]'/>
    <xsl:variable name="eptype" select='$node/@type'/>

    <xsl:choose>
      <!-- ep-rels -->
      <xsl:when test="substring($eptype,1,6)='ep-rel'">
        <xsl:variable name="eptype-posfull" select="substring($eptype,8,string-length($eptype)-7)"/>
        <xsl:variable name="eptype-subtype" select="substring-after($eptype-posfull,'_')"/>
        <xsl:variable name="eptype-stem">

          <xsl:choose>  
            <xsl:when test="$eptype-subtype">
              <xsl:value-of select="substring-before($eptype-posfull,'_')"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="$eptype-posfull"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <!-- e. g. ep-rel-art_def: eptype-posfull="art_def", 
                                   eptype-subtype="def",
                                   eptype-stem="art" -->
        <xsl:choose>
          <!-- ep-rels that are mapped to realpreds (check)-->
          <xsl:when test='$eptype-stem="adjective" or
                          $eptype-stem="adverb" or
                          $eptype-stem="adv" or
                          $eptype-stem="abbr" or
                          $eptype-stem="noun" or
                          $eptype-stem="verb" or
                          $eptype-stem="interjection" or
                          $eptype-stem="conj" or
                          $eptype-stem="card" or
                          $eptype-stem="part" or
                          $eptype-stem="pnoun" or                          
                          $eptype-stem="prep" or                          
                          $eptype-stem="other" or                          
                          $eptype-stem="u" or                          
                          $eptype-stem="" or                          
                          $eptype-stem="adp"'>
            <xsl:call-template name="write-pred">
              <xsl:with-param name="eprelfs" select="$node"/>
              <xsl:with-param name="lb" select="$lb"/>
              <xsl:with-param name="var" select="$var"/>
              <xsl:with-param name="rel" select="$rel"/>
              <xsl:with-param name="pos" select="$eptype-posfull"/>
              <xsl:with-param name="predtype" select='"realpred"'/>
              <xsl:with-param name="cfrom" select="$cfrom"/>
              <xsl:with-param name="cto" select="$cto"/>
              <xsl:with-param name="counter" select="$counter"/>
              <xsl:with-param name="listroot" select="$listroot"/>
            </xsl:call-template>
          </xsl:when>
          <!-- ep-rels that are mapped to gpreds -->
          <xsl:when test='$eptype-stem="art" or
                          $eptype-stem="pron" or
                          $eptype-stem="udef" or
                          $eptype-stem="def" or
                          $eptype-stem="prpstn" or
                          $eptype-stem="que" or
                          $eptype-stem="neg" or
                          $eptype-stem="unpec" or
                          $eptype-stem="which" or
                          $eptype-stem="poss"'>
            <xsl:call-template name="write-pred">
              <xsl:with-param name="eprelfs" select="$node"/>
              <xsl:with-param name="lb" select="$lb"/>
              <xsl:with-param name="var" select="$var"/>
              <xsl:with-param name="rel" select="$rel"/>
              <xsl:with-param name="pos" select="$eptype-posfull"/>
              <xsl:with-param name="predtype" select='"gpred"'/>
              <xsl:with-param name="cfrom" select="$cfrom"/>
              <xsl:with-param name="cto" select="$cto"/>
              <xsl:with-param name="counter" select="$counter"/>
              <xsl:with-param name="listroot" select="$listroot"/>
            </xsl:call-template>
          </xsl:when>
        </xsl:choose>
      </xsl:when>
      <!-- ep-arg types -->
      <xsl:when test="substring($eptype,1,6)='ep-arg'">
        <xsl:variable name="argnum" select="substring($eptype,7,string-length($eptype)-6)"/>

        <xsl:call-template name="write-rarg">
          <xsl:with-param name="argfs" select="$node"/>
          <xsl:with-param name="lb" select="$lb"/>
          <xsl:with-param name="argname" select="concat('ARG',$argnum)"/>
          <xsl:with-param name="counter" select="$counter"/>
          <xsl:with-param name="listroot" select="$listroot"/>
        </xsl:call-template>
      </xsl:when>

      <xsl:when test="substring($eptype,1,7)='ep-rstr'">
        <xsl:call-template name="write-rarg">
          <xsl:with-param name="argfs" select="$node"/>
          <xsl:with-param name="lb" select="$lb"/>
          <xsl:with-param name="argname" select="'RSTR'"/>
          <xsl:with-param name="counter" select="$counter"/>
          <xsl:with-param name="listroot" select="$listroot"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="substring($eptype,1,7)='ep-body'">
        <xsl:call-template name="write-rarg">
          <xsl:with-param name="argfs" select="$node"/>
          <xsl:with-param name="lb" select="$lb"/>
          <xsl:with-param name="argname" select="'BODY'"/>
          <xsl:with-param name="counter" select="$counter"/>
          <xsl:with-param name="listroot" select="$listroot"/>
        </xsl:call-template>
      </xsl:when>

    </xsl:choose>
  </xsl:template>


 
 <!-- writing eps for realpreds and gpreds -->
 <xsl:template name="write-pred">
   <xsl:param name="eprelfs"/>
   <xsl:param name="lb"/>
   <xsl:param name="var"/>	
   <xsl:param name="rel"/>	
   <xsl:param name="pos"/>	
   <xsl:param name="predtype"/>
   <xsl:param name="cfrom"/>
   <xsl:param name="cto"/>
   <xsl:param name="counter"/>
   <xsl:param name="listroot"/>

   <xsl:element name="ep">
     <xsl:attribute name="cfrom"><xsl:value-of select="translate($cfrom,'&quot;','')"/></xsl:attribute>
     <xsl:attribute name="cto"><xsl:value-of select="translate($cto,'&quot;','')"/></xsl:attribute>
     <xsl:element name="label">
       <xsl:variable name="corefvalue">
         <xsl:call-template name="searchcorefvalue">
           <xsl:with-param name="node" select="$listroot/FS[1]"/>
           <xsl:with-param name="counter" select="$counter"/>
           <xsl:with-param name="sort" select='"label"'/>
           <xsl:with-param name="corefname" select="$lb/FS/@coref"/>
           <xsl:with-param name="i" select="'1'"/>
         </xsl:call-template>
       </xsl:variable>
       <xsl:attribute name="vid"><xsl:value-of select='$corefvalue'/></xsl:attribute>
     </xsl:element>

     <xsl:variable name="lemma">
      <xsl:choose>
       <xsl:when test='$rel/FS/@type="string"'>
         <xsl:call-template name="strip-quotes">
           <xsl:with-param name="quoted">
              <xsl:value-of select='$rel/../../../../../../../../../F[@name="M-SYN"]/FS/F[@name="STEM"]/FS/@type'/>
           </xsl:with-param>
        </xsl:call-template>
       </xsl:when>
       <xsl:otherwise>
         <xsl:call-template name="strip-quotes">
           <xsl:with-param name="quoted">
              <xsl:value-of select='$rel/FS/@type'/>
           </xsl:with-param>
        </xsl:call-template>
       </xsl:otherwise>
      </xsl:choose>
     </xsl:variable>

     <xsl:choose>
     <!-- write realpreds -->
       <xsl:when test='$predtype="realpred"'>
         <xsl:element name="realpred">
           <xsl:attribute name="lemma">
             <xsl:value-of select='$lemma'/>
           </xsl:attribute>
           <xsl:attribute name="pos"><xsl:value-of select='$pos'/></xsl:attribute>
         </xsl:element>
       </xsl:when>
     <!-- write gpreds -->
       <xsl:when test='$predtype="gpred"'>
         <xsl:element name="gpred">
           <xsl:value-of select="concat('_', $lemma, '_', $pos, '_rel')"/>
         </xsl:element>
       </xsl:when>
     </xsl:choose>

   <xsl:call-template name="write-var">
     <xsl:with-param name="var" select="$var"/>   
     <xsl:with-param name="counter" select="$counter"/>
     <xsl:with-param name="listroot" select="$listroot"/>
   </xsl:call-template>
  </xsl:element>
 </xsl:template>

 <!-- writing rargs -->
 <xsl:template name="write-rarg">
   <xsl:param name="argfs"/>
   <xsl:param name="argname"/>
   <xsl:param name="counter"/>
   <xsl:param name="lb"/>
   <xsl:param name="listroot"/>

   <xsl:if test="not($argname='ARG0')">

     <xsl:element name="rarg">
       <xsl:element name="rargname">
         <xsl:value-of select='$argname'/>
       </xsl:element>

       <xsl:element name="label">
         <xsl:variable name="corefvalue">
           <xsl:call-template name="searchcorefvalue">
             <xsl:with-param name="node" select="$listroot/FS[1]"/>
             <xsl:with-param name="counter" select="$counter"/>
             <xsl:with-param name="sort" select='"label"'/>
             <xsl:with-param name="corefname" select="$lb/FS/@coref"/>
             <xsl:with-param name="i" select="'1'"/>
           </xsl:call-template>
         </xsl:variable>
         <xsl:attribute name="vid"><xsl:value-of select='$corefvalue'/></xsl:attribute>
       </xsl:element>

       <xsl:choose>
         <xsl:when test="substring($argname,1,3)='ARG'">
           <xsl:call-template name="write-var">
             <xsl:with-param name="var" select="$argfs/F[@name='ARGX']"/>
             <xsl:with-param name="counter" select="$counter"/>
             <xsl:with-param name="listroot" select="$listroot"/>
           </xsl:call-template>
         </xsl:when>
         <xsl:when test="$argname='RSTR'">
           <xsl:call-template name="write-var">
             <xsl:with-param name="var" select="$argfs/F[@name='RSTR']"/>
             <xsl:with-param name="counter" select="$counter"/>
             <xsl:with-param name="listroot" select="$listroot"/>
           </xsl:call-template>
         </xsl:when>
         <xsl:when test="$argname='BODY'">
           <xsl:call-template name="write-var">
             <xsl:with-param name="var" select="$argfs/F[@name='BODY']"/>
             <xsl:with-param name="counter" select="$counter"/>
             <xsl:with-param name="listroot" select="$listroot"/>
           </xsl:call-template>
         </xsl:when>
       </xsl:choose>
     </xsl:element>    

   </xsl:if> 

<!-- arg0 appears as "var" within element "ep", see template "write-pred" -->
<!--   <xsl:if test="$argname='ARG0'">
     <xsl:call-template name="write-var">
       <xsl:with-param name="var" select="$argfs/F[@name='ARG0']"/>
       <xsl:with-param name="counter" select="$counter"/>
       <xsl:with-param name="listroot" select="$listroot"/>
     </xsl:call-template>
   </xsl:if>
-->
 </xsl:template>

<!-- writing variables -->
 <xsl:template name="write-var">
    <xsl:param name="var"/>
    <xsl:param name="counter"/>
    <xsl:param name="listroot"/>

    <xsl:element name="var">
       <xsl:variable name="corefvalue">
         <xsl:call-template name="searchcorefvalue">
           <xsl:with-param name="node" select="$listroot/FS[1]"/>
           <xsl:with-param name="counter" select="$counter"/>
           <xsl:with-param name="sort" select='"var"'/>
           <xsl:with-param name="corefname" select="$var/FS/@coref"/>
           <xsl:with-param name="i" select="'1'"/>
         </xsl:call-template>
       </xsl:variable>
      <xsl:attribute name="vid"><xsl:value-of select='$corefvalue'/></xsl:attribute>
      <xsl:choose>
        <xsl:when test='$var/FS/@type="ref-var"'>
          <xsl:variable name="index" select="$var/FS/F[@name='PNG']/FS"/>
          <xsl:attribute name="sort"><xsl:value-of select='"x"'/></xsl:attribute>
          <xsl:if test="$index/F[@name='PERSON']/FS/@type">        
            <xsl:attribute name="person">
              <xsl:value-of select="$index/F[@name='PERSON']/FS/@type"/>
            </xsl:attribute>
          </xsl:if>
          <xsl:if test="$index/F[@name='NUMBER']/FS/@type">        
            <xsl:attribute name="number">
              <xsl:value-of select="$index/F[@name='NUMBER']/FS/@type"/>
            </xsl:attribute>
          </xsl:if>
          <xsl:if test="$index/F[@name='GENDER']/FS/@type">        
            <xsl:attribute name="gender">
              <xsl:value-of select="$index/F[@name='GENDER']/FS/@type"/>
            </xsl:attribute>
          </xsl:if>
        </xsl:when>
        <xsl:when test='$var/FS/@type="event-var"'>
          <xsl:variable name="index" select="$var/FS/F[@name='TM']/FS"/>
          <xsl:attribute name="sort"><xsl:value-of select='"e"'/></xsl:attribute>

          <xsl:if test="$index/F[@name='TENSE']/FS/@type">        
            <xsl:attribute name="tense">
              <xsl:value-of select="$index/F[@name='TENSE']/FS/@type"/>
            </xsl:attribute>
          </xsl:if>
          <xsl:if test="$index/F[@name='MOOD']/FS/@type">        
            <xsl:attribute name="mood">
              <xsl:value-of select="$index/F[@name='MOOD']/FS/@type"/>
            </xsl:attribute>
          </xsl:if>
        </xsl:when>
        <xsl:when test='$var/FS/@type="individual"'>
          <xsl:attribute name="sort"><xsl:value-of select='"u"'/></xsl:attribute>
        </xsl:when>
        <xsl:when test='$var/FS/@type="hole"'>
          <xsl:attribute name="sort"><xsl:value-of select='"h"'/></xsl:attribute>
        </xsl:when>
      </xsl:choose>
    </xsl:element>
  </xsl:template>



  <!-- process constraints (qeq etc.) -->
  <xsl:template name="hconsfslist">
    <xsl:param name="node"/>
    <xsl:param name="counter"/>
    <xsl:param name="cfrom"/>
    <xsl:param name="cto"/>
    <xsl:param name="listroot"/>
    <xsl:param name="listroot_rels"/>
    <!-- output next element -->

    <xsl:for-each select='$listroot/FS'>
    <xsl:call-template name="hconsfsbranch">
      <xsl:with-param name="node" select='.'/>
      <xsl:with-param name="counter" select="position()"/>
      <xsl:with-param name="cfrom" select="$cfrom"/>
      <xsl:with-param name="cto" select="$cto"/>
      <xsl:with-param name="listroot" select="$listroot"/>
      <xsl:with-param name="listroot_rels" select="$listroot_rels"/>
    </xsl:call-template>
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="hconsfsbranch">
    <xsl:param name="node"/>
    <xsl:param name="counter"/>
    <xsl:param name="cfrom"/>
    <xsl:param name="cto"/>
    <xsl:param name="listroot"/>
    <xsl:param name="listroot_rels"/>
    <xsl:variable name="hi" select='$node/F[@name="HI"]'/>
    <xsl:variable name="lo" select='$node/F[@name="LO"]'/>
    <xsl:variable name="hconstype" select='$node/@type'/>


    <xsl:choose>
      <!-- qeq-constrains -->
      <xsl:when test="$hconstype='qeq'">
            <xsl:call-template name="write-hcons">
              <xsl:with-param name="hconsfs" select="$node"/>
              <xsl:with-param name="hconstype" select="$hconstype"/>
              <xsl:with-param name="hi" select="$hi"/>
              <xsl:with-param name="lo" select="$lo"/>
              <xsl:with-param name="cfrom" select="$cfrom"/>
              <xsl:with-param name="cto" select="$cto"/>
              <xsl:with-param name="counter" select="$counter"/>
              <xsl:with-param name="listroot" select="$listroot"/>
              <xsl:with-param name="listroot_rels" select="$listroot_rels"/>
            </xsl:call-template>
      </xsl:when>

    </xsl:choose>
  </xsl:template>



  <!-- process ing-constraints  -->
  <xsl:template name="ingfslist">
    <xsl:param name="node"/>
    <xsl:param name="counter"/>
    <xsl:param name="cfrom"/>
    <xsl:param name="cto"/>
    <xsl:param name="listroot"/>
    <xsl:param name="listroot_rels"/>
    <!-- output next element -->

    <xsl:for-each select='$listroot/FS'>
    <xsl:call-template name="ingfsbranch">
      <xsl:with-param name="node" select='.'/>
      <xsl:with-param name="counter" select="position()"/>
      <xsl:with-param name="cfrom" select="$cfrom"/>
      <xsl:with-param name="cto" select="$cto"/>
      <xsl:with-param name="listroot" select="$listroot"/>
      <xsl:with-param name="listroot_rels" select="$listroot_rels"/>
    </xsl:call-template>
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="ingfsbranch">
    <xsl:param name="node"/>
    <xsl:param name="counter"/>
    <xsl:param name="cfrom"/>
    <xsl:param name="cto"/>
    <xsl:param name="listroot"/>
    <xsl:param name="listroot_rels"/>
    <xsl:variable name="ing-a" select='$node/F[@name="ING-A"]'/>
    <xsl:variable name="ing-b" select='$node/F[@name="ING-B"]'/>
    <xsl:variable name="ingtype" select='$node/@type'/>


    <xsl:choose>
      <!-- ing-constrains -->
      <xsl:when test="$ingtype='ing'">
            <xsl:call-template name="write-ing">
              <xsl:with-param name="ingfs" select="$node"/>
              <xsl:with-param name="ingtype" select="$ingtype"/>
              <xsl:with-param name="ing-a" select="$ing-a"/>
              <xsl:with-param name="ing-b" select="$ing-b"/>
              <xsl:with-param name="cfrom" select="$cfrom"/>
              <xsl:with-param name="cto" select="$cto"/>
              <xsl:with-param name="counter" select="$counter"/>
              <xsl:with-param name="listroot" select="$listroot"/>
              <xsl:with-param name="listroot_rels" select="$listroot_rels"/>
            </xsl:call-template>
      </xsl:when>

    </xsl:choose>
  </xsl:template>





<!-- writing constraints -->
 <xsl:template name="write-hcons">
    <xsl:param name="hconsfs"/>
    <xsl:param name="hconstype"/>
    <xsl:param name="hi"/>
    <xsl:param name="lo"/>
    <xsl:param name="cfrom"/>
    <xsl:param name="cto"/>
    <xsl:param name="counter"/>
    <xsl:param name="listroot"/>
    <xsl:param name="listroot_rels"/>

    <xsl:element name="hcons">
      <xsl:attribute name="hreln"><xsl:value-of select='$hconstype'/></xsl:attribute>

      <xsl:element name="hi">
       <xsl:element name="var">
          <xsl:variable name="corefvalue">
            <xsl:call-template name="searchcorefvalue">
              <xsl:with-param name="node" select="$listroot_rels/FS[1]"/>
              <xsl:with-param name="counter" select="$counter"/>
              <xsl:with-param name="sort" select='"hole"'/>
              <xsl:with-param name="corefname" select="$hconsfs/F[@name='HI']/FS/@coref"/>
              <xsl:with-param name="i" select="'1'"/>
            </xsl:call-template>
          </xsl:variable>
          <xsl:attribute name="vid"><xsl:value-of select='$corefvalue'/></xsl:attribute>
          <xsl:attribute name="sort"><xsl:value-of select="'h'"/></xsl:attribute>
       </xsl:element>
      </xsl:element>  

      <xsl:element name="lo">
        <xsl:element name="label">
          <xsl:variable name="corefvalue">
            <xsl:call-template name="searchcorefvalue">
              <xsl:with-param name="node" select="$listroot_rels/FS[1]"/>
              <xsl:with-param name="counter" select="$counter"/>
              <xsl:with-param name="sort" select='"label"'/>
              <xsl:with-param name="corefname" select="$hconsfs/F[@name='LO']/FS/@coref"/>
              <xsl:with-param name="i" select="'1'"/>
            </xsl:call-template>
          </xsl:variable>
          <xsl:attribute name="vid"><xsl:value-of select='$corefvalue'/></xsl:attribute>
        </xsl:element>
      </xsl:element>  

    </xsl:element>
  </xsl:template>



<!-- writing ings -->
 <xsl:template name="write-ing">
    <xsl:param name="ingfs"/>
    <xsl:param name="ingtype"/>
    <xsl:param name="ing-a"/>
    <xsl:param name="ing-b"/>
    <xsl:param name="cfrom"/>
    <xsl:param name="cto"/>
    <xsl:param name="counter"/>
    <xsl:param name="listroot"/>
    <xsl:param name="listroot_rels"/>

    <xsl:element name="ing">
<!--      <xsl:attribute name="hreln"><xsl:value-of
select='$hconstype'/></xsl:attribute>  -->

      <xsl:element name="ing-a">
       <xsl:element name="var">
          <xsl:variable name="corefvalue">
            <xsl:call-template name="searchcorefvalue">
              <xsl:with-param name="node" select="$listroot_rels/FS[1]"/>
              <xsl:with-param name="counter" select="$counter"/>
              <xsl:with-param name="sort" select='"label"'/>
              <xsl:with-param name="corefname" select="$ing-a/FS/@coref"/>
              <xsl:with-param name="i" select="'1'"/>
            </xsl:call-template>
          </xsl:variable>
          <xsl:attribute name="vid"><xsl:value-of select='$corefvalue'/></xsl:attribute>
          <xsl:attribute name="sort"><xsl:value-of select='"h"'/></xsl:attribute>
       </xsl:element>
      </xsl:element>  

      <xsl:element name="ing-b">
        <xsl:element name="var">
          <xsl:variable name="corefvalue">
            <xsl:call-template name="searchcorefvalue">
              <xsl:with-param name="node" select="$listroot_rels/FS[1]"/>
              <xsl:with-param name="counter" select="$counter"/>
              <xsl:with-param name="sort" select='"label"'/>
              <xsl:with-param name="corefname" select="$ingfs/F[@name='ING-B']/FS/@coref"/>
              <xsl:with-param name="i" select="'1'"/>
            </xsl:call-template>
          </xsl:variable>
          <xsl:attribute name="vid"><xsl:value-of select='$corefvalue'/></xsl:attribute>
          <xsl:attribute name="sort"><xsl:value-of select='"h"'/></xsl:attribute>
        </xsl:element>
      </xsl:element>  

    </xsl:element>
  </xsl:template>




  <xsl:template name="searchcorefvalue">
    <xsl:param name="i"/>
    <xsl:param name="counter"/>
    <xsl:param name="sort"/>
    <xsl:param name="corefname"/>
    <xsl:param name="node"/>

    <!-- check next element -->
    <!-- label corefs need to be treated separately in that
         their corefvalue must be taken from (the position of)
	 an ep-rel, for otherwise RSTRs are assigned the same
	 coref value as their label -->

    <xsl:choose>
      <xsl:when test='($sort = "label")'>
        <xsl:choose>
          <xsl:when test='(($node/F[@name="LB"]/FS/@coref = $corefname)
                           and substring($node/@type,1,6)="ep-rel")'>  
            <xsl:value-of select="$i"/>
          </xsl:when>
          <xsl:when test='$node/following-sibling::FS'>
            <xsl:call-template name="searchcorefvalue">
              <xsl:with-param name="node" select='$node/following-sibling::FS[1]'/>
              <xsl:with-param name="counter" select="$counter"/>
              <xsl:with-param name="i" select="$i + 1"/>
              <xsl:with-param name="sort" select="$sort"/>
              <xsl:with-param name="corefname" select="$corefname"/>
            </xsl:call-template>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$counter"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:otherwise>
        <xsl:choose>
          <xsl:when test='(($node/F[@name="VAR"]/FS/@coref = $corefname)
                            and ($sort = "var"))
                           or                                                      
                          (($node/F[@name="RSTR"]/FS/@coref = $corefname)
                            and ($sort = "hole"))
                           or
                          (($node/F[@name="BODY"]/FS/@coref = $corefname)
                            and ($sort = "hole"))'>
            <xsl:value-of select="$i"/>
          </xsl:when>
          <xsl:when test='$node/following-sibling::FS'>
            <!-- <xsl:text>|</xsl:text> -->
            <xsl:call-template name="searchcorefvalue">
              <xsl:with-param name="node" select='$node/following-sibling::FS[1]'/>
              <xsl:with-param name="counter" select="$counter"/>
              <xsl:with-param name="i" select="$i + 1"/>
              <xsl:with-param name="sort" select="$sort"/>
              <xsl:with-param name="corefname" select="$corefname"/>
            </xsl:call-template>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$counter"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- remove extra quotes from lemma -->
  <xsl:template name="strip-quotes">
    <xsl:param name="quoted"/>
      <xsl:variable name="stripped">
      <xsl:value-of select="substring($quoted, 2, string-length($quoted)-2)"/>
    </xsl:variable>
    <xsl:value-of select="$stripped"/>
  </xsl:template>


</xsl:stylesheet>
