<?xml version="1.0" encoding="utf-8"?>

<!-- adjust_nespans.xsl
*    Adjust RASP/PET ep spans to SPROUT ne ep spans
*
* (C) 2006  DFKI Language Technology Lab http://www.dfki.de/lt
*   Authors: Anette Frank, Anette.Frank@dfki.de
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Lesser General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public
*   License along with this library; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  
  <!-- adjust RASP/PET ep spans to SPROUT ne ep spans -->
  <!-- introduce temporary attributes: cfrom_org, cto_org, to be reset to cfrom/cto after merging -->
  <!-- applies to RASP/PET after merging ep/rarg into arg-s -->

  <xsl:output method="xml" indent="yes"/>
  <xsl:strip-space elements="*"/>

  <xsl:param name="uri">file:/tmp/uschaefer/hog/Sprout.xml</xsl:param>

  <xsl:template match="/*">
    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <xsl:apply-templates select="*"/>
    </xsl:copy>
  </xsl:template>
  
  <!-- elements that can be deep-copied -->
  <xsl:template match="metadata|ing|hcons">
    <xsl:copy-of select="."/>
  </xsl:template>

  <xsl:template match="rmrs">
    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <xsl:apply-templates select="*"/>   
    </xsl:copy>
  </xsl:template>
  
  <!-- matching rels from RASP, but before outputting, read NE-RMRS from Sprout, to adjust spans -->

  <xsl:template match="rels">
    <xsl:copy-of select="../label"/>
    <xsl:variable name="ne_rmrses" select="document($uri)/rmrs-list"/>
    <xsl:call-template name="adjust_spans">
      <xsl:with-param name="ne_rmrses" select="$ne_rmrses"/>
      <xsl:with-param name="rasp_rels" select="."/>
    </xsl:call-template>
    <xsl:apply-templates select="*"/>   
  </xsl:template>

  <!-- recurse over eps from RASP, determine and adjust RASP spans to potential RMRSs from SPROUT -->

  <xsl:template name="adjust_spans">
    <xsl:param name="ne_rmrses"/>
    <xsl:param name="rasp_rels"/>
    <xsl:element name="rels">
      <xsl:choose>
        <xsl:when test="$ne_rmrses/*">
          <xsl:for-each select="$rasp_rels/arg-s">         
            <xsl:element name="arg-s">
              <xsl:call-template name="write_ep">
                <xsl:with-param name="ep" select="ep"/>
                <xsl:with-param name="rargs" select="rarg"/>
                <xsl:with-param name="cfrom_rasp" select="ep/@cfrom"/>
                <xsl:with-param name="cto_rasp" select="ep/@cto"/>
                <xsl:with-param name="ne_rmrses" select="$ne_rmrses"/>
              </xsl:call-template>
            </xsl:element>
          </xsl:for-each>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy-of select="$rasp_rels/arg-s"/>
        </xsl:otherwise>
     </xsl:choose>
   </xsl:element>
  </xsl:template>

  <xsl:template name="write_ep">
    <xsl:param name="ep"/>
    <xsl:param name="rargs"/>
    <xsl:param name="cfrom_rasp"/>
    <xsl:param name="cto_rasp"/>
    <xsl:param name="ne_rmrses"/>    
    <xsl:variable name="cfrom_sprout">
      <xsl:for-each select="$ne_rmrses/rmrs">
        <xsl:if test="@cfrom &lt;= $cfrom_rasp and @cto &gt;= $cto_rasp">
          <xsl:value-of select="@cfrom"/>
        </xsl:if>
      </xsl:for-each>
    </xsl:variable>
    <xsl:variable name="cto_sprout">
      <xsl:for-each select="$ne_rmrses/rmrs">
        <xsl:if test="@cfrom &lt;= $cfrom_rasp and @cto &gt;= $cto_rasp">
          <xsl:value-of select="@cto"/>
        </xsl:if>
      </xsl:for-each>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="($cfrom_rasp &gt;= $cfrom_sprout and $cto_rasp &lt;= $cto_sprout) and not($cfrom_rasp = $cfrom_sprout and $cto_rasp = $cto_sprout)"> 
        <xsl:element name="ep">
          <xsl:attribute name="cfrom">
            <xsl:value-of select="$cfrom_sprout"/>
          </xsl:attribute>
          <xsl:attribute name="cto">
            <xsl:value-of select="$cto_sprout"/>
          </xsl:attribute>
          <xsl:attribute name="cfrom_atom">
            <xsl:value-of select="ep/@cfrom"/>
          </xsl:attribute>
          <xsl:attribute name="cto_atom">
            <xsl:value-of select="ep/@cto"/>
          </xsl:attribute>
          <xsl:attribute name="surface">
            <xsl:value-of select="ep/@surface"/>
          </xsl:attribute>
          <xsl:for-each select="ep/*">
            <xsl:copy-of select="."/>
          </xsl:for-each>
        </xsl:element>
        <xsl:for-each select="$rargs">
          <xsl:copy-of select="."/>
        </xsl:for-each>
      </xsl:when>
      <xsl:otherwise>
        <xsl:copy-of select="*"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="text()"/>

</xsl:stylesheet>


