<?xml version="1.0" encoding="utf-8"?>

<!-- merge-ne-to-rasp.xsl
*    insert NE types into RASP/PET RMRS
*
* (C) 2006  DFKI Language Technology Lab http://www.dfki.de/lt
*   Authors: Anette Frank, Anette.Frank@dfki.de
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Lesser General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public
*   License along with this library; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  
  <!-- insert ne-types output by sprout in sprout.xml -->

  <xsl:output method="xml" indent="yes"/>
  <xsl:strip-space elements="*"/>

  <xsl:param name="uri">file:/tmp/uschaefer/hog/Sprout.xml</xsl:param>

  <xsl:template match="/*">
    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <xsl:apply-templates select="*"/>
    </xsl:copy>
  </xsl:template>
  
  <!-- elements that can be deep-copied -->
  <xsl:template match="metadata|arg-s|ing|hcons">
    <xsl:copy-of select="."/>
  </xsl:template>

  <xsl:template match="rmrs">
    <xsl:copy>
      <xsl:for-each select="@*">
        <xsl:choose>
          <xsl:when test=".='&#10;'">
            <xsl:attribute name="surface">
              <xsl:copy-of select="translate(document($uri)/text/text(),'&#10;','')"/>
            </xsl:attribute>
          </xsl:when>   
          <xsl:otherwise>
            <xsl:copy-of select="."/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:for-each>
      <xsl:apply-templates/>   
    </xsl:copy>
  </xsl:template>

  <!-- matching rels from RASP, but before outputting, process NE-RMRS from Sprout -->
  <xsl:template match="rels">
    <xsl:copy-of select="../label"/>
    <xsl:element name="rels">
      <xsl:variable name="ne_rmrses" select="document($uri)/rmrs-list"/>
      <xsl:call-template name="make_ne_eps_rargs">
        <xsl:with-param name="ne_rmrses" select="$ne_rmrses"/>
        <xsl:with-param name="rasp_rels" select="."/>
      </xsl:call-template>
      <xsl:apply-templates select="*"/>   
    </xsl:element>
  </xsl:template>

  <!-- fish appropriate eps from Sprout, determine corresponding RASP vars and rewrite eps and rargs as arg-s -->
  <xsl:template name="make_ne_eps_rargs">
    <xsl:param name="ne_rmrses"/>
    <xsl:param name="rasp_rels"/>    
    <xsl:for-each select="$ne_rmrses/rmrs">
      <xsl:variable name="cstart" select="@cfrom"/>
      <xsl:variable name="cend" select="@cto"/>
      <xsl:variable name="rmrs_label" select="label/@vid"/>
      <xsl:variable name="ne_ep" select="ep[@cfrom=$cstart and @cto=$cend]"/>
      <xsl:variable name="ne_ep_var" select="ep[@cfrom=$cstart and @cto=$cend]/var/@vid"/>
      <xsl:variable name="ne_rargs" select="rarg/label[@vid=$ne_ep_var]/.."/>
      <xsl:variable name="rasp_rels_for_ne" select="$rasp_rels/arg-s/ep[@cfrom=$cstart and @cto=$cend]/.."/>
      <xsl:variable name="rasp_anchor_var">
        <xsl:value-of select="$rasp_rels_for_ne/ep/var/@vid"/>
      </xsl:variable>
      <xsl:variable name="ne_anchor_var">
        <!-- xsl:value-of select="$ne_ep_var"/ -->
        <xsl:value-of select="ep[@cfrom=$cstart and @cto=$cend]/label[@vid=$rmrs_label]/../var/@vid"/>
      </xsl:variable>
      <xsl:for-each select="ep[@cfrom=$cstart and @cto=$cend]">         
        <xsl:variable name="local_label" select="label/@vid"/>      
        <xsl:element name="arg-s">
          <xsl:element name="ep">
            <xsl:copy-of select="@*"/>
            <xsl:call-template name="replacevars_in_ep">
              <xsl:with-param name="rasp_rels_for_ne_span" 
                              select="$rasp_rels_for_ne/ep[@cfrom=$cstart and @cto=$cend]/.."/>
              <xsl:with-param name="rasp_anchor_var" select="$rasp_anchor_var"/>
              <xsl:with-param name="ne_anchor_var" select="$ne_anchor_var"/>
              <xsl:with-param name="ne_ep" select="."/>
            </xsl:call-template>
          </xsl:element>
          <xsl:for-each select="../rarg/label[@vid=$local_label]/..">
            <xsl:call-template name="replacevars_in_rargs">
              <xsl:with-param name="rasp_rels_for_ne_span" 
                              select="$rasp_rels_for_ne/ep[@cfrom=$cstart and @cto=$cend]/.."/>
              <xsl:with-param name="rasp_anchor_var" select="$rasp_anchor_var"/>
              <xsl:with-param name="ne_anchor_var" select="$ne_anchor_var"/>
              <xsl:with-param name="ne_rarg" select="."/>
            </xsl:call-template>
          </xsl:for-each>
        </xsl:element>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:template>

  <!-- REWRITING Sprout EPs -->
  <xsl:template name="replacevars_in_ep">
    <xsl:param name="rasp_rels_for_ne_span"/>     
    <xsl:param name="rasp_anchor_var"/>     
    <xsl:param name="ne_anchor_var"/>     
    <xsl:param name="ne_ep"/>     
    <xsl:variable name="offset" select="$ne_ep/@cto"/>
    <xsl:copy-of select="$ne_ep/gpred|$ne_ep/realpred"/>
    <xsl:element name="label">
      <xsl:attribute name="vid">
        <xsl:value-of select="$ne_ep/label/@vid + $offset * 1000000"/>
      </xsl:attribute>
    </xsl:element>
    <xsl:choose>
      <xsl:when test="$ne_ep/var/@vid=$ne_anchor_var">
        <xsl:element name="var">
          <xsl:attribute name="vid">
            <xsl:value-of select="$rasp_anchor_var"/>
          </xsl:attribute>
          <xsl:copy-of select="$ne_ep/var/@sort"/>
        </xsl:element>
      </xsl:when>
      <xsl:otherwise>
        <xsl:element name="var">
          <xsl:attribute name="vid">
            <xsl:value-of select="$ne_ep/var/@vid + $offset * 1000000"/>
          </xsl:attribute>
          <xsl:copy-of select="$ne_ep/var/@sort"/>
        </xsl:element>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- REWRITING Sprout RARGs -->
  <xsl:template name="replacevars_in_rargs">
    <xsl:param name="rasp_rels_for_ne_span"/>     
    <xsl:param name="rasp_anchor_var"/>     
    <xsl:param name="ne_anchor_var"/>
    <xsl:param name="ne_rarg"/>     
    <xsl:variable name="offset" select="$rasp_rels_for_ne_span/ep/@cto"/>
    <xsl:element name="rarg">
      <xsl:element name="label">
        <xsl:attribute name="vid">
          <xsl:value-of select="$ne_rarg/label/@vid + $offset * 1000000"/>
        </xsl:attribute>
      </xsl:element>
      <xsl:copy-of select="$ne_rarg/rargname"/>
      <xsl:if test="$ne_rarg/constant=true()">
        <xsl:copy-of select="$ne_rarg/constant"/>
      </xsl:if>
      <xsl:if test="$ne_rarg/var=true()">
        <xsl:choose>
          <xsl:when test="$ne_rarg/var/@vid=$ne_anchor_var">
            <xsl:element name="var">
              <xsl:attribute name="vid">
                <xsl:value-of select="$rasp_anchor_var"/>
              </xsl:attribute>
              <xsl:copy-of select="$ne_rarg/var/@sort"/>
            </xsl:element>
          </xsl:when>
          <xsl:otherwise>
            <xsl:element name="var">
              <xsl:attribute name="vid">
                <xsl:value-of select="$ne_rarg/var/@vid + $offset * 1000000"/>
              </xsl:attribute>
              <xsl:copy-of select="$ne_rarg/var/@sort"/>
            </xsl:element>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:if>
    </xsl:element>
  </xsl:template>
  
  <xsl:template name="get_rasp_anchorvar">
    <xsl:param name="current_node"/>
    <xsl:param name="local_label"/>
    <xsl:param name="rasp_rels_for_ne_var"/>
    <xsl:if test="not($current_node/../rarg/label[@vid=$local_label]/../rargname='ARG1')">
      <xsl:value-of select="$rasp_rels_for_ne_var/@vid"/>  
    </xsl:if>
  </xsl:template>

  <xsl:template name="get_ne_anchorvar">
    <xsl:param name="current_node"/>
    <xsl:param name="local_label"/>
    <xsl:if test="not($current_node/../rarg/label[@vid=$local_label]/../rargname='ARG1')">
      <xsl:value-of select="var/@vid"/>
    </xsl:if>
  </xsl:template>

  <xsl:template match="text()"/>

</xsl:stylesheet>

