<?xml version="1.0" encoding="utf-8"?>

<!-- reorderrmrsdtrs.xsl
*    Reorder <rmrs> daughter nodes according to RMRS DTD definition
*
* (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
*   Authors: Ulrich Schaefer Ulrich.Schaefer@dfki.de
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Lesser General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public
*   License along with this library; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0">

  <!-- reorder <rmrs> daughter nodes according to RMRS DTD definition -->
  <!-- !! this stylesheet has to be adapted when DTD changes !! -->
  <!-- 2004-06-21 Ulrich.Schaefer@dfki.de -->
  <!-- 2005-03-16: added surface attribute reconstruction from xmltext annotation -->

  <xsl:output method="xml" indent="yes"/>
  <xsl:strip-space elements="*"/>

  <!-- get surface string from xmltext annotation -->
  <xsl:param name="uri">file:/tmp/uschaefer/hog/xmltext.xml</xsl:param>
  <xsl:variable name="surface" select="document($uri)/text/text()"/>

  <!-- start at root element (pet|/rasp|/rmrs-list)-->
  <xsl:template match="/*">
    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="rmrs">
    <!-- special handling for <rmrs> daughter elements -->
    <!-- DTD fragment: ELEMENT rmrs (text?, label, ep*, rarg*, ing*, hcons*) -->
    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <xsl:if test="@cfrom and $surface!=''">
        <xsl:attribute name="surface">
          <xsl:value-of select="substring($surface, @cfrom + 2, @cto - @cfrom + 1)"/>
        </xsl:attribute>
      </xsl:if>
      <xsl:apply-templates select="text"/>
      <xsl:apply-templates select="label"/>
      <xsl:apply-templates select="ep"/>
      <xsl:apply-templates select="rarg"/>
      <xsl:apply-templates select="ing"/>
      <xsl:apply-templates select="hcons"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="ep|hcons|text|c|label|rarg|rargname|constant|ing|ing-a|ing-b|metadata|conf|id|entry|gpred|realpred|var|hi|lo">
    <!-- copy all other nodes -->
    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <xsl:if test="@cfrom and $surface!=''">
        <xsl:attribute name="surface">
          <xsl:value-of select="substring($surface, @cfrom + 2, @cto - @cfrom + 1)"/>
        </xsl:attribute>
      </xsl:if>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

</xsl:stylesheet>
