<?xml version="1.0" encoding="iso-8859-1"?>

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  
  <!-- copy the input TIGER-XML structure, add attribute lemma for  -->
  <!-- elements <t>, i.e. for terminals                             -->
  <!-- 2005-09-12 Kathrin Spreyer kspreyer@dfki.de  -->

  <!-- 
   (C) 2004 - 2005  DFKI Language Technology Lab http://www.dfki.de/lt
   Author: Kathrin Spreyer kspreyer@dfki.de

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
  -->

  <xsl:output method="xml" indent="yes"/>
  <xsl:strip-space elements="*"/>

  <xsl:param name="uri" select="'treetagger.xml'"/>

  <!-- copy nodes, start at root element -->
  <xsl:template match="/">
    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <xsl:apply-templates select="*"/>
    </xsl:copy>
  </xsl:template>


  <!-- elements that can be deep-copied -->
  <xsl:template match="head|nonterminals|sem|text()">
    <xsl:copy-of select="."/>
  </xsl:template>


  <!-- elements on future path to lemma -->
  <xsl:template match="corpus|body|s|graph|terminals">
    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <xsl:apply-templates select="*"/>
    </xsl:copy>
  </xsl:template>


  <!-- insert attribute lemma for terminals -->
  <xsl:template match="t">

    <xsl:variable name="lemma">
      <xsl:call-template name="get-lemma">
        <!-- cstart/cend are encoded in attribute id in TIGER-XML, 
          separated by '_' -->
        <xsl:with-param name="cstart" select="substring-before(@id,'_')"/>
        <xsl:with-param name="cend" select="substring-after(@id,'_')"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <xsl:attribute name="lemma">
        <xsl:value-of select="$lemma"/>
      </xsl:attribute>
<!--
      <xsl:apply-templates select="*"/>
-->
    </xsl:copy>
  </xsl:template>


  <!--go to TreeTagger output, get + return lemma for corresponding span -->
  <xsl:template name="get-lemma">
    <xsl:param name="cstart"/>
    <xsl:param name="cend"/>
    <xsl:value-of select="document($uri)/treetagger/tokens/t[@cstart=$cstart and @cend=$cend]/@lemma"/>
  </xsl:template>

</xsl:stylesheet>

