<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">


<!-- combine multiple smaf documents (in $urilist) into a single document

   (C) 2007 DFKI Language Technology Lab http://www.dfki.de/lt
   author: Ulrich.Schaefer@dfki.de

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
-->


  <xsl:output method="xml" omit-xml-declaration="yes" indent="yes"/>
  <xsl:strip-space elements="*"/>

  <xsl:param name="urilist"/>

  <xsl:template match = "/">

    <xsl:variable name = "init"><xsl:value-of select="/smaf/lattice/@init"/></xsl:variable>
    <xsl:variable name = "final"><xsl:value-of select="/smaf/lattice/@final"/></xsl:variable>
    <xsl:variable name = "cfrom"><xsl:value-of select="/smaf/lattice/@cfrom"/></xsl:variable>
    <xsl:variable name = "cto"><xsl:value-of select="/smaf/lattice/@cto"/></xsl:variable>


  <smaf document="">

  <text><xsl:value-of select="/smaf/text"/></text>
  <lattice>
    <xsl:attribute name = "init"><xsl:value-of select="$init"/></xsl:attribute>
    <xsl:attribute name = "final"><xsl:value-of select="$final"/></xsl:attribute>
    <xsl:attribute name = "cfrom"><xsl:value-of select="$cfrom"/></xsl:attribute>
    <xsl:attribute name = "cto"><xsl:value-of select="$cto"/></xsl:attribute>

  <xsl:call-template name = "combine_edges">
    <xsl:with-param name="urilist" select="$urilist" />
  </xsl:call-template>

  </lattice>
  </smaf>
</xsl:template>

<xsl:template name = "combine_edges">
      <xsl:param name = "urilist" />
      <xsl:variable name = "car" />
  
      <!-- iterate through all nodes -->
      <xsl:for-each select = ".">
        <!-- copy every edge -->
        <xsl:copy-of select = "//edge" />  
        <!-- find last 'edge' node -->
        <xsl:if test = "//edge[position() = last()]">
          <xsl:choose>
            <xsl:when test='contains($urilist, ",")'>
              <xsl:variable name="car" select = 'substring-before($urilist, ",")' />
              <!-- insert here the other edges! -->
              <xsl:copy-of select = "document($car)/smaf/lattice/edge" />
              <xsl:call-template name="combine_edges">
                <xsl:with-param name="urilist" select = 'substring-after($urilist, ",")' />
              </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
              <xsl:copy-of select = "document($urilist)/smaf/lattice/edge" />
            </xsl:otherwise>
          </xsl:choose>
        </xsl:if>
      </xsl:for-each>
  </xsl:template>

</xsl:stylesheet>
