<?xml version='1.0'?>
<xsl:stylesheet version='1.0' xmlns:xsl='http://www.w3.org/1999/XSL/Transform'>

  <!-- Translates XML generated by LingPipeModule to SMAF
       remark: creates no POS-nodes because lingpipe does not give any 

   (C) 2007  DFKI Language Technology Lab http://www.dfki.de/lt
   Author: Daniel Contag, daniel.contag@dfki.de

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
-->

  <!-- Date: 2007-08-16 -->

  <xsl:output method='xml' indent="yes"/>
  <xsl:strip-space elements="*"/>
  <xsl:output doctype-system="smaf.dtd"/>

  <!-- process root element -->
  <xsl:template match='/LINGPIPE'>
    <smaf>
      
    <!-- required - what value to use here? -->
    <xsl:attribute name = "document" />

    <!-- extract text from NEs -->
    <text>
      <xsl:for-each select = "NE">
        <xsl:value-of select = "@string" />
        <xsl:text> </xsl:text>
      </xsl:for-each>
    </text>

    <xsl:variable name = "numberOfTokens"><xsl:value-of select = "count(NE)"/></xsl:variable>
    <lattice>
      <xsl:attribute name = "init">v0</xsl:attribute>
      <xsl:attribute name = "final">
        <xsl:value-of select="concat('v', $numberOfTokens)" />
      </xsl:attribute>
      <xsl:attribute name = "cfrom">0</xsl:attribute>

      <!-- select last NE from end of document and get 'cend' -->
      <xsl:attribute name = "cto">
        <xsl:value-of select = "child::NE[position() = last()]/@cend + 1" />
      </xsl:attribute>

    <!-- named entities -->
    <xsl:for-each select = "NE">
      <xsl:call-template name = "edge_ne" />
    </xsl:for-each>

    </lattice>  
    </smaf>
  </xsl:template>

  <!-- generate pet input chart element for enamex node 
       (taken from pic/en_news_lingpipe2pixml.xsl)
  -->
  <xsl:template name = "edge_ne">
    <xsl:variable name='hpsgtype'>
      <xsl:choose>
        <xsl:when test='@type = "PERSON"'>
          <xsl:value-of select='"generic_name"'/>
        </xsl:when>

        <xsl:when test='@type = "LOCATION"'>
          <xsl:value-of select='"generic_name"'/>
        </xsl:when>

        <xsl:when test='@type = "ORGANIZATION"'>
          <xsl:value-of select='"generic_name"'/>
        </xsl:when>
        
        <xsl:otherwise>
          <xsl:value-of select='"nope"'/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:if test='$hpsgtype != "nope"'>
      <xsl:element name='edge'>
        <xsl:attribute name='type'>namedEntity</xsl:attribute>
        <xsl:attribute name = "id">
          <xsl:value-of select = "concat('n', position() - 1)" />
        </xsl:attribute>
        <xsl:attribute name='cfrom'>
          <xsl:value-of select='@cstart'/>
        </xsl:attribute>
        <xsl:attribute name='cto'>
          <xsl:value-of select='@cend + 1'/>
        </xsl:attribute>
        <xsl:attribute name = "source">
          <xsl:value-of select = "concat('v', position() - 1)" />
        </xsl:attribute>
        <xsl:attribute name = "target">
          <xsl:value-of select = "concat('v', position())" />
        </xsl:attribute>
        <xsl:if test = "@prio">
          <slot>
            <xsl:attribute name = "name">weight</xsl:attribute>
              <xsl:value-of select = "@prio" />
          </slot>
        </xsl:if>
        <slot>    
          <xsl:attribute name = "name">surface</xsl:attribute>
          <xsl:value-of select = "@string" />
        </slot>

        <fs>
          <xsl:attribute name = "type" >
            <xsl:value-of select = "$hpsgtype" />
          </xsl:attribute>
        </fs>
      </xsl:element>
    </xsl:if>

  </xsl:template>

</xsl:stylesheet>
