<?xml version="1.0" encoding="utf-8"?>

<!-- converts pixml to SMAF -->	

<!-- 
   (C) 2007 DFKI Language Technology Lab http://www.dfki.de/lt
   Author: Daniel Contag, daniel.contag@dfki.de

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  
  <xsl:output method="xml" indent="yes"/>
  <xsl:strip-space elements="*"/>

  <!-- generates <!DOCTYPE smaf SYSTEM 'smaf.dtd'> in output-->
  <xsl:output doctype-system="smaf.dtd"/>


<xsl:template match = "pet-input-chart">
  <smaf>
    <!-- required - what value to use here? -->
    <xsl:attribute name = "document" />

    <!-- extract text but not again from named entity entries (which have typeinfos) -->
    <text>
      <xsl:for-each select = "w[not(typeinfo)]">
        <xsl:value-of select = "surface/text()" />
        <xsl:text> </xsl:text>
      </xsl:for-each>
    </text>
    
    <xsl:variable name = "numberOfTokens"><xsl:value-of select = "count(w[not(typeinfo)])"/></xsl:variable>
    <lattice>
      <xsl:attribute name = "init">v0</xsl:attribute>
      <xsl:attribute name = "final">
        <xsl:value-of select="concat('v', $numberOfTokens)" />
      </xsl:attribute>
      <xsl:attribute name = "cfrom">0</xsl:attribute>

      <!-- select last w from end of document and get 'cend' which is smaf for 'cto' in pixml -->
      <xsl:attribute name = "cto">
        <xsl:value-of select = "child::w[not(typeinfo)][position() = last()]/@cend + 1" />
      </xsl:attribute>

    <!-- iterate over w's and convert to 3 types of edges -->
    <xsl:for-each select = "w[not(typeinfo)]">
      <xsl:call-template name = "edge_token" />
    </xsl:for-each>

    <xsl:for-each select = "w[not(typeinfo)]">
      <xsl:call-template name = "edge_pos" /> 
    </xsl:for-each>

    <!-- named entities -->
    <xsl:for-each select = "w[typeinfo]">
      <xsl:call-template name = "edge_ne" />
    </xsl:for-each>

  </lattice>
  </smaf>
</xsl:template>


<!-- generates token edges with text -->
<xsl:template name = "edge_token" >
  <edge>
    <xsl:attribute name = "type">token</xsl:attribute>
    <xsl:attribute name = "id">
      <xsl:value-of select = "concat('t', position())" />
    </xsl:attribute>
    
    <xsl:attribute name = "cfrom">
      <xsl:value-of select = "@cstart" />
    </xsl:attribute>
    
    <xsl:attribute name = "cto">
      <xsl:value-of select = "@cend + 1" />
    </xsl:attribute>

    <xsl:attribute name = "source">
      <xsl:value-of select = "concat('v', position() - 1)" />
    </xsl:attribute>

    <xsl:attribute name = "target">
      <xsl:value-of select = "concat('v', position())" />
    </xsl:attribute>

    <xsl:value-of select = "surface/text()" />
  </edge>
</xsl:template>


<!-- generates edges with slots (slots contain pos-tags) -->
<xsl:template name = "edge_pos" >
  <edge>
    <xsl:attribute name = "type">pos</xsl:attribute>

    <xsl:attribute name = "id">
      <xsl:value-of select = "concat('p', position())" />
    </xsl:attribute>
    
    <xsl:attribute name = "source">
      <xsl:value-of select = "concat('v', position() - 1)" />
    </xsl:attribute>

    <xsl:attribute name = "target">
      <xsl:value-of select = "concat('v', position())" />
    </xsl:attribute>

    <xsl:attribute name = "deps">
      <xsl:value-of select = "concat('t', position())" />
    </xsl:attribute>
  <xsl:for-each select="pos" >
    <slot>
      <xsl:attribute name = "name">tag</xsl:attribute>
      <xsl:value-of select = "@tag" />
    </slot>

    <xsl:if test = "@prio">
      <slot>
        <xsl:attribute name = "name">weight</xsl:attribute>
        <xsl:value-of select = "@prio" />
      </slot>
    </xsl:if>

  </xsl:for-each>
  </edge>

</xsl:template>


<!-- handle named entities -->
<xsl:template name = "edge_ne" >

  <edge>
    <xsl:attribute name = "type">namedEntity</xsl:attribute>    

    <xsl:attribute name = "id">
      <xsl:value-of select = "concat('n', position())" />
    </xsl:attribute>
    
    <xsl:attribute name = "cfrom">
      <xsl:value-of select = "@cstart" />
    </xsl:attribute>
    
    <xsl:attribute name = "cto">
      <xsl:value-of select = "@cend + 1" />
    </xsl:attribute>

    <xsl:attribute name = "source">
      <xsl:value-of select = "concat('v', position() - 1)" />
    </xsl:attribute>

    <xsl:attribute name = "target">
      <xsl:value-of select = "concat('v', position())" />
    </xsl:attribute>

    <xsl:text>
      </xsl:text>

    <xsl:if test = "@prio">
      <slot>
        <xsl:attribute name = "name">weight</xsl:attribute>
        <xsl:value-of select = "@prio" />
      </slot>
    </xsl:if>

    <xsl:text>
      </xsl:text>

    <slot>    
      <xsl:attribute name = "name">surface</xsl:attribute>
      <xsl:value-of select = "surface/text()" />
    </slot>

    <xsl:text>
      </xsl:text>

    <fs>
      <xsl:attribute name = "type" >
        <xsl:value-of select = "typeinfo/stem/text()" />
      </xsl:attribute>
    </fs>

    <xsl:text>
    </xsl:text>

  </edge>

</xsl:template>

</xsl:stylesheet>
