<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!-- This xslt stylesheet extracts brackets from topo-trees to be 
     delivered to HPSG parser.
     To be processed in a cascade of xslt-transformations

   (C) 2002  DFKI Language Technology Lab http://www.dfki.de/lt
   Author: Anette Frank, anette.frank@dfki.de

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
-->


<xsl:output method="xml" indent="yes"/>
<xsl:strip-space elements="*"/>

  <xsl:template match="/">
    <xsl:element name="TOPOMAP">
      <xsl:copy-of select="*/@*"/>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>


<!-- Most rules apply (recursively) to clause node CL (exception: ROOT node of MF tree) 
     We distinguish various constructions, and call the respective templates for bracket extraction
-->


<!-- ===================================================
     Toplevel rule with construction-wise template calls
     Matches: CL
     =================================================== -->
 
  <xsl:template match="CL">
<!-- ==================
     rule : vlast_subcl    (was Rule 1)
     ================== -->
       <xsl:if test='@fn = "SUBCL"'>
           <xsl:call-template name="vlast_subcl"/>
       </xsl:if>

<!-- ==================
     rule : vlast_relwh    (was Rule 2)
     ================== -->
       <xsl:if test='(@fn = "WH") or (@fn = "REL")'>
           <xsl:call-template name="vlast_relwh"/>
       </xsl:if>

<!-- ================
     rule : vlast_inf      (was Rule 6)
     ================ -->
       <xsl:if test='(@fn = "INF") and (count(LK) = 0) and RK'>
           <xsl:call-template name="vlast_inf"/>
       </xsl:if>

<!-- ===================
     rule : vlast_fin_rk   (was Rule 45)
     =================== -->
       <xsl:if test="RK/VVFIN | RK/VMFIN | RK/VAFIN">  <!-- isFinite -->
           <xsl:call-template name="vlast_fin_rk"/>
       </xsl:if>

<!-- ===================
     rule : vlast_inf_rk  (was Rule 3)
     =================== -->
       <xsl:if test='./LK/@fn = "COMPL"'>
           <xsl:call-template name="vlast_inf_rk"/>
       </xsl:if>

<!-- ===============
     rule : vfronted      (was Rule 7)
     =============== -->
       <xsl:if test='((@fn = "V1") or (@fn = "V2")) and (count(KON) = 0) and (LK/@fn = "VFIN")'>
           <xsl:call-template name="vfronted"/>
       </xsl:if>

<!-- =====================
     rule : extrapos  (was Rule 8)
     ===================== -->
       <xsl:if test="NF and RK">
           <xsl:call-template name="extrapos"/>
       </xsl:if>

<!-- =====================
     rule : cp_coord (was Rule 13)
     ===================== -->
       <xsl:if test='KON and (count(preceding::CL)=0) and (../CL and last())'>
       <xsl:call-template name="cp_coord"/>
       </xsl:if>
     <xsl:apply-templates/>
  </xsl:template>


<!-- =====================================
     rule "root_mf": phrasal MF under ROOT  (freely applying recursive template) 
     matches: single MF dtr of ROOT
     brackets: "non-clausal-mf"  => span of MF 
     ===================================== -->

  <xsl:template match="ROOT"> 
     <xsl:copy-of select="*"/>
     <xsl:if test="MF and (count(*) = 1)">
       <xsl:variable name="leftcorner">
          <xsl:call-template name="getLeftCorner">
             <xsl:with-param name="nodeset" select="MF//W"/>
          </xsl:call-template>
       </xsl:variable>
       <xsl:variable name="rightcorner">
          <xsl:call-template name="getRightCorner">
             <xsl:with-param name="nodeset" select="MF//W"/>
          </xsl:call-template>
       </xsl:variable>
       <xsl:call-template name="bracket">
          <xsl:with-param name="map_constr" select='"non-clausal-mf"'/>
          <xsl:with-param name="left" select="$leftcorner"/>
          <xsl:with-param name="right" select="$rightcorner"/>
          <xsl:with-param name="rule" select='"root_mf"'/>
       </xsl:call-template>
     </xsl:if>
    <xsl:apply-templates/>
  </xsl:template>



<!-- =====================================
     rule "vlast_subcl":                        (was Rule 1)
     matches: CL-SUBCL with LK-COMPL dtr, and no KON-dtr
     brackets: "vl_cpfin_compl" => span of cp
               "vl_cpinf_compl" => span of cp
               "vl_compl_vp"    => span of vp
     called templates: "vl_in_coord" 
                        (=> brackets: "vl_vp", "vp_coord",
                                       "vl_cpfin_compl" "vl_cpinf_compl")
     ===================================== -->

  <xsl:template name="vlast_subcl">

     <xsl:if test='(count(KON) = 0) and (LK/@fn = "COMPL")'>

        <xsl:variable name="wchildren" select=".//W"/>
        <xsl:variable name="leftcorner">
                <xsl:call-template name="getLeftCorner">
                  <xsl:with-param name="nodeset" select="$wchildren"/>
                </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="rightcorner">
                <xsl:call-template name="getRightCorner">
                  <xsl:with-param name="nodeset" select="$wchildren"/>
                </xsl:call-template>
        </xsl:variable>

        <xsl:choose>   <!-- call rule "vl_in_coord" if embedded in coord-structure -->

          <xsl:when test='(../KON) and (count(preceding::CL)=0) and (../CL[(@fn="VP") and last() ]) '>
             <xsl:call-template name="vl_in_coord">
                 <xsl:with-param name="leftcorner" select="$leftcorner"/>
                 <xsl:with-param name="rightcorner" select="$rightcorner"/>
             </xsl:call-template> 
          </xsl:when>            
          <xsl:otherwise>  
            <!-- original Rule continues -->
            <xsl:choose>
             <xsl:when test="RK/VVFIN | RK/VMFIN | RK/VAFIN">  <!-- isFinite -->
               <xsl:call-template name="bracket">
                  <xsl:with-param name="map_constr" select='"vl_cpfin_compl"'/>
                  <xsl:with-param name="left" select="$leftcorner"/>
                  <xsl:with-param name="right" select="$rightcorner"/>
                  <xsl:with-param name="rule" select='"vlast_subcl"'/>
               </xsl:call-template>
             </xsl:when>
             <xsl:otherwise>
               <xsl:call-template name="bracket">
                  <xsl:with-param name="map_constr" select='"vl_cpinf_compl"'/>
                  <xsl:with-param name="left" select="$leftcorner"/>
                  <xsl:with-param name="right" select="$rightcorner"/>
                  <xsl:with-param name="rule" select='"vlast_subcl"'/>
               </xsl:call-template>
             </xsl:otherwise>      
            </xsl:choose> 
          </xsl:otherwise>
        </xsl:choose>
        
        <xsl:variable name="rightcorner1">
                <xsl:call-template name="getRightCorner">
                  <xsl:with-param name="nodeset" select=".//W"/>
                </xsl:call-template>
        </xsl:variable>

        <xsl:variable name="leftcorner1">
                <xsl:call-template name="getLeftCorner">
                  <xsl:with-param name="nodeset" select="LK/following-sibling::*//W"/>
                </xsl:call-template>
        </xsl:variable>

        <xsl:call-template name="bracket">
                  <xsl:with-param name="map_constr" select='"vl_compl_vp"'/>
                  <xsl:with-param name="left" select="$leftcorner1"/>
                  <xsl:with-param name="right" select="$rightcorner1"/>
                  <xsl:with-param name="rule" select='"vlast_subcl"'/>
        </xsl:call-template>

     </xsl:if>

  </xsl:template>



<!-- =====================================
     rule "vlast_relwh":                     (was Rule 2)
     matches: CL-REL, CL-WH and no KON-dtr
     brackets: "vl_cpfin_rel" => span of cp
               "vl_cpfin_wh"  => span of cp
               "vl_relwh_vp"  => span of vp
               "vl_rel_vf"    => span of vf
               "vl_wh_vf"     => span of vf
     called templates: "vl_in_coord" 
                        (=> brackets: "vl_vp", "vp_coord",
                                       "vl_cpfin_rel" "vl_cpfin_wh")
     ===================================== -->

  <xsl:template name="vlast_relwh">

     <xsl:if test="count(KON) = 0">
        <xsl:variable name="rule" select="vlast_relwh"/>
        <xsl:variable name="wchildren" select=".//W"/>
        <xsl:variable name="leftcorner">
                <xsl:call-template name="getLeftCorner">
                  <xsl:with-param name="nodeset" select="$wchildren"/>
                </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="rightcorner">
                <xsl:call-template name="getRightCorner">
                  <xsl:with-param name="nodeset" select="$wchildren"/>
                </xsl:call-template>
        </xsl:variable>

        <xsl:choose>   <!-- call rule "vl_in_coord" if embedded in coord-structure -->

          <xsl:when test='(../KON) and (count(preceding::CL)=0) and (../CL[(@fn="VP") and last() ]) '>
             <xsl:call-template name="vl_in_coord">
                 <xsl:with-param name="leftcorner" select="$leftcorner"/>
                 <xsl:with-param name="rightcorner" select="$rightcorner"/>
             </xsl:call-template> 
          </xsl:when>

          <xsl:otherwise>  <!-- original rule continues -->

            <xsl:if test='@fn = "REL"'>
               <xsl:call-template name="bracket">
                  <xsl:with-param name="map_constr" select='"vl_cpfin_rel"'/>
                  <xsl:with-param name="left" select="$leftcorner"/>
                  <xsl:with-param name="right" select="$rightcorner"/>
                  <xsl:with-param name="rule" select='"vlast_relwh"'/>
               </xsl:call-template>
            </xsl:if>
            <xsl:if test='@fn = "WH"'>
               <xsl:call-template name="bracket">
                  <xsl:with-param name="map_constr" select='"vl_cpfin_wh"'/>
                  <xsl:with-param name="left" select="$leftcorner"/>
                  <xsl:with-param name="right" select="$rightcorner"/>
                  <xsl:with-param name="rule" select='"vlast_relwh"'/>
               </xsl:call-template>
            </xsl:if>

            <xsl:variable name="rightcorner1">
                <xsl:call-template name="getRightCorner">
                  <xsl:with-param name="nodeset" select="RK//W"/>
                </xsl:call-template>
            </xsl:variable>

            <xsl:variable name="leftcorner1">
                <xsl:call-template name="getLeftCorner">
                  <xsl:with-param name="nodeset" select="VF/following-sibling::*//W"/>
                </xsl:call-template>
            </xsl:variable>

            <xsl:call-template name="bracket">
                  <xsl:with-param name="map_constr" select='"vl_relwh_vp"'/>
                  <xsl:with-param name="left" select="$leftcorner1"/>
                  <xsl:with-param name="right" select="$rightcorner1"/>
                  <xsl:with-param name="rule" select='"vlast_relwh"'/>
            </xsl:call-template>
          </xsl:otherwise>
        </xsl:choose>

        <xsl:if test='(count(VF) != 0) and (count(RK)!=0)'>
          <xsl:variable name="vfwchildren" select="VF//W"/>
          <xsl:variable name="vfleftcorner">
                <xsl:call-template name="getLeftCorner">
                  <xsl:with-param name="nodeset" select="$vfwchildren"/>
                </xsl:call-template>
          </xsl:variable>
          <xsl:variable name="vfrightcorner">
                <xsl:call-template name="getRightCorner">
                  <xsl:with-param name="nodeset" select="$vfwchildren"/>
                </xsl:call-template>
          </xsl:variable>

          <xsl:if test='VF/@fn = "REL"'>
                 <xsl:call-template name="bracket">
                    <xsl:with-param name="map_constr" select='"vl_rel_vf"'/>
                    <xsl:with-param name="left" select="$vfleftcorner"/>
                    <xsl:with-param name="right" select="$vfrightcorner"/>
                    <xsl:with-param name="rule" select='"vlast_relwh"'/>
                  </xsl:call-template>
          </xsl:if>

          <xsl:if test='VF/@fn = "WH"'>
                 <xsl:call-template name="bracket">
                    <xsl:with-param name="map_constr" select='"vl_wh_vf"'/>
                    <xsl:with-param name="left" select="$vfleftcorner"/>
                    <xsl:with-param name="right" select="$vfrightcorner"/>
                    <xsl:with-param name="rule" select='"vlast_relwh"'/>
                  </xsl:call-template>
          </xsl:if>
        </xsl:if>

     </xsl:if>
  </xsl:template>

<!-- =====================================
     rule "vlast_inf":  infinite CL without LK  (was Rule 6)
     matches: CL-INF without LK
     brackets: "vl_cp_nocompl_inf" => span of cp
               "vl_nocompl_rk_inf+ptk"  => span of rk with PTK
               "vl_nocompl_rk_inf-ptk"  => span of rk without PTK
     ===================================== -->

  <xsl:template name="vlast_inf"> 

        <xsl:variable name="wchildren" select=".//W"/>
        <xsl:variable name="leftcorner">
                <xsl:call-template name="getLeftCorner">
                  <xsl:with-param name="nodeset" select="$wchildren"/>
                </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="rightcorner">
                <xsl:call-template name="getRightCorner">
                  <xsl:with-param name="nodeset" select="$wchildren"/>
                </xsl:call-template>
        </xsl:variable>

        <xsl:if test='RK/@fn = "VINF"'>
               <xsl:call-template name="bracket">
                  <xsl:with-param name="map_constr" select='"vl_cp_nocompl_inf"'/>
                  <xsl:with-param name="left" select="$leftcorner"/>
                  <xsl:with-param name="right" select="$rightcorner"/>
                  <xsl:with-param name="rule" select='"vlast_inf"'/>
               </xsl:call-template>
        </xsl:if>
        <xsl:variable name="rkwchildren" select="RK//W"/>
        <xsl:variable name="rkleftcorner">
                <xsl:call-template name="getLeftCorner">
                  <xsl:with-param name="nodeset" select="$rkwchildren"/>
                </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="rkrightcorner">
                <xsl:call-template name="getRightCorner">
                  <xsl:with-param name="nodeset" select="$rkwchildren"/>
                </xsl:call-template>
        </xsl:variable>

        <xsl:choose>
          <xsl:when test="RK/PTKZU">
               <xsl:call-template name="bracket">
                  <xsl:with-param name="map_constr" select='"vl_nocompl_rk_inf+ptk"'/>
                  <xsl:with-param name="left" select="$rkleftcorner"/>
                  <xsl:with-param name="right" select="$rkrightcorner"/>
                  <xsl:with-param name="rule" select='"vlast_inf"'/>
               </xsl:call-template>
          </xsl:when>
          <xsl:otherwise>
               <xsl:call-template name="bracket">
                  <xsl:with-param name="map_constr" select='"vl_nocompl_rk_inf-ptk"'/>
                  <xsl:with-param name="left" select="$rkleftcorner"/>
                  <xsl:with-param name="right" select="$rkrightcorner"/>
                  <xsl:with-param name="rule" select='"vlast_inf"'/>
               </xsl:call-template>
          </xsl:otherwise> 
        </xsl:choose>
  </xsl:template>

<!-- =====================================
     rule "vlast_fin_rk":  finite RK  (was Rule 45)
     matches: CL with finite RK
     brackets: "vl_rk_fin-complex" => span of unary branching rk
               "vl_rk_fin+complex+finlast"  => span of rk with VFIN last
               "vl_rk_fin+complex-finlast"  => span of rk where not VFIN last
     ===================================== -->

  <xsl:template name="vlast_fin_rk">
        <xsl:if test="RK/VVFIN | RK/VMFIN | RK/VAFIN">  <!-- isFinite -->
          <xsl:variable name="rkwchildren" select="RK//W"/>
          <xsl:variable name="rkleftcorner">
                <xsl:call-template name="getLeftCorner">
                  <xsl:with-param name="nodeset" select="$rkwchildren"/>
                </xsl:call-template>
          </xsl:variable>
          <xsl:variable name="rkrightcorner">
                <xsl:call-template name="getRightCorner">
                  <xsl:with-param name="nodeset" select="$rkwchildren"/>
                </xsl:call-template>
          </xsl:variable>
        
          <xsl:choose>
            <xsl:when test="count(RK/*)=1">  <!-- isUnary -->
               <xsl:call-template name="bracket">
                  <xsl:with-param name="map_constr" select='"vl_rk_fin-complex"'/>
                  <xsl:with-param name="left" select="$rkleftcorner"/>
                  <xsl:with-param name="right" select="$rkrightcorner"/>
                  <xsl:with-param name="rule" select='"vlast_fin_rk"'/>
               </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
              <xsl:choose>
               <xsl:when test='RK/@fn = "VFIN"'>
                  <xsl:call-template name="bracket">
                    <xsl:with-param name="map_constr" select='"vl_rk_fin+complex+finlast"'/>
                    <xsl:with-param name="left" select="$rkleftcorner"/>
                    <xsl:with-param name="right" select="$rkrightcorner"/>
                    <xsl:with-param name="rule" select='"vlast_fin_rk"'/>
                  </xsl:call-template>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:call-template name="bracket">
                    <xsl:with-param name="map_constr" select='"vl_rk_fin+complex-finlast"'/>
                    <xsl:with-param name="left" select="$rkleftcorner"/>
                    <xsl:with-param name="right" select="$rkrightcorner"/>
                    <xsl:with-param name="rule" select='"vlast_fin_rk"'/>
                  </xsl:call-template>
               </xsl:otherwise>
             </xsl:choose>
            </xsl:otherwise>
           </xsl:choose> 
          </xsl:if>
  </xsl:template>

<!-- =====================================
     rule "vlast_inf_rk":  infinite RK  (was Rule 3)
     matches: CL with infinite RK
     brackets: "vl_rk_inf-complex" => span of unary branching rk
               "vl_rk_inf+complex" => span of multiple branching rk
     ===================================== -->

  <xsl:template name="vlast_inf_rk">
     <xsl:if test='(count(RK) != 0) and (count(RK/VVFIN | RK/VMFIN | RK/VAFIN)=0)'>
        <xsl:variable name="wchildren" select="RK//W"/>
        <xsl:variable name="leftcorner">
                <xsl:call-template name="getLeftCorner">
                  <xsl:with-param name="nodeset" select="$wchildren"/>
                </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="rightcorner">
                <xsl:call-template name="getRightCorner">
                  <xsl:with-param name="nodeset" select="$wchildren"/>
                </xsl:call-template>
        </xsl:variable>

        <xsl:choose>
          <xsl:when test="count(RK/*)=1">  <!-- isUnary -->
               <xsl:call-template name="bracket">
                  <xsl:with-param name="map_constr" select='"vl_rk_inf-complex"'/>
                  <xsl:with-param name="left" select="$leftcorner"/>
                  <xsl:with-param name="right" select="$rightcorner"/>
                  <xsl:with-param name="rule" select='"vlast_inf_rk"'/>
               </xsl:call-template>
          </xsl:when>
          <xsl:otherwise>
               <xsl:call-template name="bracket">
                  <xsl:with-param name="map_constr" select='"vl_rk_inf+complex"'/>
                  <xsl:with-param name="left" select="$leftcorner"/>
                  <xsl:with-param name="right" select="$rightcorner"/>
                  <xsl:with-param name="rule" select='"vlast_inf_rk"'/>
               </xsl:call-template>
          </xsl:otherwise>
       </xsl:choose> 
     </xsl:if>
  </xsl:template>


<!-- =====================================
     rule "vfronted":                     (was Rule 7)
     matches: CL-V1 or CL-V2, no KON-dtr, LK-VFIN
     brackets: (if not embedded in coord)
               "v1_cp" => span of CL-V1
               "v2_cp" => span of CL-V2
               "vfronted_vfin+rk"    => span of vfin if there is RK-dtr
               "vfronted_vfin+vp+rk" => span of vfin+vp if there is RK-dtr
               "vfronted_vp+rk"      => span of vp if there is RK-dtr
               "vfronted_rk-complex" => span of RK if unary branching
               "vfronted_rk+complex" => span of RK if multiple-branching
               "vfronted_vfin-rk"    => span of vfin if there is no RK
               "vfronted_vfin+vp-rk" => span of vfin+vp if there is no RK-dtr
               "v2_vf"               => span of vf of CL-V2
               "v2_vfin_pvp+rk"      => span of vfin if pvp-Vorfeld and RK 
               "v2_vfin_pvp-rk"      => span of vfin if pvp-Vorfeld and no RK 
               (if embedded in coord)
               "v2_cp"               => span of full coordinated V2 complex
               "v1_cp"               => span of v1-conjuncts
               "cbar_coord"          => span of coordinated v1-conjuncts
     ===================================== -->


  <xsl:template name="vfronted">

       <xsl:variable name="clchildren" select=".//W"/>
       <xsl:variable name="clleftcorner">
              <xsl:call-template name="getLeftCorner">
                <xsl:with-param name="nodeset" select="$clchildren"/>
              </xsl:call-template>
       </xsl:variable>
       <xsl:variable name="clrightcorner">
              <xsl:call-template name="getRightCorner">
                <xsl:with-param name="nodeset" select="$clchildren"/>
              </xsl:call-template>
       </xsl:variable>
      
       <xsl:if test='@fn = "V1"'>
               <xsl:call-template name="bracket">
                  <xsl:with-param name="map_constr" select='"v1_cp"'/>
                  <xsl:with-param name="left" select="$clleftcorner"/>
                  <xsl:with-param name="right" select="$clrightcorner"/>
                  <xsl:with-param name="rule" select='"vfronted"'/>
               </xsl:call-template>
       </xsl:if>
       
       <xsl:if test='@fn = "V2"'>
         <xsl:choose>
              <!-- part of rule 10 for coord inserted here -->
           <xsl:when test='(../KON) and (count(preceding::CL)=0) and (../CL[(@fn="V1") and last() ]) '>
             <xsl:variable name="lkleftcorner">
               <xsl:call-template name="getLeftCorner">
                 <xsl:with-param name="nodeset" select="LK//W"/>
               </xsl:call-template>
             </xsl:variable>
             <xsl:variable name="clarightcorner">
               <xsl:call-template name="getRightCorner">
                 <xsl:with-param name="nodeset" select=".//W"/>
               </xsl:call-template>
             </xsl:variable>
             <xsl:call-template name="bracket">
               <xsl:with-param name="map_constr" select='"v1_cp"'/>
               <xsl:with-param name="left" select="$lkleftcorner"/>
               <xsl:with-param name="right" select="$clarightcorner"/>
               <xsl:with-param name="rule" select='"vfronted"'/>
             </xsl:call-template>

             <xsl:variable name="claleftcorner">
               <xsl:call-template name="getLeftCorner">
                 <xsl:with-param name="nodeset" select=".//W"/>
               </xsl:call-template>
             </xsl:variable>
            <xsl:variable name="clbrightcorner">
               <xsl:call-template name="getRightCorner">
                 <xsl:with-param name="nodeset" select='../CL[(@fn="V1") and last()]//W'/>
<!-- ../CL[@fn="V1"]/following::CL)=0) -->
               </xsl:call-template> 
             </xsl:variable>
             <xsl:call-template name="bracket">
               <xsl:with-param name="map_constr" select='"v2_cp"'/>
               <xsl:with-param name="left" select="$claleftcorner"/>
               <xsl:with-param name="right" select="$clbrightcorner"/>
               <xsl:with-param name="rule" select='"vfronted"'/>
             </xsl:call-template>        
             <xsl:call-template name="bracket">
               <xsl:with-param name="map_constr" select='"cbar_coord"'/>
               <xsl:with-param name="left" select="$lkleftcorner"/>
               <xsl:with-param name="right" select="$clbrightcorner"/>
               <xsl:with-param name="rule" select='"vfronted"'/>
             </xsl:call-template>
           </xsl:when>

           <xsl:otherwise> 
            <!-- original rule continues here -->
            <xsl:choose>
             <xsl:when test='(count(../KON)!=0) and (count(preceding::CL)=0) and (../CL[(@fn="V1") and last() ])'>
              </xsl:when>
              <xsl:otherwise>
               <xsl:call-template name="bracket">
                  <xsl:with-param name="map_constr" select='"v2_cp"'/>
                  <xsl:with-param name="left" select="$clleftcorner"/>
                  <xsl:with-param name="right" select="$clrightcorner"/>
                  <xsl:with-param name="rule" select='"vfronted"'/>
               </xsl:call-template>
              </xsl:otherwise>
            </xsl:choose>
 
          </xsl:otherwise>
          </xsl:choose>
         
       </xsl:if>

       <xsl:variable name="lkchildren" select="LK//W"/>
       <xsl:variable name="lkleftcorner">
              <xsl:call-template name="getLeftCorner">
                <xsl:with-param name="nodeset" select="$lkchildren"/>
              </xsl:call-template>
       </xsl:variable>
       <xsl:variable name="lkrightcorner">
              <xsl:call-template name="getRightCorner">
                <xsl:with-param name="nodeset" select="$lkchildren"/>
              </xsl:call-template>
       </xsl:variable>

       <xsl:variable name="rkchildren" select="RK//W"/>
       <xsl:variable name="rkleftcorner">
             <xsl:call-template name="getLeftCorner">
               <xsl:with-param name="nodeset" select="$rkchildren"/>
             </xsl:call-template>
       </xsl:variable>
       <xsl:variable name="rkrightcorner">
             <xsl:call-template name="getRightCorner">
               <xsl:with-param name="nodeset" select="$rkchildren"/>
             </xsl:call-template>
       </xsl:variable>

       <xsl:choose>
          <xsl:when test="RK">
              <xsl:call-template name="bracket">
                <xsl:with-param name="map_constr" select='"vfronted_vfin+rk"'/>
                <xsl:with-param name="left" select="$lkleftcorner"/>
                <xsl:with-param name="right" select="$lkrightcorner"/>
                <xsl:with-param name="rule" select='"vfronted"'/>
              </xsl:call-template>
              <xsl:call-template name="bracket">
                <xsl:with-param name="map_constr" select='"vfronted_vfin+vp+rk"'/>
                <xsl:with-param name="left" select="$lkleftcorner"/>
                <xsl:with-param name="right" select="$clrightcorner"/>
                <xsl:with-param name="rule" select='"vfronted"'/>
              </xsl:call-template>
              <xsl:variable name="siblingleftcorner">
                <xsl:call-template name="getLeftCorner">
                  <xsl:with-param name="nodeset" select="LK/following-sibling::*//W"/>
                </xsl:call-template>
              </xsl:variable>
              <xsl:call-template name="bracket">
                <xsl:with-param name="map_constr" select='"vfronted_vp+rk"'/>
                <xsl:with-param name="left" select="$siblingleftcorner"/>
                <xsl:with-param name="right" select="$clrightcorner"/>
                <xsl:with-param name="rule" select='"vfronted"'/>
              </xsl:call-template>

              <xsl:choose>
                 <xsl:when test="count(RK/*)=1">    <!-- isUnary -->
                   <xsl:call-template name="bracket">
                      <xsl:with-param name="map_constr" select='"vfronted_rk-complex"'/>
                      <xsl:with-param name="left" select="$rkleftcorner"/>
                      <xsl:with-param name="right" select="$rkrightcorner"/>
                      <xsl:with-param name="rule" select='"vfronted"'/>
                   </xsl:call-template>
                 </xsl:when>
                 <xsl:otherwise>
                   <xsl:call-template name="bracket">
                      <xsl:with-param name="map_constr" select='"vfronted_rk+complex"'/>
                      <xsl:with-param name="left" select="$rkleftcorner"/>
                      <xsl:with-param name="right" select="$rkrightcorner"/>
                      <xsl:with-param name="rule" select='"vfronted"'/>
                   </xsl:call-template>
                 </xsl:otherwise>
               </xsl:choose>
            </xsl:when>
            <xsl:otherwise>  <!-- no RK daughter -->
                   <xsl:call-template name="bracket">
                      <xsl:with-param name="map_constr" select='"vfronted_vfin-rk"'/>
                      <xsl:with-param name="left" select="$lkleftcorner"/>
                      <xsl:with-param name="right" select="$lkrightcorner"/>
                      <xsl:with-param name="rule" select='"vfronted"'/>
                   </xsl:call-template>
                   <xsl:call-template name="bracket">
                      <xsl:with-param name="map_constr" select='"vfronted_vfin+vp-rk"'/>
                      <xsl:with-param name="left" select="$lkleftcorner"/>
                      <xsl:with-param name="right" select="$clrightcorner"/>
                      <xsl:with-param name="rule" select='"vfronted"'/>
                   </xsl:call-template>
            </xsl:otherwise>
         </xsl:choose>

     <xsl:if test='VF[@fn = "TOPIC"]'>
       <xsl:variable name="vfchildren" select="VF//W"/>
       <xsl:variable name="vfleftcorner">
         <xsl:call-template name="getLeftCorner">
           <xsl:with-param name="nodeset" select="$vfchildren"/>
         </xsl:call-template>
       </xsl:variable>
       <xsl:variable name="vfrightcorner">
         <xsl:call-template name="getRightCorner">
           <xsl:with-param name="nodeset" select="$vfchildren"/>
         </xsl:call-template>
       </xsl:variable>
       <xsl:call-template name="bracket">
           <xsl:with-param name="map_constr" select='"v2_vf"'/>
           <xsl:with-param name="left" select="$vfleftcorner"/>
           <xsl:with-param name="right" select="$vfrightcorner"/>
           <xsl:with-param name="rule" select='"vfronted"'/>
       </xsl:call-template>
       <xsl:if test='(VF/CL[@fn = "VP"]/RK[@fn = "VPART"]) or (VF/RK[@fn = "VPART"])'>
           <xsl:choose>
              <xsl:when test="RK">
                <xsl:call-template name="bracket">
                  <xsl:with-param name="map_constr" select='"v2_vfin_pvp+rk"'/>
                  <xsl:with-param name="left" select="$lkleftcorner"/>
                  <xsl:with-param name="right" select="$lkrightcorner"/>
                  <xsl:with-param name="rule" select='"vfronted"'/>
                </xsl:call-template>
                <xsl:call-template name="bracket">
                  <xsl:with-param name="map_constr" select='"v2_rk_pvp"'/>
                  <xsl:with-param name="left" select="$rkleftcorner"/>
                  <xsl:with-param name="right" select="$rkrightcorner"/>
                  <xsl:with-param name="rule" select='"vfronted"'/>
                </xsl:call-template>
               </xsl:when>
               <xsl:otherwise>
                <xsl:call-template name="bracket">
                  <xsl:with-param name="map_constr" select='"v2_vfin_pvp-rk"'/>
                  <xsl:with-param name="left" select="$lkleftcorner"/>
                  <xsl:with-param name="right" select="$lkrightcorner"/>
                  <xsl:with-param name="rule" select='"vfronted"'/>
                </xsl:call-template>                        
               </xsl:otherwise>
            </xsl:choose>
       </xsl:if>
     </xsl:if>
  </xsl:template>

<!-- =====================================
     rule "extrapos": NF following RK
     matches: CL with RK and NF dtrs
     brackets: "extrapos_rk+nf"  => span of RK + NF
     ===================================== -->

  <xsl:template name="extrapos">
       <xsl:variable name="leftcorner">
          <xsl:call-template name="getLeftCorner">
             <xsl:with-param name="nodeset" select="RK//W"/>
          </xsl:call-template>
       </xsl:variable>
       <xsl:variable name="rightcorner">
          <xsl:call-template name="getRightCorner">
             <xsl:with-param name="nodeset" select="NF//W"/>
          </xsl:call-template>
       </xsl:variable>
       <xsl:call-template name="bracket">
          <xsl:with-param name="map_constr" select='"extrapos_rk+nf"'/>
          <xsl:with-param name="left" select="$leftcorner"/>
          <xsl:with-param name="right" select="$rightcorner"/>
          <xsl:with-param name="rule" select='"extrapos"'/>
       </xsl:call-template>
  </xsl:template>


<!-- =====================================
     template for CL-SUBCL/CL-REL/CL-WH within coordination
     matches: leftmost CL node with embedding CL, a KON-sibling, and a rightmost CL-VP sibling
     brackets: "vl_vp"
               "vp_coord",
               "vl_cpfin_compl" 
               "vl_cpinf_compl"
               "vl_cpfin_rel"
               "vl_cpfin_wh"
     ===================================== -->


  <xsl:template name="vl_in_coord">
    <xsl:param name="leftcorner"/>
    <xsl:param name="rightcorner"/>
    <xsl:variable name="lconj1">
     <xsl:choose>
      <xsl:when test='@fn = "SUBCL"'>
               <xsl:call-template name="getLeftCorner">
                 <xsl:with-param name="nodeset" select="LK/following-sibling::*//W"/>
               </xsl:call-template>
      </xsl:when>
      <xsl:when test='(@fn = "WH" ) or (@fn = "REL")'>
               <xsl:call-template name="getLeftCorner">
                 <xsl:with-param name="nodeset" select="VF/following-sibling::*//W"/>
               </xsl:call-template>
      </xsl:when>
     </xsl:choose>
    </xsl:variable>
             <xsl:variable name="lconj2">
               <xsl:call-template name="getLeftCorner">
                 <xsl:with-param name="nodeset" select='../CL[(@fn="VP") and last()]//W'/>
               </xsl:call-template> 
             </xsl:variable>
             <xsl:variable name="rconj2">
               <xsl:call-template name="getRightCorner">
                 <xsl:with-param name="nodeset" select='../CL[(@fn="VP") and last()]//W'/>
               </xsl:call-template>
             </xsl:variable>
             <xsl:call-template name="bracket">
               <xsl:with-param name="map_constr" select='"vl_vp"'/>
               <xsl:with-param name="left" select="$lconj1"/>
               <xsl:with-param name="right" select="$rightcorner"/>
               <xsl:with-param name="rule" select='"vl_in_coord"'/>
             </xsl:call-template>
             <xsl:call-template name="bracket">
               <xsl:with-param name="map_constr" select='"vl_vp"'/>
               <xsl:with-param name="left" select="$lconj2"/>
               <xsl:with-param name="right" select="$rconj2"/>
               <xsl:with-param name="rule" select='"vl_in_coord"'/>
             </xsl:call-template>
             <xsl:call-template name="bracket">
               <xsl:with-param name="map_constr" select='"vp_coord"'/>
               <xsl:with-param name="left" select="$lconj1"/>
               <xsl:with-param name="right" select="$rconj2"/>
               <xsl:with-param name="rule" select='"vl_in_coord"'/>
             </xsl:call-template>

             <xsl:if test='@fn = "SUBCL"'>
              <xsl:choose>
               <xsl:when test="RK/VVFIN | RK/VMFIN | RK/VAFIN">  <!-- isFinite -->
                <xsl:call-template name="bracket">
                  <xsl:with-param name="map_constr" select='"vl_cpfin_compl"'/>
                  <xsl:with-param name="left" select="$leftcorner"/>
                  <xsl:with-param name="right" select="$rconj2"/>
                  <xsl:with-param name="rule" select='"vl_in_coord"'/>
                </xsl:call-template>
               </xsl:when>
               <xsl:otherwise>
                <xsl:call-template name="bracket">
                  <xsl:with-param name="map_constr" select='"vl_cpinf_compl"'/>
                  <xsl:with-param name="left" select="$leftcorner"/>
                  <xsl:with-param name="right" select="$rconj2"/>
                  <xsl:with-param name="rule" select='"vl_in_coord"'/>
                </xsl:call-template>
               </xsl:otherwise>
              </xsl:choose> 
             </xsl:if>

             <xsl:if test='@fn = "REL"'>
               <xsl:call-template name="bracket">
                  <xsl:with-param name="map_constr" select='"vl_cpfin_rel"'/>
                  <xsl:with-param name="left" select="$leftcorner"/>
                  <xsl:with-param name="right" select="$rconj2"/>
                  <xsl:with-param name="rule" select='"vl_in_coord"'/>
               </xsl:call-template>
             </xsl:if>
             <xsl:if test='@fn = "WH"'>
               <xsl:call-template name="bracket">
                  <xsl:with-param name="map_constr" select='"vl_cpfin_wh"'/>
                  <xsl:with-param name="left" select="$leftcorner"/>
                  <xsl:with-param name="right" select="$rconj2"/>
                  <xsl:with-param name="rule" select='"vl_in_coord"'/>
             </xsl:call-template>
            </xsl:if>

  </xsl:template>


<!-- =====================================
     rule : cp_coord  (was Rule 13)
     template for symmetric cp-coordination 
     need to set apart asymmetric cases where last=CL-VP or (first CL-V2 & last CL-V1))
     matches: leftmost CL node with embedding CL, a KON-sibling, and a rightmost CL sibling
     brackets: "cp_coord"
     ===================================== -->

  <xsl:template name="cp_coord">
     <xsl:if test='  (@fn="V2" and count(../CL[@fn="V1" and last()])=0)
                 and (count(../CL[@fn="VP" and last()])=0)' > 
             <xsl:variable name="lconj1">
             <xsl:call-template name="getLeftCorner">
                <xsl:with-param name="nodeset" select='.//W'/>
             </xsl:call-template> 
             </xsl:variable>
             <xsl:variable name="rconj2">
               <xsl:call-template name="getRightCorner">
                 <xsl:with-param name="nodeset" select='../CL[last()]//W'/>
               </xsl:call-template>
             </xsl:variable>
             <xsl:call-template name="bracket">
               <xsl:with-param name="map_constr" select='"cp_coord"'/>
               <xsl:with-param name="left" select="$lconj1"/>
               <xsl:with-param name="right" select="$rconj2"/>
               <xsl:with-param name="rule" select='"cp_coord"'/>
             </xsl:call-template>
     </xsl:if>
  </xsl:template>


<!-- ===  New Rules for Chunk Brackets  inserted here -->


<!-- ===================================================
     Toplevel rule with construction-wise template calls
     Matches: CHUNK constituents
     =================================================== -->
 
  <xsl:template match="CHUNK">

       <xsl:if test='(@cat = "NP") or (@cat = "PP")'>
       <xsl:call-template name="chunk"/>
       </xsl:if>

     <xsl:apply-templates/>
  </xsl:template>


<!-- =====================================
     Extract left corner of NP, PP, .. chunks
     ===================================== -->

  <xsl:template name="chunk">
        <xsl:variable name="wchildren" select=".//W"/>
        <xsl:variable name="leftcorner">
                <xsl:call-template name="getLeftCorner">
                  <xsl:with-param name="nodeset" select="$wchildren"/>
                </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="rightcorner">
                <xsl:call-template name="getRightCorner">
                  <xsl:with-param name="nodeset" select="$wchildren"/>
                </xsl:call-template>
        </xsl:variable>

        <xsl:if test='@cat = "NP"'>
        <xsl:call-template name="bracket">
             <xsl:with-param name="map_constr" select='"np_chunk_leftcorner"'/>
             <xsl:with-param name="left" select="$leftcorner"/>
             <xsl:with-param name="right" select="$rightcorner"/>
             <xsl:with-param name="rule" select='"chunk"'/>
        </xsl:call-template>
        </xsl:if>

        <xsl:if test='@cat = "PP"'>
        <xsl:call-template name="bracket">
             <xsl:with-param name="map_constr" select='"pp_chunk_leftcorner"'/>
             <xsl:with-param name="left" select="$leftcorner"/>
             <xsl:with-param name="right" select="$rightcorner"/>
             <xsl:with-param name="rule" select='"chunk"'/>
        </xsl:call-template>
        </xsl:if>
  </xsl:template>



  



<!-- =======  templates to access left and right edge indices ==== -->


  <xsl:template name="getLeftCorner">
       <xsl:param name="nodeset"/>
       <xsl:value-of select="$nodeset[position() = 1]/@id"/>
  </xsl:template>
 
  <xsl:template name="getRightCorner">
       <xsl:param name="nodeset"/>
       <xsl:value-of select="$nodeset[last()]/@id"/>
  </xsl:template>

<!--  <xsl:template match="LK|RK">
<xsl:call-template name="isFinite">
<xsl:with-param name="node" select="."/>
       </xsl:call-template>
<xsl:apply-templates/>
  </xsl:template>
-->
  
  <xsl:template name="isFinite">
       <xsl:param name="node"/>
       <xsl:value-of select="$node/VVFIN | $node/VMFIN | $node/VAFIN"/>
  </xsl:template>

  <xsl:template name="bracket">
       <xsl:param name="map_constr"/>
       <xsl:param name="prob"/>
       <xsl:param name="left"/>
       <xsl:param name="right"/>
       <xsl:param name="rule"/>
       <xsl:element name="MAP_CONSTR">
         <xsl:attribute name="id"><xsl:value-of select="generate-id()"/></xsl:attribute>
         <xsl:attribute name="map_constr"><xsl:value-of select="$map_constr"/></xsl:attribute>
         <xsl:if test="$prob">
             <xsl:attribute name="prob"><xsl:value-of select="$prob"/></xsl:attribute>
         </xsl:if>
         <xsl:attribute name="rule"><xsl:value-of select="$rule"/></xsl:attribute>

         <xsl:attribute name="left"><xsl:value-of select="$left"/></xsl:attribute>
         <xsl:attribute name="right"><xsl:value-of select="$right"/></xsl:attribute>
       </xsl:element>
  </xsl:template>


  <xsl:template match="text()">
  </xsl:template>


</xsl:stylesheet>
