<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">


<!-- This xslt stylesheet extracts brackets from topo-trees to be 
     delivered to HPSG parser.
     To be processed in a cascade of xslt-transformations

   (C) 2002  DFKI Language Technology Lab http://www.dfki.de/lt
   Author: Anette Frank, anette.frank@dfki.de

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
-->

<!-- edited 2007-06-09
     by Daniel.Contag@dfki.de
     Changes:
      - <xsl:template match="CHUNK"> to lowercase chunk
      - replaced "PoS" with "pos"
-->

  <xsl:output method="xml" indent="yes"/>
  <xsl:strip-space elements="*"/>

  <xsl:template match="/">
    <xsl:element name="TOPOMAP">
      <xsl:copy-of select="*/@*"/>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>


<!-- ===  New Rules for Chunk Brackets  inserted here -->


<!-- ===================================================
     Toplevel rule with construction-wise template calls
     Matches: CHUNK constituents
     =================================================== -->



  <xsl:template match="chunk">

       <xsl:if test='(@cat = "NP") and 
                     ( *[1]/@pos="ART"
                   or  *[1]/@pos="PposAT"
                   or  *[1]/@pos="PIAT"
                   or  *[1]/@pos="PDAT") '>
       <xsl:call-template name="chunk">
           <xsl:with-param name="bracketname" select='"chunk_np+det"'/>
       </xsl:call-template>
       </xsl:if>

       <xsl:if test='(@cat = "NP") and 
                     ( *[1]/@pos="PWAT" 
                   or  *[1]/@pos="PRELAT") '>
       <xsl:call-template name="chunk">
           <xsl:with-param name="bracketname" select='"chunk_np+relwh_det"'/>
       </xsl:call-template>



       </xsl:if>


       <xsl:if test='(@cat = "NP") and 
                     ( *[1]/@pos!="ART"
                   and *[1]/@pos!="PposAT"
                   and *[1]/@pos!="PIAT"
                   and *[1]/@pos!="PDAT"
                   and *[1]/@pos!="PWAT"
                   and *[1]/@pos!="PPELAT"
                   and *[1]/@pos!="PPER"
                   and *[1]/@pos!="PposS"
                   and *[1]/@pos!="PDS"
                   and *[1]/@pos!="PIS"
                   and *[1]/@pos!="PRELS"
                   and *[1]/@pos!="PWS"  ) '>
       <xsl:call-template name="chunk">
           <xsl:with-param name="bracketname" select='"chunk_np-det"'/>
       </xsl:call-template>
       </xsl:if>

       <xsl:if test='(@cat = "NP") and 
                     ( *[1]/@pos="PPER"
                    or *[1]/@pos="PposS"
                    or *[1]/@pos="PDS"
                    or *[1]/@pos="PIS" ) '>
       <xsl:call-template name="chunk">
           <xsl:with-param name="bracketname" select='"chunk_np+pron"'/>
       </xsl:call-template>
       </xsl:if>

       <xsl:if test='(@cat = "NP") and 
                     ( *[1]/@pos="PRELS"
                    or *[1]/@pos="PWS" )'>
       <xsl:call-template name="chunk">
           <xsl:with-param name="bracketname" select='"chunk_np+relwh_pron"'/>
       </xsl:call-template>
       </xsl:if>

       <xsl:if test='@cat = "PP"'>
       <xsl:call-template name="chunk">
           <xsl:with-param name="bracketname" select='"chunk_pp"'/>
       </xsl:call-template>
       </xsl:if>

     <xsl:apply-templates/>
  </xsl:template>


<!-- =====================================
     Extract left corner of NP, PP, .. chunks
     ===================================== -->

  <xsl:template name="chunk">
        <xsl:param name="bracketname"/>
        <xsl:variable name="wchildren" select=".//W"/>
        <xsl:variable name="leftcorner">
                <xsl:call-template name="getLeftCorner">
                  <xsl:with-param name="nodeset" select="$wchildren"/>
                </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="rightcorner">
                <xsl:call-template name="getRightCorner">
                  <xsl:with-param name="nodeset" select="$wchildren"/>
                </xsl:call-template>
        </xsl:variable>

        <xsl:call-template name="bracket">
             <xsl:with-param name="map_constr" select="$bracketname"/>
             <xsl:with-param name="left" select="$leftcorner"/>
             <xsl:with-param name="right" select="$rightcorner"/>
             <xsl:with-param name="rule" select='"chunk"'/>
        </xsl:call-template>

  </xsl:template>



  



<!-- =======  templates to access left and right edge indices ==== -->


  <xsl:template name="getLeftCorner">
       <xsl:param name="nodeset"/>
       <xsl:value-of select="$nodeset[position() = 1]/@id"/>
  </xsl:template>
 
  <xsl:template name="getRightCorner">
       <xsl:param name="nodeset"/>
       <xsl:value-of select="$nodeset[last()]/@id"/>
  </xsl:template>


  <xsl:template name="bracket">
       <xsl:param name="map_constr"/>
       <xsl:param name="prob"/>
       <xsl:param name="left"/>
       <xsl:param name="right"/>
       <xsl:param name="rule"/>
       <xsl:element name="MAP_CONSTR">
         <xsl:attribute name="id"><xsl:value-of select="generate-id()"/></xsl:attribute>
         <xsl:attribute name="map_constr"><xsl:value-of select="$map_constr"/></xsl:attribute>
         <xsl:if test="$prob">
             <xsl:attribute name="prob"><xsl:value-of select="$prob"/></xsl:attribute>
         </xsl:if>
         <xsl:attribute name="rule"><xsl:value-of select="$rule"/></xsl:attribute>

         <xsl:attribute name="left"><xsl:value-of select="$left"/></xsl:attribute>
         <xsl:attribute name="right"><xsl:value-of select="$right"/></xsl:attribute>
       </xsl:element>
  </xsl:template>


  <xsl:template match="text()">
  </xsl:template>


</xsl:stylesheet>
